
import java.io.*;
import java.util.*;

/** W3C's DOM API */
import org.w3c.dom.*;
import org.w3c.dom.Document;

/** DOM Core Level 1 implemenetation */
import edu.npac.iwt.dom.*;
import edu.npac.iwt.domUtil.*;

/**
* Encapsulates the each resource in directory.
*
* @author H. Timucin Ozdemir
* @date   March 9, 1999 (Updated on March 9, 1999)
*/
public class ResourceRecord
{
  Vector resources = new Vector();
  String name=null;
  String type=null;
  String date=null;
  String length=null;
 
  /**
  * @param ele an Element object to extract the resource element.
  */
  public ResourceRecord(Element ele)
  {
    extractResource(ele);

  }// end of CONSTRUCTOR

  /**
  * It serializes the object in HTML format.
  * @param baseURL A base url of this resource. 
  * @return Returns the serialized version in HTML.
  */
  public String toString(String baseURL)
  {
     StringWriter out = new StringWriter();

     out.write("<tr>");

     if(baseURL!=null) 
           out.write("<td><a href=\""+baseURL+name+"\">"+name+"</a></td>");
     else
           out.write("<td><a href=\""+name+"\">"+name+"</a></td>");

     out.write("<td>"+(type!=null?type:"")+"</td>");
     out.write("<td>"+(length!=null?length:"")+"</td>");
     out.write("<td>"+(date!=null?date:"")+"</td>");

     out.write("</tr>");

     return out.toString();

  }// end of toString()

  /**
  * Extracts the resource from Element object.
  *
  * @param ele an Element object.
  */
  void extractResource(Element ele)
  {
 
     if( ele.getTagName().equals("resource") )
     {

       NodeList bList = ele.getElementsByTagName("name");
       if( bList!=null && bList.getLength()>0 ) 
           extractName((Element)bList.item(0));

       bList = ele.getElementsByTagName("type");
       if( bList!=null && bList.getLength()>0 ) 
           extractType((Element)bList.item(0));

       bList = ele.getElementsByTagName("date");
       if( bList!=null && bList.getLength()>0 ) 
           extractDate((Element)bList.item(0));

       if(!isDirectory()) 
       {
          bList = ele.getElementsByTagName("length");
          if( bList!=null && bList.getLength()>0 ) 
              extractLength((Element)bList.item(0));

       }// end of if

     }// end of if

  }// end of extractResource()

  /**
  * Extracts the <i>type</i> element.
  * @param ele an Element object.
  */
  void extractType(Element ele)
  {
 
     if( ele.getTagName().equals("type") )
     {
       Node aNode = ele.getFirstChild();
       if( aNode.getNodeType() == Node.TEXT_NODE ) 
           type = ((Text)aNode).getData();
     }

  }// end of extractType()

  /**
  * Extracts the <i>name</i> element.
  * @param ele an Element object.
  */
  void extractName(Element ele)
  {
 
     if( ele.getTagName().equals("name") )
     {
       Node aNode = ele.getFirstChild();
       if( aNode.getNodeType() == Node.TEXT_NODE ) 
           name = ((Text)aNode).getData();
     }

  }// end of extractName()

  /**
  * Extracts the <i>date</i> element.
  * @param ele an Element object.
  */
  void extractDate(Element ele)
  {
 
     if( ele.getTagName().equals("date") )
     {
       Node aNode = ele.getFirstChild();
       if( aNode.getNodeType() == Node.TEXT_NODE ) 
           date = ((Text)aNode).getData();
     }

  }// end of extractDate()

  /**
  * Extracts the <i>length</i> element.
  * @param ele an Element object.
  */
  void extractLength(Element ele)
  {
 
     if( ele.getTagName().equals("length") )
     {
       Node aNode = ele.getFirstChild();
       if( aNode.getNodeType() == Node.TEXT_NODE ) 
           length = ((Text)aNode).getData();
     }

  }// end of extractLength()

  /**
  * Is this resource is directory?
  * @return true if it is a directory.
  */
  boolean isDirectory()
  {
     return ( type!=null && ( type.equals("DIR")||type.equals("dir") ) );

  }// end of isDirectory()

}// end of ResourceRecord

