
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

/** W3C's DOM API */
import org.w3c.dom.*;
import org.w3c.dom.Document;

/** DOM Core Level 1 implemenetation */
import edu.npac.iwt.dom.*;
import edu.npac.iwt.domUtil.*;

/**
*
* This is TinyBrowser/0.2, it handles xsl-stylesheet element and applies
* formatting on the client side. TinyServer do not process xml documents
* anymore, it assumes that TinyBrowser/0.2 is capable of processing these
* files.
*
* @date March 8, 1999 (Updated March 8, 1999) 
* @author H. Timucin Ozdemir
*
*/
public class 
TinyBrowser extends JFrame implements ActionListener, HyperlinkListener
{

  String OpenURL = "Open URL";
  String ExitTxt = "Exit";
  String docURL  = "";

  JMenuItem exitItem;
  JMenuItem openItem;

  JTextArea   output;
  JScrollPane scrollPane;
  JScrollPane htmlView;
  JEditorPane htmlPane;
  JSplitPane  splitPane ;
  JLabel      statusBar;

  Properties  props;

  /**
  * @param _props Receives the browser settings about XML/DOM/XSL 
  *               implementations.
  */
  public TinyBrowser(Properties _props) 
  {
     props = _props; 

     JMenuBar menuBar;
     JMenu menu, submenu;
     JMenuItem menuItem;
     JCheckBoxMenuItem cbMenuItem;
     JRadioButtonMenuItem rbMenuItem;

     addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                //System.exit(0);
                TinyBrowser.this.doExit();
            }
     });

     //Add regular components to the window, using the default BorderLayout.
     Container contentPane = getContentPane();
     output = new JTextArea(5, 30);
     output.setEditable(false);
     scrollPane = new JScrollPane(output);
     contentPane.add(scrollPane, BorderLayout.CENTER);

     //Create the menu bar.
     menuBar = new JMenuBar();
     setJMenuBar(menuBar);

     //Build the first menu.
     menu = new JMenu("Menu");
     menuBar.add(menu);

     openItem = new JMenuItem(OpenURL);
     exitItem = new JMenuItem(ExitTxt);
     menu.add(openItem);
     menu.addSeparator();
     menu.add(exitItem);
     openItem.addActionListener(this);
     exitItem.addActionListener(this);

     htmlPane = new JEditorPane();
     htmlPane.setEditable(false);
     htmlView = new JScrollPane(htmlPane);
     htmlPane.addHyperlinkListener( this );

     //Add the scroll panes to a split pane.
     splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
     splitPane.setBottomComponent(htmlView);

     Dimension minimumSize = new Dimension(100, 50);
     htmlView.setMinimumSize(minimumSize);
     splitPane.setDividerLocation(100);

     splitPane.setPreferredSize(new Dimension(500, 300));

     //Add the split pane to this frame
     getContentPane().add(splitPane,BorderLayout.CENTER);
     statusBar = new JLabel(" ");
     statusBar.setForeground(Color.red);
     getContentPane().add(statusBar,BorderLayout.SOUTH);

  }// end of CONSTUCTOR

  /**
  * ActionListener interface to collect the user events.
  * @param e ActionEvent
  */
  public void 
  actionPerformed(ActionEvent e) 
  {
     JMenuItem source = (JMenuItem)(e.getSource());

     if( source.getText().equals(ExitTxt)) doExit();
     else if( source.getText().equals(OpenURL) ) openURLProcess();

  }// end of actionPerformed()

  /**
  * Move the frame back, and destroy the execution.
  */
  void doExit()
  {
     toBack();
     System.exit(0);

  }// end of doExit()

  /**
  * Asks the user URL address which is gonna be displayed.
  */
  void openURLProcess()
  {
     GetURL dialog = new GetURL(this,docURL);
     dialog.pack();
     dialog.setLocationRelativeTo(this);
     dialog.show();

     if( dialog.getPageURL().trim().length()>0) 
          displayURL(dialog.getPageURL().trim());
     else giveError(new String("Empty URL"));

  }// end of openURLProcess()

  /**
  * Displays the error message in the warning window.
  * @param mess Error message.
  */
  void giveError(String mess)
  {
     JOptionPane.showMessageDialog( this, "Sorry!"+"\n"+mess,"",
                                    JOptionPane.ERROR_MESSAGE);

  }// end of giveError()

  /**
  * Displays the Web Page.
  * @param pageURL The URL address of the Web Page.
  */
  void displayURL(String pageURL)
  {

     try
     {
       // Get the URL of the Web Page.
       URL url = new URL(pageURL);

       // Display
       displayURL(url);

       // Update the current document.
       docURL = pageURL;

     } catch( MalformedURLException mue) 
       { giveError(new String("There is no Such Page")); }
 
  }// end of displayURL()

  /**
  *
  * Displays the Web Page pointed by URL object.
  *
  * @param url URL address of the Web Page.
  */
  void displayURL(URL url)
  {
     try
     {
       // We are gonna use htmlView
       splitPane.setBottomComponent(htmlView);

       // Open a URL Connection to the Page to able say that 
       // this is a TinyBrowser/0.2
       URLConnection urlConnection = url.openConnection();

       // Update the User-Agent HTTP header.
       urlConnection.setRequestProperty("User-Agent","TinyBrowser/0.2");

       // Open the input stream.
       BufferedReader in = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));
 
       // Get the content type of this resource.
       String ctype = urlConnection.getContentType();

       HTMLDocument aHtml = new HTMLDocument();

       if( ctype.equals("text/xml") )
       {
           iwt.web.xmldomxsl.XMLDOMXSL web = null;

           String xmldomxsl = props.getProperty("org.w3c.xmldomxsl");

           if( xmldomxsl!=null )
           {
              try
              {
                Class xmldomxslClass   = Class.forName(xmldomxsl);;
                Object xmldomxslIns  = xmldomxslClass.newInstance();

                if ( xmldomxslIns instanceof iwt.web.xmldomxsl.XMLDOMXSL ) 
                {  
                   web = (iwt.web.xmldomxsl.XMLDOMXSL)xmldomxslIns;
                   web.setAll(props);

                }// end of if

              } catch(ClassNotFoundException e1) 
                { giveError(new String("XMLDOMXSL class:"+xmldomxsl+" not found! ")); }
                catch(Exception e) 
                { giveError(new String("XMLDOMXSL object instantiation error!")); }

           }// end of if

           if(web!=null)
           {
              htmlPane.setContentType("text/html");
              htmlPane.setDocument(aHtml);
              aHtml.setBase(urlConnection.getURL());

              Document doc = web.openDocument(urlConnection.getInputStream(),url.toString());

              NodeList aList = doc.getChildNodes();

              String styleSheet = null;
              Node a = null;
              boolean stay=true;
              boolean styleSheetFound = false;
              
              // Look for the "axml-stylesheet" PI
              if( aList!=null )
               for(int i=0;!styleSheetFound&&i<aList.getLength();i++)
               {
                a = aList.item(i);
                if( a!=null && a.getNodeName().equals("axml-stylesheet") && 
                    a.getNodeType()==org.w3c.dom.Node.PROCESSING_INSTRUCTION_NODE) 
                {
                   String adat = ((org.w3c.dom.ProcessingInstruction)a).getData();
                   if( adat.indexOf("href")>=0 )
                   {
                     String ahref = adat.substring(adat.indexOf("href"));

                     try
                     {  
                        int    f1 = ahref.indexOf('"');
                        int    f2 = ahref.indexOf('"',f1+1);
                        styleSheet = ahref.substring(f1+1,f2);

                        if( styleSheet!=null && styleSheet.trim().length()>0)
                            styleSheetFound = true;

                     } catch(Exception e) {}

                   }// end of if

                }// end of if
 
               }// end of for

              // If axml-stylesheet does not found, then
              // xml-stylesheet will do.
              if( !styleSheetFound )
              {
                 org.w3c.dom.Element docRoot = doc.getDocumentElement();

                 aList = docRoot.getElementsByTagName("xml-stylesheet");  

                 a = null;
                 if(aList!=null);
                  for(int i=0;!styleSheetFound&&i<aList.getLength();i++)
                  {
                    a = aList.item(i);
                    if( a!=null && 
                        a.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE) 
                    {
                       styleSheet = ((org.w3c.dom.Element)a).getAttribute("href");
                       if( styleSheet!=null && styleSheet.trim().length()>0)
                             styleSheetFound=true;

                    }// end of if
 
                  }// end of for

              }// end of if

              // If we have a style-sheet information, then we can go ahead. 
              //if( styleSheet!=null )
              if( styleSheetFound )
              {
                 // If the style-sheet address is not URL, then we use
                 // current document's base address. 
                 if(!styleSheet.startsWith("http://"))
                 {
                   String base = url.toString();
                   styleSheet = base.substring(0,base.lastIndexOf('/'))+"/"+styleSheet; 
          
                 }// end of if 

                 try
                 {
                   // Let's try to open the style-sheet document
                   URL styleURL = new URL(styleSheet);
                
                   // Get the document
                   //Document styleDoc = web.openDocument(styleURL);//.openConnection().getInputStream(),styleSheet);
                   Document styleDoc = web.openDocument(styleURL);

                   // Produce the XSL rule output
                   ByteArrayOutputStream bout = new ByteArrayOutputStream();
                   PrintStream pout = new PrintStream(bout);
                   web.applyXSLRules(doc,styleDoc,bout); 

                   // set the text in the htmlPane
                   htmlPane.setText(bout.toString());
   
                 } catch(MalformedURLException e1) 
                   {  System.out.println("StyleSheet is not available:"+styleSheet); 
                      displayPlainText(url);
                   } 
                   catch(Exception e) 
                   { 
                      displayPlainText(url);
                   }
             
              } else
                 {  
                    System.out.println("No StyleSheet. Displaying in ASCII!");
                    /// text/plain : Since it does not have stylesheet
                    /// then display it as an ASCII file.
                    displayPlainText(url);

                 }

           }else
              {  /// text/plain : Display it as an ASCII file
                 /// since there is no XMLDOMXSL Handler.
                 displayPlainText(urlConnection.getInputStream());
              }
           statusBar.setText(url.toString());
       } else
          { 
            if( !ctype.equals("text/html") )
                 displayPlainText(urlConnection.getInputStream());
            else htmlPane.setPage(url);
            statusBar.setText(url.toString());
          }

     } catch( IOException ioe) 
       { giveError(new String("There is no Such Page")); }
 
  }// end of displayURL()

  /**
  *
  * Displays the inputstream in text/plain mode.
  *
  * @param In InputStream to the information that it is supposed to be 
  *           rendered.
  */
  void displayPlainText(InputStream In)
  {

     try
     {
       BufferedReader in = new BufferedReader(new InputStreamReader(In));
       String inputLine;
       JTextArea out = new JTextArea();
       out.setEditable(false);

       while ((inputLine = in.readLine()) != null) out.append(inputLine+"\n");
       in.close();

       splitPane.setBottomComponent(new JScrollPane(out));

     } catch(IOException ioe) 
       { giveError(new String("IOError")); }

  }// en of displayPlainText()

  /**
  *
  * Displays the page given with URL in text/plain mode.
  *
  * @param url URL of the web Resource, It connects with TinyBrowser/0.2
  *            User-Agent value.
  */
  void displayPlainText(URL url)
  {

     try
     {
       URLConnection urlConnection = url.openConnection();

       // Update the User-Agent HTTP header.
       urlConnection.setRequestProperty("User-Agent","TinyBrowser/0.2");

       /// text/plain : display it as an ASCII file.
       displayPlainText(urlConnection.getInputStream());

     } catch( IOException ioe) 
       { giveError(new String("There is no Such Page")); }

  }// en of displayPlainText()

  /**
  *
  * HyperlinkListener interface to receive the click event
  * on the hyperlink.
  *
  * @param e HyperlinkEvent
  */
  public void 
  hyperlinkUpdate(HyperlinkEvent e) 
  {
     if ( e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
     { 
        // Display the selected URL.
        displayURL(e.getURL());
     }// end of if

  }// end of hyperlinkUpdate()

  /**
  * 
  */
  public static void 
  main(String[] args) 
  {
     int i=0; 
     Properties props = new Properties();
     String propName = "browser.properties";

     try
     {
       while(i<args.length)
       {
        if( args[i].equals("-prop") && i<args.length-1) propName=args[++i];
        else i++;

       }// end of while

       props.load(new FileInputStream(propName));
       props.list(System.out);

     } catch(Exception e) {}

     TinyBrowser window = new TinyBrowser(props);

     window.setTitle("TinyBrowser/0.2");
     window.setSize(450, 260);
     window.setVisible(true);


  }// end of main

}// end of TinyBrowser

