import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.net.*;

/** W3C DOM Core Level 1 Interfaces */
import org.w3c.dom.*;

/** RDF Interfaces */
import iwt.rdf.*;

/** DOM Core Level 1 implemenetation */
import edu.npac.iwt.dom.*;

/** RDF implemenetation */
import edu.npac.iwt.rdf.*;

/**
*
* Describes the RDF operations.
*
* @author Hasan Timucin Ozdemir
* @date   February 22, 1999
*/
public class 
StudentTable extends AbstractTableModel 
{
    JDialog dialog = null;
    
    boolean DEBUG=true;

    final String[] columnNames = {"Student ID", "Name", "Surname", "vCard" };
   
    Hashtable objs = new Hashtable();
    Vector    keys = new Vector();

    Document myDoc;
    JFrame   myFrame;

    /**
    * @param _myDoc a Document object to extract the StudentRecord objects.
    * @param _myFrame 
    */
    public StudentTable( Document _myDoc, JFrame _myFrame )
    {
  
      myDoc = _myDoc;
      myFrame = _myFrame;
      extractRecords(myDoc);
       
    }// end of constructor

    /** returns the number of columns */
    public int getColumnCount() { return columnNames.length; }

    /** returns the number of rows */
    public int getRowCount() { return keys.size(); } 

    /** returns the name of a particular column */
    public String 
    getColumnName(int col) { return (col>=0&&col<4?columnNames[col]:""); }

    /** returns the value of the particular cell in the table */
    public Object 
    getValueAt(int row, int col) 
    { 
       Object ret = null;

       String aKey = (String) keys.elementAt(row);

       if( aKey!=null && col>=0 && col<5 )
       {
          StudentRecord aRec = (StudentRecord) objs.get(aKey);

          if( aRec!=null )
          {
            switch(col)
            {  case 0: ret = aRec.getID(); break;
               case 1: ret = aRec.getName(); break;
               case 2: ret = aRec.getSurname(); break;
               case 3: ret = new Boolean(false); break;
            }// end of switch

          }// end of if

       }// end of if

       return ret;

    }// end of getValueAt()

    /*
    * JTable uses this method to determine the default renderer/
    * editor for each cell.  
    */
    public Class 
    getColumnClass(int c) 
    { String st = new String();
      Boolean bl = new Boolean(false);
      return (c<3?st.getClass():bl.getClass());
    }// end of getColumnClass()

    /*
    * Don't need to implement this method unless your table's editable.
    */
    public boolean isCellEditable(int row, int col) { return (col==3); }

    /*
    * Don't need to implement this method unless your table's data can change.
    */
    public void 
    setValueAt(Object value, int row, int col) 
    {
          
       // If the vCard column is clicked, then show the vCard.
       if(col==3)
       {
          String aKey = (String) keys.elementAt(row);

          if( aKey!=null && col>=0 && col<5 )
          {
             StudentRecord aRec = (StudentRecord) objs.get(aKey);

             if( aRec!=null )
             { 
                dialog = createVCard(aRec); 

                if(dialog!=null)
                {
                   dialog.pack();
                   dialog.setLocationRelativeTo(myFrame);
                   dialog.show();

                }// end of if

             }// end of if

          }// end of if

       }// end of if

    }// end of setValueAt()

    /** adds a new student information */
    public void 
    addStudentRecord(StudentRecord aRec)
    { 
       if( !keys.contains(aRec.getID()) ) keys.addElement(aRec.getID());
       else objs.remove(aRec.getID());

       objs.put(aRec.getID(),aRec);

       fireTableDataChanged(); 

    }// end of addEntityRec()

    /** extracts Student Records from the given Document */
    void extractRecords(Document myDoc)
    {
      // Get the Root Node
      Element dbRoot = myDoc.getDocumentElement();
      NodeList bList = null;

      // Select all the "student" nodes
      if( dbRoot!=null && dbRoot.getTagName().equals("students") ) 
          bList = dbRoot.getElementsByTagName("student");

      // If there is something in the list
      if( bList!=null && bList.getLength()>0) 
      { 
         int i=0;
         while( i<bList.getLength() )
         {
            Node current = bList.item(i++);
 
            // Student record is encapsulated in student element node
            if( current!=null && 
                current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals("student"))
            {
              
               // extract the information
               StudentRecord sRec = new StudentRecord((Element)current);
 
               // add this record to our table
               addStudentRecord(sRec);

            }// end of if 

         }// end of while

      }// end of if 

    }// end of extractRecords()

    /** Creates the vCard representtation for the given Student */
    JDialog createVCard(StudentRecord aRec)
    {
      JDialog ret = null;


      try
      {
        // Define the url for vCard
        URL url = new URL("file:"+ System.getProperty("user.dir")
                                 + System.getProperty("file.separator")
                                 + aRec.getVCard());
   
        System.out.println("url:"+url);
        // Define RDFFactory object
        RDFFactory rdfFac = new RDFFactory();
     
        // Obtain the RDF of the document
        RDFElement dbRoot = rdfFac.getRDFElement(url);

        // If RDF exists
        if( dbRoot!=null )
        {
           // Get list of Descriptions
           DescriptionList dList = dbRoot.getDescriptionElements();
       
           if( dList!=null && dList.getLength()>0) 
           {  
              int i=0;
              while( i<dList.getLength() )
              {
                 DescriptionElement current = dList.item(i++);

                 // Find the Description defining vCard
                 if( current!=null && current.isAbout() &&
                     current.getAbout().equals(aRec.getVCard()) )
                 { 

                   // Put these information into the JDialog if possible 
                   JPanel  box = showVCard(current.getProperties());

                   if(box!=null) 
                   {
                      ret = new JDialog(myFrame,true);

                      ret.setTitle("Information About "+aRec.getID());

                      ret.setContentPane(box);

                      ret.addWindowListener(new WindowAdapter() {
                          public void windowClosing(WindowEvent e) {
                                 StudentTable.this.closeDialog();
                          }
                      });

                   }// end of if

                 }// end of if

              }// end of while

           }// end of if
          
       } else JOptionPane.showMessageDialog( myFrame,"RDF is not available.","",
                                             JOptionPane.ERROR_MESSAGE);

     } catch (Exception e) 
       {  System.err.println("Couldn't create vCard URL: " ); 
          e.printStackTrace();
          JOptionPane.showMessageDialog(myFrame,
                               "vCard is not available.","",
                                JOptionPane.ERROR_MESSAGE);
       }

     return ret;

  }// end of createVCard() 

  /**
  * Extracts the vCard properties.
  */
  JPanel showVCard(PropertyList props)
  {
     JLabel label ;
     JLabel value ;

     JPanel  infoP = new JPanel();
     infoP.setLayout(new GridLayout(3, 2));

     if( props!=null && props.getLength()>0 )
     {
        int i=0;
        while( i<props.getLength() )
        {  
           PropertyElement pElem = props.item(i++);
           if( pElem!=null && !pElem.isResource() )
           { 

              if( pElem.getTagName().equals("vCard:Family") ||
                  pElem.getTagName().equals("Family") )
              {   
                  label = new JLabel("Family Name");
                  value = new JLabel("");

                  label.setBackground(Color.white);
                  label.setForeground(Color.red);
                  value.setBackground(Color.white);
                  value.setForeground(Color.blue);

                  Node aText = pElem.getValue();
                  if( aText.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                      value.setText(((Text)aText).getData());

                  infoP.add(label);
                  infoP.add(value);
              }

              else if( pElem.getTagName().equals("vCard:Given") ||
                       pElem.getTagName().equals("Given") )
                   {   
                      label = new JLabel("Given");
                      value = new JLabel("");

                      label.setBackground(Color.white);
                      label.setForeground(Color.red);
                      value.setBackground(Color.white);
                      value.setForeground(Color.blue);

                      Node aText = pElem.getValue();
                      if( aText.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                         value.setText(((Text)aText).getData());

                      infoP.add(label);
                      infoP.add(value);
                   }

              else if( pElem.getTagName().equals("vCard:Prefix") ||
                       pElem.getTagName().equals("Prefix") )
                   {   
                      label = new JLabel("Prefix");
                      value = new JLabel("");

                      label.setBackground(Color.white);
                      label.setForeground(Color.red);
                      value.setBackground(Color.white);
                      value.setForeground(Color.blue);

                      Node aText = pElem.getValue();
                      if( aText.getNodeType()==org.w3c.dom.Node.TEXT_NODE )
                         value.setText(((Text)aText).getData());

                      infoP.add(label);
                      infoP.add(value);
                   }

           }// end of element

        }// end of while

     }// end of if

     JPanel  box = new JPanel();
     box.setLayout(new BorderLayout());
     box.setBackground(Color.white);

     JButton okButton = new JButton(" Ok ");

     okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                   StudentTable.this.closeDialog(); 
            }
     });

     box.add(infoP,BorderLayout.CENTER);
     box.add(okButton,BorderLayout.SOUTH);

     return box;

  }// end of showVCard()

  /** closes the dialog */
  public void 
  closeDialog() 
  {
     if(dialog!=null)
     {
        dialog.toBack();
        dialog.dispose();
        dialog=null; 

     }// end of if

  }// end of closeDialog()

}// end of StudentTable

