
/** SWING/JFC package */
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.table.*;

/** Java Networking Package */
import java.net.URL;

/** Java AWT Package */
import java.awt.*;
import java.awt.event.*;

/** Java IO Package */
import java.io.*;

/** W3C's DOM API */
import org.w3c.dom.*;

/** DOM Core Level 1 implemenetation */
import edu.npac.iwt.dom.*;

/**
* @author H. Timucin Ozdemir
* @date   February 19, 1999
*/
public class DOMDemo extends JFrame 
{
    private JEditorPane htmlPane;
    private static boolean DEBUG = false;

    private URL helpURL;
    JSplitPane splitPane ;
    JScrollPane htmlView ;

    /**
    */
    public DOMDemo() 
    {

       super("XML/XSL Demo");

       //Create the nodes.
       DefaultMutableTreeNode top = new DefaultMutableTreeNode("Student");
       createDB(top);

       //Create a tree that allows one selection at a time.
       JTree tree = new JTree(top);
       tree.getSelectionModel().setSelectionMode 
            (TreeSelectionModel.SINGLE_TREE_SELECTION);

       //Listen for when the selection changes.
       tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {

                DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                   (e.getPath().getLastPathComponent());

                Object nodeInfo = node.getUserObject();

                if (node.isLeaf()) 
                {
                    WebPageInfo aPage = (WebPageInfo)nodeInfo;
                    displayURL(aPage.resourceURL);

                } else { displayURL(helpURL); }

            }

        });

        //Create the scroll pane and add the tree to it. 
        JScrollPane treeView = new JScrollPane(tree);

        //Create the HTML viewing pane.
        htmlPane = new JEditorPane();
        htmlPane.setEditable(false);
        
        initHelp();
        htmlView = new JScrollPane(htmlPane);

        //Add the scroll panes to a split pane.
        splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setTopComponent(treeView);
        splitPane.setBottomComponent(htmlView);

        Dimension minimumSize = new Dimension(100, 50);
        htmlView.setMinimumSize(minimumSize);
        treeView.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(100); 

        splitPane.setPreferredSize(new Dimension(500, 300));

        //Add the split pane to this frame
        getContentPane().add(splitPane);
    }

    private class WebPageInfo {
        public String resourceName;
        public URL    resourceURL;
        public String prefix = "file:" 
                               + System.getProperty("user.dir")
                               + System.getProperty("file.separator");

        public WebPageInfo(String _resourceName, String filename) {
            resourceName = _resourceName;
            try {
                resourceURL = new URL(prefix + filename);
            } catch (java.net.MalformedURLException exc) {
                System.err.println("Attempted to create a WebPageInfo "
                                   + "with a bad URL: " + resourceURL);
                resourceURL = null;
            }
        }

        public String toString() { return resourceName; }

    }// end of WebPageInfo class

    /**
    */
    void initHelp() 
    {
        String s = null;
        try 
        {
            s = "file:" 
                + System.getProperty("user.dir")
                + System.getProperty("file.separator")
                + "DOMDemoHelp.html";

            helpURL = new URL(s);
            displayURL(helpURL);
        } catch (Exception e) 
          { System.err.println("Couldn't create help URL: " + s); }
    }

    /**
    */
    void displayURL(URL url) 
    {
        
        try {

            if( url.toString().endsWith(".html") || 
                url.toString().endsWith(".htm") )
            { 
               splitPane.setBottomComponent(htmlView);
               htmlPane.setPage(url);
            }
            else 
            if(url.toString().endsWith(".xml"))
            {
               BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));

               String inputLine;
               JTextArea output = new JTextArea();
               output.setEditable(false);

               while ((inputLine = in.readLine()) != null) output.append(inputLine+"\n");

               in.close();

               splitPane.setBottomComponent(new JScrollPane(output));

            }else 
            if(url.toString().endsWith(".sdb"))
            { 
               Freedom dom = new Freedom("edu.npac.iwt.dom.SAXDriver");
               dom.setProperty("sax.driver","com.microstar.xml.SAXDriver");
               Document doc = dom.openDocument(url);

               JTable aTable = new JTable(new StudentTable(doc));
                              
               splitPane.setBottomComponent(new JScrollPane(aTable));
            }

        } catch (IOException e) 
          { System.err.println("Attempted to read a bad URL: " + url); }
    }//

    /**
    * Defines the Tree.
    */
    void createDB(DefaultMutableTreeNode top) 
    {
        DefaultMutableTreeNode category = null;
        DefaultMutableTreeNode resource = null;

        category = new DefaultMutableTreeNode("CIS");
        top.add(category);

        //Define first tree
        resource = new DefaultMutableTreeNode(new WebPageInfo("CIS720.html","CIS720.html"));
        category.add(resource);
        resource = new DefaultMutableTreeNode(new WebPageInfo("Student.xml","student.xml"));
        category.add(resource); 
        resource = new DefaultMutableTreeNode(new WebPageInfo("Student.sdb","student.sdb"));
        category.add(resource);

    }// end of createDB()

    /**
    *
    */
    public static void 
    main(String[] args)
    {
        JFrame frame = new DOMDemo();
 
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);} 
        });  
 
        frame.pack();
        frame.setVisible(true);

    }// end of main()

}// end of DOMDemo
