import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

// W3C's DOM Package
import org.w3c.dom.*;

/**
* Encapsulates the student list in table representation.
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*/
public class 
StudentTable extends AbstractTableModel 
{
  boolean DEBUG=true;
  final String[] columnNames = {"Student ID", "Name", "Surname" };
  Hashtable objs = new Hashtable();
  Vector    keys = new Vector();
  Document myDoc;

  /**
  * @param _myDoc A Document object to extract the student entries.
  */
  public StudentTable( Document _myDoc )
  {
      myDoc = _myDoc;
      extractRecords(myDoc);

  }// end of CONSTUCTOR

  /**
  * Returns the number of columns in the table.
  * @return The number of columns.
  */
  public int getColumnCount() { return columnNames.length; }

  /**
  * Returns the number of rows in the table.
  * @return The number of rows in the table.
  */
  public int getRowCount() { return keys.size(); } 

  /**
  * Returns the name of column.
  * @return The name of the column.
  * @param col The index of column.
  */
  public String 
  getColumnName(int col) { return (col>=0&&col<3?columnNames[col]:""); }

  /**
  * Returns the value at (row,col).
  * @param row
  * @param col
  */
  public Object 
  getValueAt(int row, int col) 
  { 
     Object ret = null;
     String aKey = (String) keys.elementAt(row);

     if( aKey!=null && col>=0 && col<5 )
     {
        StudentRecord aRec = (StudentRecord) objs.get(aKey);

        if( aRec!=null )
          switch(col)
          {  case 0: ret = aRec.getID(); break;
             case 1: ret = aRec.getName(); break;
             case 2: ret = aRec.getSurname(); break;
          }// end of switch

     }// end of if

     return ret;

  }// end of getValueAt()

  /*
  * JTable uses this method to determine the default renderer/
  * editor for each cell.  If we didn't implement this method,
  * then the last column would contain text ("true"/"false"),
  * rather than a check box.
  */
  public Class 
  getColumnClass(int c) 
  { String st = new String();
    return st.getClass();
  }

  /**
  * Adds a new StudentRecord entry.
  * @param aRec a StudentRecord object.
  */
  public void 
  addStudentRecord(StudentRecord aRec)
  { 
     if( !keys.contains(aRec.getID()) ) 
         keys.addElement(aRec.getID());
     else
         objs.remove(aRec.getID());

     objs.put(aRec.getID(),aRec);

     fireTableDataChanged(); 

  }// end of addEntityRec()

  /**
  * Extracts the all student information from the Document.
  * @param myDoc a Document object containing student records.
  */
  void extractRecords(Document myDoc)
  {
            
     Element dbRoot = myDoc.getDocumentElement();
     NodeList bList = null;

     if( dbRoot!=null && dbRoot.getTagName().equals("students") ) 
         bList = dbRoot.getElementsByTagName("student");

     if( bList!=null && bList.getLength()>0) 
     {  int i=0;
        while( i<bList.getLength() )
        {
           Node current = bList.item(i++);
           if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals("student"))
           {
              StudentRecord sRec = new StudentRecord((Element)current);
              addStudentRecord(sRec);

           }// end of if 

        }// end of while

     }// end of if 

  }// end of extractRecords()

}// end of StudentTable

