
import java.io.*;
import java.util.*;

/** W3C's DOM API */
import org.w3c.dom.*;
import org.w3c.dom.Document;

/** DOM Core Level 1 implemenetation */
import edu.npac.iwt.dom.*;
import edu.npac.iwt.domUtil.*;

/**
* DIRDOM encapsulates the XML directory protocol between
* TinyServer and TinyBrowser.
*
* @author H. Timucin Ozdemir
* @date   March 9, 1999 (Updated on March 9, 1999)
*/
public class DIRDOM
{
  String title=null;
  Vector resources = new Vector();
  Vector ancestors = new Vector();
  
  /**
  * @param doc Document object
  */
  public DIRDOM(Document doc)
  {
    extractRecords(doc);

  }// end of CONSTRUCTOR

  /**
  * TinyBrowser uses this method to render the directory XML
  * resource.
  *
  * @param baseURL gives the base URL of this directory.
  * @return The HTML formatted directory representation.
  */
  public String toString(String baseURL)
  {
     String bURL = baseURL;
     if(!bURL.endsWith("/")) bURL+="/";
     StringWriter out = new StringWriter();

     out.write("<html>");
     out.write("<title>"+(title!=null?title:"")+"</title>");
     out.write("<body>");
     out.write("<h1>"+(title!=null?title:"")+"</h1>");

     out.write("<table border=2>");
     Enumeration e = resources.elements();

     while( e.hasMoreElements() )
     {
        ResourceRecord rec = (ResourceRecord) e.nextElement();
        if(rec!=null) out.write(rec.toString(bURL));
     }
     out.write("</table>");

     out.write("</body>");
     out.write("</html>");

     return out.toString();

  }// end of toString()

  /**
  * Extracts the object from Document.
  * @param nyDoc a Document object.
  */
  void extractRecords(Document myDoc)
  {
     Element dbRoot = myDoc.getDocumentElement();
     NodeList bList = null;

     if( dbRoot!=null && dbRoot.getTagName().equals("directory") )
     {

       bList = dbRoot.getElementsByTagName("title");
       if( bList!=null && bList.getLength()>0 )
           extractTitle((Element)bList.item(0));

       bList = dbRoot.getElementsByTagName("ancest");
       if( bList!=null && bList.getLength()>0 )
           extractAncestors((Element)bList.item(0));
       
       bList = dbRoot.getElementsByTagName("resource");

       if( bList!=null && bList.getLength()>0) 
       {  
          int i=0;
          while( i<bList.getLength() )
          {
             Node current = bList.item(i++);
             if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               ((Element)current).getTagName().equals("resource"))
             {
                ResourceRecord sRec = new ResourceRecord((Element)current);
                resources.addElement(sRec); 
             } 

          }// end of while

       }// end of if 

     }// end of if 

  }// end of extractRecords()

  /**
  * Extracts the <i>title</i> element.
  * @param ele an Element object. 
  */
  void extractTitle(Element ele)
  {

     if( ele.getTagName().equals("title") )
     {
       Node aNode = ele.getFirstChild();
       if( aNode.getNodeType() == Node.TEXT_NODE )
           title = ((Text)aNode).getData();
     }

  }// end of extractTitle()

  /**
  * Extracts the <i>ances</i> element.
  * @param ele an Element object.
  */
  void extractAncestors(Element ele)
  {

     if( ele.getTagName().equals("ancest") )
     {
       org.w3c.dom.NamedNodeMap attribs = ele.getAttributes();
       org.w3c.dom.Attr a1 = (org.w3c.dom.Attr) attribs.getNamedItem("name");
       ancestors.addElement(a1.getValue());

       NodeList bList = ele.getElementsByTagName("ancest");
       if( bList!=null && bList.getLength()>0 )
           extractAncestors((Element)bList.item(0));
       
     }// end of if

   }// end of extractAncestors()

}// end of DIRDOM

