
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

/** W3C's DOM API */
import org.w3c.dom.*;
import org.w3c.dom.Document;

/** DOM Core Level 1 implemenetation */
import edu.npac.iwt.dom.*;
import edu.npac.iwt.domUtil.*;

/**
*
* This is TinyBrowser/0.1, it handles directory XML resource
* description and produces HTML based on this XML format. DIRDOM
* object extracts the necessary information from the Document
* object.
*
* @date March 8, 1999 (Updated March 8, 1999) 
* @author H. Timucin Ozdemir
*
*/
public class 
TinyBrowser extends JFrame implements ActionListener, HyperlinkListener
{

  String OpenURL = "Open URL";
  String ExitTxt = "Exit";
  String docURL  = "";

  JMenuItem exitItem;
  JMenuItem openItem;

  JTextArea   output;
  JScrollPane scrollPane;
  JScrollPane htmlView;
  JEditorPane htmlPane;
  JSplitPane  splitPane ;
  
  Properties  props;

  /**
  * @param _props Receives the browser settings about XML/DOM/XSL 
  *               implementations.
  */
  public TinyBrowser(Properties _props) 
  {
     props = _props; 

     JMenuBar menuBar;
     JMenu menu, submenu;
     JMenuItem menuItem;
     JCheckBoxMenuItem cbMenuItem;
     JRadioButtonMenuItem rbMenuItem;

     addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                //System.exit(0);
                TinyBrowser.this.doExit();
            }
     });

     //Add regular components to the window, using the default BorderLayout.
     Container contentPane = getContentPane();
     output = new JTextArea(5, 30);
     output.setEditable(false);
     scrollPane = new JScrollPane(output);
     contentPane.add(scrollPane, BorderLayout.CENTER);

     //Create the menu bar.
     menuBar = new JMenuBar();
     setJMenuBar(menuBar);

     //Build the first menu.
     menu = new JMenu("Menu");
     menuBar.add(menu);

     openItem = new JMenuItem(OpenURL);
     exitItem = new JMenuItem(ExitTxt);
     menu.add(openItem);
     menu.addSeparator();
     menu.add(exitItem);
     openItem.addActionListener(this);
     exitItem.addActionListener(this);

     htmlPane = new JEditorPane();
     htmlPane.setEditable(false);
     htmlView = new JScrollPane(htmlPane);
     htmlPane.addHyperlinkListener( this );

     //Add the scroll panes to a split pane.
     splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
     splitPane.setBottomComponent(htmlView);

     Dimension minimumSize = new Dimension(100, 50);
     htmlView.setMinimumSize(minimumSize);
     splitPane.setDividerLocation(100);

     splitPane.setPreferredSize(new Dimension(500, 300));

     //Add the split pane to this frame
     getContentPane().add(splitPane);

  }// end of CONSTUCTOR

  /**
  * ActionListener interface to collect the user events.
  * @param e ActionEvent
  */
  public void 
  actionPerformed(ActionEvent e) 
  {
     JMenuItem source = (JMenuItem)(e.getSource());
     if( source.getText().equals(ExitTxt)) doExit();
     else if( source.getText().equals(OpenURL) ) openURLProcess();

  }// end of actionPerformed()

  /**
  * Move the frame back, and destroy the execution.
  */
  void doExit()
  {
     toBack();
     System.exit(0);

  }// end of doExit()

  /**
  * Asks the user URL address which is gonna be displayed.
  */
  void openURLProcess()
  {
     GetURL dialog = new GetURL(this,docURL);
     dialog.pack();
     dialog.setLocationRelativeTo(this);
     dialog.show();

     if( dialog.getPageURL().trim().length()>0) 
          displayURL(dialog.getPageURL().trim());
     else giveError(new String("Empty URL"));

  }// end of openURLProcess()

  /**
  * Displays the error message in the warning window.
  * @param mess Error message.
  */
  void giveError(String mess)
  {
     JOptionPane.showMessageDialog( this, "Sorry!"+"\n"+mess,"",
                                    JOptionPane.ERROR_MESSAGE);

  }// end of giveError()

  /**
  * Displays the Web Page.
  * @param pageURL The URL address of the Web Page.
  */
  void displayURL(String pageURL)
  {

     try
     { 
       // Get the URL of the Web Page.
       URL url = new URL(pageURL);

       // Display
       displayURL(url);

       // Update the current document.
       docURL = pageURL;

     } catch( MalformedURLException mue) 
       { giveError(new String("There is no Such Page")); }
 
  }// end of displayURL()

  /**
  *
  * Displays the Web Page pointed by URL object.
  *
  * @param url URL address of the Web Page.
  */
  void displayURL(URL url)
  {
     try
     {
       splitPane.setBottomComponent(htmlView);
       URLConnection urlConnection = url.openConnection();

       urlConnection.setRequestProperty("User-Agent","TinyBrowser/0.1");

       BufferedReader in = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));
 
       String ctype = urlConnection.getContentType();

       HTMLDocument aHtml = new HTMLDocument();

       System.out.println(" ContentType:"+ctype);

       if( ctype.equals("text/xml") )
       {
           iwt.web.xmldomxsl.XMLDOMXSL web = null;

           String xmldomxsl = props.getProperty("org.w3c.xmldomxsl");

           if( xmldomxsl!=null )
           {
              try
              {
                Class xmldomxslClass   = Class.forName(xmldomxsl);;
                Object xmldomxslIns  = xmldomxslClass.newInstance();
                System.out.println(" xmldomxsl:"+xmldomxsl);
                if ( xmldomxslIns instanceof iwt.web.xmldomxsl.XMLDOMXSL ) 
                {  
                   web = (iwt.web.xmldomxsl.XMLDOMXSL)xmldomxslIns;
                   web.setAll(props);

                }// end of if

              } catch(Exception e) {}

           }// end of if

           if(web!=null)
           {
              htmlPane.setContentType("text/html");
              htmlPane.setDocument(aHtml);
              aHtml.setBase(urlConnection.getURL());

              Document doc = web.openDocument(urlConnection.getInputStream(),url.toString());

              DIRDOM dirDOM = new DIRDOM(doc);

              //System.out.println(" after reading into DOM");
              //printDOM.print(doc,"",new PrintWriter(System.out,true)); 
              //System.out.println(" after printing ");

              htmlPane.setText(dirDOM.toString(url.toString()));

           } else displayPlainText(urlConnection.getInputStream());
       } 
       else
        {
          if( !ctype.equals("text/html") )
               displayPlainText(urlConnection.getInputStream());
          else htmlPane.setPage(url);

        }

     } catch( IOException ioe) 
       { giveError(new String("There is no Such Page")); }
 
  }// end of displayURL()

  /**
  *
  * Displays the inputstream in text/plain mode.
  *
  * @param In InputStream to the information that it is supposed to be 
  *           rendered.
  */
  void displayPlainText(InputStream In)
  {
     try
     {
       BufferedReader in = new BufferedReader(new InputStreamReader(In));
       String inputLine;
       JTextArea out = new JTextArea();
       out.setEditable(false);

       while ((inputLine = in.readLine()) != null) out.append(inputLine+"\n");
       in.close();

       splitPane.setBottomComponent(new JScrollPane(out));

     } catch(IOException ioe) 
       { giveError(new String("IOError")); }

  }// en of displayPlainText()

  /**
  *
  * HyperlinkListener interface to receive the click event
  * on the hyperlink.
  *
  * @param e HyperlinkEvent
  */
  public void 
  hyperlinkUpdate(HyperlinkEvent e) 
  {
     if ( e.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
     {  // Display the selected URL.
        displayURL(e.getURL());
     }// end of if

  }// end of hyperlinkUpdate()

  /**
  * 
  */
  public static void 
  main(String[] args) 
  {
     int i=0; 
     Properties props = new Properties();
     String propName = "browser.properties";

     try
     {
       while(i<args.length)
       {
        if( args[i].equals("-prop") && i<args.length-1) propName=args[++i];
        else i++;

       }// end of while

       props.load(new FileInputStream(propName));
       props.list(System.out);

     } catch(Exception e) {}

     TinyBrowser window = new TinyBrowser(props);

     window.setTitle("TinyBrowser/0.1");
     window.setSize(450, 260);
     window.setVisible(true);


  }// end of main

}// end of TinyBrowser

