package edu.npac.iwt;

import java.net.*;
import java.io.*;

import org.w3c.dom.*;
import edu.npac.iwt.dom.*;

import iwt.web.xmldomxsl.*;

/**
*
* Uses our DOM implementation to build the DOM representation of
* a document.
*
* org.xml.sax.driver
* @see edu.npac.iwt.DOMFactory 
*
* @author H. Timucin Ozdemir
* @date   March 3, 1999 (Updated March 3, 1999)
*
*/
public class IWTDOMFactory implements DOMFactory 
{
   String xmlParser    = "com.microstar.xml.SAXDriver";

   /**
   *
   * It only allows to set the xml parser.
   * org.xml.sax.driver property name.
   *
   * @param pName   the name of property.
   * @param pValue  the value of property.
   */
   public void 
   setProperty(String pName, String pValue)
   {
     if( pName.equals("org.xml.sax.driver") )
     {
        try
        {
	      Class parserClass = Class.forName(pValue);;
	    	    
	      Object parser     = parserClass.newInstance();
    
	      if ( parser instanceof org.xml.sax.Parser ) xmlParser = pValue;
          else
            System.out.println(pValue+" does not implement org.xml.sax.Parser!");

        } catch( ClassNotFoundException  e1)
          { System.out.println(" Invalid SAXDriver implementation "+pValue); }
          catch(Exception e) {}

     }// end of if

   }// end of setProperty()

   /**
   * 
   * @return The Document object of the given XML document.
   */
   public org.w3c.dom.Document 
   openDocument(String xmlFile)
   { 
     Document doc = null;
     Freedom dom = new Freedom("edu.npac.iwt.dom.SAXDriver");
    
     // Set the parser.
     dom.setProperty("sax.driver",xmlParser);
  
     try
     {  
        URL docurl      = createURL(xmlFile);
        doc = dom.openDocument(docurl);

     } catch( java.io.IOException ioe) {}

     return doc;

   }// end of openDocument()

   /**
   * 
   * @return The Document object of the given XML document.
   */
   public org.w3c.dom.Document 
   openDocument(URL docurl)
   { 
     Freedom dom = new Freedom("edu.npac.iwt.dom.SAXDriver");
    
     // Set the parser.
     dom.setProperty("sax.driver",xmlParser);
  
     return dom.openDocument(docurl);

   }// end of openDocument()

   /**
   * 
   * @return The Document object of the given XML document.
   */
   public org.w3c.dom.Document 
   openDocument(InputStream docIn, String fName)
   { 
     Freedom dom = new Freedom("edu.npac.iwt.dom.SAXDriver");
    
     // Set the parser.
     dom.setProperty("sax.driver",xmlParser);
  
     return dom.openDocument(docIn);

   }// end of openDocument()

   /** Create an URL from a String */
   public URL 
   createURL(String uri) throws IOException 
   {
	URL url = null;
	try {
	    url = new URL(uri);
	} catch (MalformedURLException ex) {
	    File f = new File(uri);
	    //TIMUCIN url = new URL("file", null, -1, f.getCanonicalPath());
          try { url = new URL(makeAbsoluteURL(uri)); } catch (MalformedURLException ex1) {} 
	}
	return url;
   }

   /**
   */
   public static String 
   makeAbsoluteURL (String url) throws MalformedURLException
   {
      URL baseURL;
      String currentDirectory = System.getProperty("user.dir");
      String fileSep = System.getProperty("file.separator");
      String file = currentDirectory.replace(fileSep.charAt(0), '/') + '/';
      if (file.charAt(0) != '/') { file = "/" + file; }
      baseURL = new URL("file", null, file);
      return new URL(baseURL,url).toString();

   }// end of makeAbsoluteURL()

}// end of IWTDOMFactory
