package edu.npac.iwt;

import java.io.*;
import java.net.*;

import org.w3c.dom.*;

import fr.dyade.koala.xml.xsl.*;
import fr.dyade.koala.xml.xsl.util.*;

import iwt.web.xmldomxsl.*;

/**
* It encapsulates the Koala XSL Processor.
* 
* <table>
* <tr><td>org.w3c.domFactory</td><td>:</td><td>DOM Factory</td></tr>
* </table>
*
* @author Hasan Timucin Ozdemir
* @date   February 9, 1999
*
* @see XSLProcessor
*
*/
public class 
IWTXSLProcessor implements XSLProcessor
{  
   String domFactory    = "edu.npac.iwt.IWTDOMFactory";
 
   /**
   */
   public void 
   setProperty(String pName, String pValue)
   {
     if( pName.equals("org.w3c.domFactory") )
     {
        try
        {
	       Class domClass   = Class.forName(pValue);;
	    	    
	       Object domFctry  = domClass.newInstance();
    
	       if ( domFctry instanceof iwt.web.xmldomxsl.DOMFactory ) 
                domFactory = pValue;
           else 
            System.out.println(pValue+" does not implement iwt.web.xmldomxsl.DOMFactory!");

        } catch( ClassNotFoundException  e1)
          { System.out.println(" Invalid DOMFactory implementation "+pValue); }
          catch(Exception e) {}


     }// end of if

   }// end of setProperty()

   ///// iwt.web.xmldomxsl.DOMFactory Interface

   /**
   * @return The Document object of the given XML document.^M
   */
   public org.w3c.dom.Document
   openDocument(String xmlFile) throws XMLUtilityException
   {
     Document doc = null;
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

	 if ( domPro != null ) doc = domPro.openDocument(xmlFile);
     else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

     return doc;

   }// end of openDocument()

   /**
   * @return The Document object of the given XML document.^M
   */
   public org.w3c.dom.Document
   openDocument(URL xmlFile) throws XMLUtilityException
   {
     Document doc = null;
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

	 if ( domPro != null ) doc = domPro.openDocument(xmlFile);
     else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

     return doc;

   }// end of openDocument()

   /**
   * @return The Document object of the given XML document.^M
   */
   public org.w3c.dom.Document
   openDocument(InputStream xmlFile, String fName) throws XMLUtilityException
   {
     Document doc = null;
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

	 if ( domPro != null ) doc = domPro.openDocument(xmlFile,fName);
     else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

     return doc;

   }// end of openDocument()

   ///// iwt.web.xmldomxsl.XSLProcessor Interface
   /**
   */
   public void 
   applyXSLRules(String doc, String xsl, OutputStream pout)
          throws XMLUtilityException
   {
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

	 if ( domPro != null )
     {
         Document xmlDoc = domPro.openDocument(doc);
         Document xslDoc = domPro.openDocument(xsl);

         applyXSLRules(xmlDoc,xslDoc,pout);

     } else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

   }// end of applyXSLRules()

   /**
   */
   public void 
   applyXSLRules(URL doc, URL xsl, OutputStream pout)
          throws XMLUtilityException
   {
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

	 if ( domPro != null )
     {
         Document xmlDoc = domPro.openDocument(doc);
         Document xslDoc = domPro.openDocument(xsl);

         applyXSLRules(xmlDoc,xslDoc,pout);

     } else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

   }// end of applyXSLRules()

   /**
   */
   public void 
   applyXSLRules(org.w3c.dom.Document doc, org.w3c.dom.Document xsl, OutputStream pout)
          throws XMLUtilityException
   {
     DefaultErrorHandler errHandler = new DefaultErrorHandler(); 

     try
     {  
       XslEval eval = new XslEval(doc,xsl,new PrintStream(pout),errHandler);
       errHandler.printAll(0);
     } catch(fr.dyade.koala.xml.xsl.exception.NotAnXslFileException nxfe) {}

   }// end of applyXSLRules()

   ///// Proprietary Methos/Interface

   /**
   */
   iwt.web.xmldomxsl.DOMFactory 
   getDOMFactory()
   {
     iwt.web.xmldomxsl.DOMFactory ret = null;

     try
     {
        Class domClass   = Class.forName(domFactory);;
	    	    
	    Object domFctry  = domClass.newInstance();
    
	    if ( domFctry instanceof iwt.web.xmldomxsl.DOMFactory ) 
             ret = (iwt.web.xmldomxsl.DOMFactory)domFctry;

     } catch( ClassNotFoundException  e1)
       { System.out.println(" DOMFactory implementation "+domFactory+" not found!"); }
       catch(Exception e) {}

     return ret;

   }// end of getDOMFactory()

}// end of IWTXSLProcessor
