package edu.npac.iwt;

import org.w3c.dom.*;

import iwt.web.xmldomxsl.*;

import java.io.*;
import java.net.*;
import java.util.Properties;

/**
*
* <table>
* <tr><td>org.xml.sax.driver</td><td>:</td><td>XML Parser</td></tr>
* <tr><td>org.w3c.domFactory</td><td>:</td><td>DOM Factory</td></tr>
* <tr><td>org.w3c.xslProcessor</td><td>:</td><td>XSL Processor</td></tr>
* <tr><td>org.w3c.xmldomxsl</td><td>:</td><td>XSL Processor</td></tr>
* </table>
*
* @see edu.npac.iwt.XMLDOMXSL
* @see edu.npac.iwt.DOMFactory
* @see org.xml.sax.Parser
*
*/
public class
XMLDOMXSLImpl implements XMLDOMXSL 
{
   String xmlParser    = "com.microstar.xml.SAXDriver";
   String domDriver    = "edu.npac.iwt.IWTDOMFactory";
   String xslProcessor = "edu.npac.iwt.IWTXSLProcessor";
   

   /**
   */
   public void
   setAll(Properties aProps)
   {
     String xParser = (String) aProps.getProperty("org.xml.sax.driver");
     if( xParser!=null ) setSAXDriver(xParser);
     xParser = (String) aProps.getProperty("org.w3c.domFactory");
     if( xParser!=null ) setDOMDriver(xParser);
     xParser = (String) aProps.getProperty("org.w3c.xslProcessor");
     if( xParser!=null ) setXSLProcessor(xParser);

   }// end of setAll()

   /**
   */
   public void
   setProperty(String propName, String value)
   {
          if( propName.equals("org.xml.sax.driver") ) setSAXDriver(value);
     else if( propName.equals("org.w3c.domFactory") ) setDOMDriver(value);
     else if( propName.equals("org.w3c.xslProcessor") ) setXSLProcessor(value);

   }// end of setProperty()

   /**
   */
   public void 
   setSAXDriver(String sax_driver)
   {      
     try
     {
       Class  saxClass   = Class.forName(sax_driver);
	    	    
	   Object saxParser  = saxClass.newInstance();
    
	   if ( saxParser instanceof org.xml.sax.Parser) xmlParser = sax_driver;
       else
        System.out.println(sax_driver+" does not implement org.xml.sax.Parser!");

     } catch( ClassNotFoundException  e1) 
       { System.out.println(" Invalid SAXDriver implementation "+sax_driver); }
       catch(Exception e) {}
  
   }// end of setSAXDriver()

   /**
   */
   public void 
   setDOMDriver(String dom_driver) 
   {      
     try
     {
       Class  domClass   = Class.forName(dom_driver);
	   Object domFactory = domClass.newInstance();
    
	   if ( domFactory instanceof iwt.web.xmldomxsl.DOMFactory ) 
            domDriver = dom_driver;
       else
        System.out.println(dom_driver+" does not implement iwt.web.xmldomxsl.DOMFactory!");

      } catch( ClassNotFoundException  e1)
       { System.out.println(" Invalid DOMFactory implementation "+dom_driver); }
       catch(Exception e) {}

   }// end of setDOMDriver()

   /**
   */   
   public void 
   setXSLProcessor(String xsl_driver) 
   { 

     try
     {
       Class  xslClass   = Class.forName(xsl_driver);

	   Object xslClassIns = xslClass.newInstance();

	   if ( xslClassIns instanceof iwt.web.xmldomxsl.XSLProcessor ) 
            xslProcessor = xsl_driver;
       else
        System.out.println(xsl_driver+" does not implement iwt.web.xmldomxsl.XSLProcessor!");


     } catch( ClassNotFoundException  e1)
       { System.out.println(" Invalid XSLProcessor implementation "+xsl_driver); }
       catch(Exception e) {}

   }// end of setXSLProcessor

   ///// iwt.web.xmldomxsl.DOMFactory Interface

   /**
   * @return The Document object of the given XML document.^M
   */
   public org.w3c.dom.Document
   openDocument(String xmlFile) throws XMLUtilityException
   {
     Document doc = null;
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

     if ( domPro != null ) doc = domPro.openDocument(xmlFile);
     else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

     return doc;

   }// end of openDocument()

   /**
   * @return The Document object of the given XML document.^M
   */
   public org.w3c.dom.Document
   openDocument(URL xmlFile) throws XMLUtilityException
   {
     org.w3c.dom.Document doc = null;
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

     if ( domPro != null ) doc = domPro.openDocument(xmlFile);
     else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

     return doc;

   }// end of openDocument()

   /**
   * @return The Document object of the given XML document.^M
   */
   public org.w3c.dom.Document
   openDocument(InputStream xmlFile, String fName) throws XMLUtilityException
   {
     Document doc = null;
     iwt.web.xmldomxsl.DOMFactory domPro = getDOMFactory();

     if ( domPro != null ) doc = domPro.openDocument(xmlFile,fName);
     else throw new XMLUtilityException(XMLUtilityException.DOM_NOT_FOUND);

     return doc;

   }// end of openDocument()

   ///// iwt.web.xmldomxsl.XSLProcessor Interface

   /**
   */
   public void 
   applyXSLRules(org.w3c.dom.Document doc, org.w3c.dom.Document xsl, OutputStream pout)
          throws XMLUtilityException
   { 
     iwt.web.xmldomxsl.XSLProcessor processor = getXSLProcessor();
     if( processor!=null ) processor.applyXSLRules(doc,xsl,pout);
     else throw new XMLUtilityException(XMLUtilityException.XSL_NOT_FOUND);

   }// end of applyXSLRules()  

   /**
   */
   public void 
   applyXSLRules(String doc, String xsl, OutputStream pout)
          throws XMLUtilityException
   {
     iwt.web.xmldomxsl.XSLProcessor processor = getXSLProcessor();
     if( processor!=null ) processor.applyXSLRules(doc,xsl,pout);
     else throw new XMLUtilityException(XMLUtilityException.XSL_NOT_FOUND);

   }// end of applyXSLRules()  

   /**
   */
   public void 
   applyXSLRules(URL doc, URL xsl, OutputStream pout) throws XMLUtilityException
   {
     iwt.web.xmldomxsl.XSLProcessor processor = getXSLProcessor();
     if( processor!=null ) processor.applyXSLRules(doc,xsl,pout);
     else throw new XMLUtilityException(XMLUtilityException.XSL_NOT_FOUND);

   }// end of applyXSLRules()  

   //// Proprietary

   /**
   */
   public iwt.web.xmldomxsl.XSLProcessor
   getXSLProcessor()
   {
     iwt.web.xmldomxsl.XSLProcessor ret = null;

     try
     {
        Class  xslClass   = Class.forName(xslProcessor);
        Object xslClassIns = xslClass.newInstance();
        if( xslClassIns instanceof iwt.web.xmldomxsl.XSLProcessor ) 
            ret = (iwt.web.xmldomxsl.XSLProcessor)xslClassIns;

     } catch( ClassNotFoundException  e1) 
       { System.out.println(" XSLProcessor implementation "+xslProcessor+" not found!"); }
       catch(Exception e) {}

     return ret;

   }// end of getXSLProcessor()

   /**
   */
   iwt.web.xmldomxsl.DOMFactory
   getDOMFactory()
   {
     iwt.web.xmldomxsl.DOMFactory ret = null;

     try
     {
        Class domClass   = Class.forName(domDriver);;

        Object domFctry  = domClass.newInstance();

        if ( domFctry instanceof iwt.web.xmldomxsl.DOMFactory )
             ret = (iwt.web.xmldomxsl.DOMFactory)domFctry;

     } catch( ClassNotFoundException  e1) 
       { System.out.println(" DOMFactory implementation "+domDriver+" not found!"); }
       catch(Exception e) {}

     return ret;

   }// end of getDOMFactory()

}// end of XMLDOMXSLIMpl
