package edu.npac.iwt ;

import java.io.* ;
import java.util.*;
import java.net.*;
import java.io.*;
import java.util.Date;

// XSL, XML, DOM related material
import org.w3c.dom.*;
import fr.dyade.koala.xml.xsl.util.*;
import fr.dyade.koala.xml.xsl.*;

import iwt.web.server.*;
import iwt.web.xmldomxsl.*;

/**
*
* Serves XML files with XSL rules. It uses Koala XSL processor to
* process XML files.
* <p>
* <ol>
* <p><li> It shall look at the xsl variable in QueryString.
* <p><li> if the file in xsl variable does not exist or it is not 
*         specified at all, then
*         <ol>
*          <p><li> it shall use filename.xsl if it exist
*          <p><li> it shall use default.xsl if it exist
*          <p><li> it shall return error.
*         </ol>
*
* </ol>
*
* @date October 11, 1998 (Updated at October 11, 1998)
* @version 0.1
* @author Hasan Timucin Ozdemir
*
*/
public class XMLEHandler extends FileResource implements HttpResource
{
   Properties sProps;
  
   public XMLEHandler() { super(); }
   
   public XMLEHandler(HttpServer aServer) { super(aServer); }

   /**
   */
   public void 
   handleResource(Properties _sProps, String resourceName, QueryString qStr, 
                  OutputStream ou, Properties props, HttpRequest req)
   {    
      PrintStream out = new PrintStream(ou);
      System.out.println("resourceName:"+resourceName);

      sProps  = _sProps;
      String UserAgent = props.getProperty("User-Agent");
      if(UserAgent.equals("TinyBrowser/0.2")) 
         serveFile(aServer.getRootDirectory()+resourceName,out);
      else
         handleResource2(_sProps,aServer.getRootDirectory()+resourceName,qStr,out,props);

      System.out.println("User-Agent:"+UserAgent);

   }// end of handleResource()

   /**
   */
   void 
   handleResource2(Properties _sProps, String resourceName, QueryString qStr, PrintStream out, Properties props)
   {    

      sProps  = _sProps;

      /************************/
      /** Get the file name   */
      /************************/
      String xmlFileName = resourceName; 
      int l = xmlFileName.indexOf('?');
      if(l<1) l = xmlFileName.length();
      xmlFileName = xmlFileName.substring(0,l); 
      

      // It should end with 'xml' 
      if(xmlFileName.endsWith("xml") )
      {
	     createInstances();

         // Get the XSL file name
         String query = qStr.getProperty("xsl");

         l = xmlFileName.lastIndexOf('/');
         String qPath = xmlFileName.substring(0,l);

         if( query!=null )
         {
            // assume that .xsl sits in the same directory with 
            // .xml file.

            if(l>0) query = qPath+"/"+query;

         } 
 
         process(query,xmlFileName,qPath,out);

      }
      //else throw new HttpException(HttpServletResponse.SC_FORBIDDEN,
      //                             xmlFileName+" is not accessible! ");

   }// end of service()

   /**
   */
   void 
   process( String xslFileName, String xmlFileName, 
            String qPath, PrintStream out) //ServletOutputStream out)
   {
      String xslFile = xslFileName;
      String xmlFile = xmlFileName;

      File b = new File(xmlFile);
      if(!b.exists() || !b.canRead() ) xmlFile=null;
      
      if(xslFile!=null)
      {
         File a = new File(xslFile);
         if(!a.exists() || !a.canRead() ) xslFile=null;

      }// end of if

      if(xslFile==null)
      {
         xslFile=xmlFileName.substring(0,xmlFileName.length()-3)+"xsl";
         File a = new File(xslFile);
         if(!a.exists() || !a.canRead() ) xslFile=null;

      }// end of if

      if(xslFile==null)
      {
         xslFile=qPath+"/default.xsl";
         File a = new File(xslFile);
         if(!a.exists() || !a.canRead() ) xslFile=null;

      }// end of if

      if(xslFile!=null&&xmlFile!=null)
      {
         try
         { ParseAFile(xslFile,xmlFile,out);
         } catch(java.lang.Exception e) 
           { sendError(out,"XSL Exception : XSL Processing Exception ");
           }

      } else 
        { 
           String messg1 = "XSL File does not found!\n";
           String messg2 = "XML File does not found!";
           String messg  = "XSL Exception : ";
           if(xslFile==null) messg += messg1;
           if(xmlFile==null) messg += messg2;

           sendError(out,messg);

        }// else

   }// end of if

   /**
   * Sends the Error Message
   */
   public void
   sendError(PrintStream out, String message)
   {
       //try
       //{
         out.println("<html>");
         out.println("<head><title>Error</title></head>");
         out.println("<body>");
         out.println("<h1>XSL Processor Error</h1><br><hr>"+message+"<br><hr>");
         out.println("</body></html>");
       //} catch(java.io.IOException e) {}

   }// end of sendError()


    protected static boolean verbose = false;
    protected static String files[]; 
    protected static String xslFile; 
    protected static ResourceBundle resources=null;
    protected static boolean toStdout = false; 

    /** Parsing Files */
    public void 
    ParseAFile(String xslFileName, String fileName, OutputStream out) throws Exception
    {
      String xmldomxsl = sProps.getProperty("org.w3c.xmldomxsl");
      if( xmldomxsl!=null )
      {
        try
        {
	    Class xmldomxslClass   = Class.forName(xmldomxsl);;
	    	    
	    Object xmldomxslIns  = xmldomxslClass.newInstance();
    
          System.out.println(" xmldomxsl:"+xmldomxsl);

	    if ( xmldomxslIns instanceof iwt.web.xmldomxsl.XMLDOMXSL ) 
          {  
            PrintStream pout = new PrintStream(out); 
            printHeader(pout,"200 OK");
            ((iwt.web.xmldomxsl.XMLDOMXSL)xmldomxslIns).setAll(sProps);
            ((iwt.web.xmldomxsl.XMLDOMXSL)xmldomxslIns).applyXSLRules(fileName,xslFileName,pout);
            pout.close();

          }// end of if

        } catch(Exception e) {}

      }// end of if

    }// end of ParseAFiles() 

    public static void 
    createInstances()
    {
	try{ 
	    /*Class outputClass;	
	      outputClass = Class.forName(outputHandler);
	      Object outputInstance = outputClass.newInstance();
	      
	      if (!(outputInstance instanceof OutputDocumentHandler)){
	      throw new IllegalArgumentException("OutputDocumentHandler named "+ outputHandler +
	      " does not support OutputDocumentHanlder interface");
	      }
	      outHandler = (OutputDocumentHandler) outputInstance;
	    */
	    Class errorClass;	
	    errorClass = Class.forName(errorHandler);
	    Object errorInstance = errorClass.newInstance();
	    
	    if (!(errorInstance instanceof XSLErrorHandler)){
		throw new IllegalArgumentException("XSLErrorHandler named "+ errorHandler +
						  " does not support XSLErrorHanlder interface");
	    }
	    errHandler = (XSLErrorHandler) errorInstance;
	}catch (Exception e){
	    e.printStackTrace();
	}
    }
    
    /** Error Handler to use */ 

    private static String errorHandler;
    private static XSLErrorHandler errHandler;

    /** Output Document Handler to use */ 
    /*    private static String outputHandler;
    private static OutputDocumentHandler outHandler;*/

    // Initialisation
    static 
    {
	  if ((errorHandler = System.getProperty("koala.xsl.errorHandler")) == null)
        { errorHandler = "fr.dyade.koala.xml.xsl.DefaultErrorHandler"; }	
    }

}// end of XMLHandler

