package edu.npac.iwt.dom;

//
//
// FreedomTextBuffer
//
//
public final class FreedomTextBuffer
{
    private char	value[];
    private int		count;

		public
	FreedomTextBuffer()
	{
		this(0);
    }

		public
	FreedomTextBuffer(int length)
	{
		value = new char[length];
    }

		public
	FreedomTextBuffer(String str)
	{
		this(str.length());
		
		append(str);
    }

		public final int
	length()
	{
		return count;
    }

		public final int
	capacity()
	{
		return value.length;
    }

		public final void
	ensureCapacity(int minimumCapacity)
	{
		int maxCapacity = value.length;

		if (minimumCapacity > maxCapacity)
		{
			int newCapacity = (maxCapacity + 1) * 2;
			if (minimumCapacity > newCapacity)
				newCapacity = minimumCapacity;

			char newValue[] = new char[newCapacity];
			System.arraycopy(value, 0, newValue, 0, count);
			value = newValue;
		}
    }

		public final void
	setLength(int newLength)
	{
		if (newLength < 0)
			throw new StringIndexOutOfBoundsException(newLength);

		ensureCapacity(newLength);

		if (count < newLength)
			for (; count < newLength; count++)
				value[count] = '\0';

		count = newLength;
    }

		public final char
	charAt(int index)
	{
		if ((index < 0) || (index >= count))
			throw new StringIndexOutOfBoundsException(index);

		return value[index];
    }

		public final void
	getChars(int srcBegin, int srcEnd, char dst[], int dstBegin)
	{
		if ((srcBegin < 0) || (srcBegin >= count))
			throw new StringIndexOutOfBoundsException(srcBegin);

		if ((srcEnd < 0) || (srcEnd > count))
			throw new StringIndexOutOfBoundsException(srcEnd);

		if (srcBegin < srcEnd)
			System.arraycopy(value, srcBegin, dst, dstBegin, srcEnd - srcBegin);
    }

		public final void
	setCharAt(int index, char ch)
	{
		if ((index < 0) || (index >= count))
			throw new StringIndexOutOfBoundsException(index);

		value[index] = ch;
    }

		public final FreedomTextBuffer
	append(Object obj)
	{
		return append(String.valueOf(obj));
    }

		public final FreedomTextBuffer
	append(String str)
	{
		if (str == null)
			str = String.valueOf(str);

		int len = str.length();
		ensureCapacity(count + len);
		str.getChars(0, len, value, count);
		count += len;
		return this;
    }

		public final FreedomTextBuffer
	append(char str[])
	{
		int len = str.length;
		ensureCapacity(count + len);
		System.arraycopy(str, 0, value, count, len);
		count += len;
		return this;
    }

	    public final FreedomTextBuffer
	append(char str[], int offset, int len)
	{
		ensureCapacity(count + len);
		System.arraycopy(str, offset, value, count, len);
		count += len;
		return this;
    }

	    public final FreedomTextBuffer
	append(boolean b)
	{
		return append(String.valueOf(b));
    }

	    public final FreedomTextBuffer
	append(char c)
	{
		ensureCapacity(count + 1);
		value[count++] = c;
		return this;
    }

	    public final FreedomTextBuffer
	append(int i)
	{
		return append(String.valueOf(i));
    }

	    public final FreedomTextBuffer
	append(long l)
	{
		return append(String.valueOf(l));
    }

	    public final FreedomTextBuffer
	append(float f)
	{
		return append(String.valueOf(f));
    }

	    public final FreedomTextBuffer
	append(double d)
	{
		return append(String.valueOf(d));
    }

	    public final FreedomTextBuffer
	insert(int offset, Object obj)
	{
		return insert(offset, String.valueOf(obj));
    }

	    public final FreedomTextBuffer
	insert(int offset, String str)
	{
		if ((offset < 0) || (offset > count))
		    throw new StringIndexOutOfBoundsException();

		int len = str.length();
		ensureCapacity(count + len);
		System.arraycopy(value, offset, value, offset + len, count - offset);
		str.getChars(0, len, value, offset);
		count += len;
		return this;
    }

	    public final FreedomTextBuffer
	insert(int offset, char str[])
	{
		if ((offset < 0) || (offset > count))
			throw new StringIndexOutOfBoundsException();

		int len = str.length;
		ensureCapacity(count + len);
		System.arraycopy(value, offset, value, offset + len, count - offset);
		System.arraycopy(str, 0, value, offset, len);
		count += len;
		return this;
    }

	    public final FreedomTextBuffer
	insert(int offset, boolean b)
	{
		return insert(offset, String.valueOf(b));
    }

	    public final FreedomTextBuffer
	insert(int offset, char c)
	{
		ensureCapacity(count + 1);
		System.arraycopy(value, offset, value, offset + 1, count - offset);
		value[offset] = c;
		count += 1;
		return this;
    }

	    public final FreedomTextBuffer
	insert(int offset, int i)
	{
		return insert(offset, String.valueOf(i));
    }

	    public final FreedomTextBuffer
	insert(int offset, long l)
	{
		return insert(offset, String.valueOf(l));
    }

	    public final FreedomTextBuffer
	insert(int offset, float f)
	{
		return insert(offset, String.valueOf(f));
    }

	    public final FreedomTextBuffer
	insert(int offset, double d)
	{
		return insert(offset, String.valueOf(d));
    }

	    public final FreedomTextBuffer
	reverse()
	{
		int n = count - 1;
		for (int j = (n-1) >> 1; j >= 0; --j)
		{
			char temp = value[j];
			value[j] = value[n - j];
			value[n - j] = temp;
		}
		return this;
    }

	    public final String
	toString()
	{
		return new String(value, 0, count);
    }

		public final String
	toString (int start, int end)
	{
		return new String(value, start, end - start);
	}

		public final char[]
	getCharArray ()
	{
		return value;
	}

		public final boolean
	regionMatches (int toffset, String  str, int  ooffset, int  len)
	{
		if (ooffset < 0 || toffset < 0 || (toffset > count - len) || ooffset > len)
			return false;

		char value[] = this.value;
		while (len-- > 0)
			if (value[toffset++] != str.charAt(ooffset++))
				return false;

		return true;
	}

		public final boolean
	regionMatchesIgnoreCase (int toffset, String  str, int  ooffset, int  len)
	{
		if (ooffset < 0 || toffset < 0 || (toffset > count - len) || ooffset > len)
			return false;

		char value[] = this.value;
		while (len-- > 0)
			if (Character.toUpperCase(value[toffset++]) != Character.toUpperCase(str.charAt(ooffset++)))
				return false;

		return true;
	}

		public final int
	indexOf (char ch, int start, int end)
	{
		for (int i = start; i < end; i++)
			if (value[i] == ch)
				return i;

		return -1;
	}

		public final int
	indexOf (String str, int start, int end)
	{
		char ch = str.charAt(0);
		end -= str.length();
		for (int i = start; i <= end; i++)
			if (value[i] == ch)
				if (regionMatches(i, str, 0, str.length()))
					return i;

		return -1;
	}

		public final int
	indexOfIgnoreCase (char ch, int start, int end)
	{
		char upperChar = Character.toUpperCase(ch);
		char lowerChar = Character.toLowerCase(ch);
		for (int i = start; i < end; i++)
		{
			ch = value[i];
			if (ch == upperChar || ch == lowerChar)
				return i;
		}
		return -1;
	}

		public final int
	indexOfIgnoreCase (String str, int start, int end)
	{
		char ch = str.charAt(0);
		end -= str.length();
		char upperChar = Character.toUpperCase(ch);
		char lowerChar = Character.toLowerCase(ch);
		for (int i = start; i <= end; i++)
		{
			ch = value[i];
			if (ch == upperChar || ch == lowerChar)
				if (regionMatchesIgnoreCase(i, str, 0, str.length()))
					return i;
		}
		return -1;
	}

		public final int
	skipIdentifier (int start, int end, char stop)
	{
		char value[] = this.value;
		for (int i = start; i < end; i++)
		{
			char ch = value[i];
			if (ch == '\"' || ch == '\'' || Character.isWhitespace(ch) || ch == stop)
				return i;
		}
		return end;
	}

		public final int
	skipIdentifier (int start, int end, String stops)
	{
		char value[] = this.value;
		for (int i = start; i < end; i++)
		{
			char ch = value[i];
			if (ch == '\"' || ch == '\'' || Character.isWhitespace(ch) || stops.indexOf(ch) != -1)
				return i;
		}

		return end;
	}

		public final int
	skipWhitespace (int start, int end)
	{
		for (int i = start; i < end; i++)
			if (!Character.isWhitespace(value[i]))
				return i;
	
		return end;
	}

		public final int
	indexOfWhitespace (int start, int end)
	{
		for (int i = start; i < end; i++)
			if (Character.isWhitespace(value[i]))
				return i;
	
		return -1;
	}

		public final void
	delete (int offset, int len)
	{
		int src = offset + len;
		int count = this.count;
		if (src < count)
		{
			char value[] = this.value;
			while (src < count)
				value[offset++] = value[src++];
		}
		this.count = offset;
	}

		public final void
	replace (int offset, int len, String data)
	{
		int dst;
		int src = offset + len;
		int count = this.count;
		if (src >= count)
		{
			this.count = offset;
			append(data);
		}
		else
		{
			int datalen = data.length();
			ensureCapacity(count - len + datalen);
			char value[] = this.value;
			if (len != datalen)
				System.arraycopy(value, src, value, offset + datalen, count - src);
			data.getChars(0, datalen, value, offset);
			this.count = count - len + datalen;
		}
	}

// test methods

		public static void
	main (String args[])
	{
		FreedomTextBuffer text = new FreedomTextBuffer("This is a simple test");
		System.out.println(text);
		text.delete(10, 7);
		System.out.println(text);
		text.insert(10, "stupid ");
		System.out.println(text);
		text.replace(10, 6, "great");
		System.out.println(text);
	}
}

