package edu.npac.iwt.dom;

import java.io.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.xml.sax.*;

//
//
// SAXDriver
//
//
public class SAXDriver extends FreedomDriverSupport
	implements FreedomDriver, DocumentHandler
{
	protected Parser				parser;
	protected FreedomDriverContext	context;

	public
	SAXDriver ()
	{
	}

	public FreedomDocument
	read (Object source)
	{
		if (parser == null && (parser = createParser()) == null)
			return null;

		FreedomDocument	result = null;
		try
		{
			context = new FreedomDriverContext(this);

			parser.setDocumentHandler(this);

			parser.parse(createInputSource(source));
		    if (source instanceof InputStream) ((InputStream)source).close();
			
			result = context.getDocument();
		}
		catch (Exception ex)
		{
			setError(ex);
		}
		finally
		{
			context = null;
		}
		return result;
	}

// internal methods

		/**
		Creates SAX parser using the fully qualified parser class name stored
		as "sax.driver" property.
		*/
		protected Parser
	createParser ()
	{
		Parser parser = null;
		try
		{
			Object driver = getProperty("sax.driver");
			if (driver == null)
				throw new IllegalArgumentException("SAX driver was not specified");

			Class clazz;
			if (driver instanceof Class)
				clazz = (Class)driver;
			else
				clazz = Class.forName(driver.toString());
			
			Object instance = clazz.newInstance();
			
			if (!(instance instanceof org.xml.sax.Parser))
				throw new IllegalArgumentException("SAX driver named " + driver +
					" does not support SAX Parser interface");

			parser = (Parser)instance;
		}
		catch (Exception ex)
		{
			setError(ex);
		}
		return parser;
	}

		protected InputSource
	createInputSource (Object source)
	{

		if (source instanceof InputStream)
			return new InputSource((InputStream)source);
		else if (source instanceof Reader)
			return new InputSource((Reader)source);
		else if (source != null)
			return new InputSource(source.toString());
		else
			return null;

	}

// DocumentHandler implementation

		public void
	setDocumentLocator (Locator locator)
	{
		// TODO
	}

		public void
	startDocument ()
		throws SAXException
	{
	}

		public void
	endDocument ()
		throws SAXException
	{
	}

		public void
	startElement (String name, org.xml.sax.AttributeList atts)
		throws SAXException
	{
		FreedomElement elem = new FreedomElement(name);
		if (atts != null)
		{
			int count = atts.getLength();
			for (int i = 0; i < count; i++)
				elem.setAttribute(atts.getName(i), atts.getValue(i));
		}

		context.add(elem);
		context.enter(elem);
	}

		public void
	endElement (String name)
		throws SAXException
	{
		context.leave();
	}

		public void
	characters (char ch[], int start, int length)
		throws SAXException
	{
		context.add(new FreedomText(new String(ch, start, length)));
	}

		public void
	ignorableWhitespace (char ch[], int start, int length)
		throws SAXException
	{
		context.add(new FreedomText(new String(ch, start, length)));
	}

	public void
	processingInstruction (String name, String remainder)
		throws SAXException
	{
		context.add(new FreedomPI(name, remainder));
	}
}
