package edu.npac.iwt.rdf;

import edu.npac.iwt.dom.*;

import org.w3c.dom.*;
import org.w3c.dom.Node;

import iwt.rdf.*;

import java.net.URL;
import java.io.*;

/**
*
* @author H. Timucin Ozdemir
* @date   Februarary 22, 1999
*
*/
public class RDFFactory implements RDFElementFactory 
{

   /** */ 
   public RDFFactory() 
   {};

   /** */ 
   public RDFElement 
   getRDFElement(URL uri) 
   {
      
      Freedom dom = new Freedom("edu.npac.iwt.dom.SAXDriver");
      dom.setProperty("sax.driver","com.microstar.xml.SAXDriver");

      Document doc = dom.openDocument(uri);

      return getRDFElement(doc);

   }// end of getRDFElement()

   /** */ 
   public RDFElement 
   getRDFElement(String fileName) 
   {
      
      RDFElement ret = null;

      try
      {
        Freedom dom = new Freedom("edu.npac.iwt.dom.SAXDriver");
        dom.setProperty("sax.driver","com.microstar.xml.SAXDriver");
        FileInputStream fin = new FileInputStream(fileName);
        
        Document doc = dom.openDocument(fin);
        fin.close();

        ret = getRDFElement(doc);

      } catch(Exception e) {}

      return ret;

   }// end of getRDFElement()

   /** */ 
   public RDFElement 
   getRDFElement(Element aNode) 
   {

      RDFElement ret = null;

      if( aNode!=null && aNode.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE && 
          ( aNode.getTagName().equals("rdf")||aNode.getTagName().equals("rdf:RDF")))
             ret = buildRDF((Element)aNode);

      return ret;

   }// end of getRDFElement()

   /** */ 
   public RDFElement 
   getRDFElement(Document myDoc)
   {


      Element    dbRoot = myDoc.getDocumentElement();
      NodeList   bList = null;
      RDFElement ret = null;

      if( dbRoot!=null )
      {
         bList = dbRoot.getElementsByTagName("rdf");
         if(bList==null || bList.getLength()<1) bList = dbRoot.getElementsByTagName("rdf:RDF");

      }// end of if

      if( bList!=null && bList.getLength()>0 )
      {
          Node current = bList.item(0);

          if( current!=null )
          {
              if( current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE )
                  ret = buildRDF((Element)current);
     
          }// end of if 
 
      }// end of if

      return ret;

   }// end of getRDFElement() 

   /** */ 
   RDFElement buildRDF(Element aNode)
   {
      RDF ret = null;
      NodeList   bList = null;

      if( aNode!=null )
      {  
         bList = aNode.getElementsByTagName("Description");
         if(bList==null || bList.getLength()<1) 
            bList = aNode.getElementsByTagName("rdf:Description");

      }// end of if

      if( bList!=null && bList.getLength()>0 )
      {
        ret = new RDF(aNode.getOwnerDocument());

        int i=0;
        while( i<bList.getLength() )
        {
           Node current = bList.item(i++);
           if( current!=null && 
               current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
               (((Element)current).getTagName().equals("Description")||
                ((Element)current).getTagName().equals("rdf:Description")))
           {
              DescriptionElement aDesc = extractDescription((Element)current);
              if(aDesc!=null) ret.addDescription(aDesc);

           }// end of if 

        }// end of while^M

      }// end of if

      return (RDFElement)ret;

   }// end of buildRDF() 

   /** */ 
   DescriptionElement extractDescription(Element aNode)
   {
      Description ret = null;

      if( aNode.getTagName().equals("Description") ||
         aNode.getTagName().equals("rdf:Description") )
      {
         org.w3c.dom.NamedNodeMap attribs = aNode.getAttributes();
         org.w3c.dom.Attr a1 = (org.w3c.dom.Attr) attribs.getNamedItem("about");

         if(a1!=null) ret = new Description(aNode.getOwnerDocument(),a1.getValue(),true);
         else
         {
            a1 = (org.w3c.dom.Attr) attribs.getNamedItem("ID");
            if(a1!=null) ret = new Description(aNode.getOwnerDocument(),a1.getValue(),false);
         }

         /** if it is about or ID */
         if( ret!=null )
         {
           NodeList bList = aNode.getChildNodes();
           if( bList!=null && bList.getLength()>0 ) 
           {
             int i=0;
             while( i<bList.getLength() )
             {
                Node current = bList.item(i++);
                if( current!=null && 
                    current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE) 
                   ret.addPropertyElement(extractProperty((Element)current));

              }// end of while^M

           }// end of if

         }// end of if

      }// end of if

      return (DescriptionElement)ret;

   }// end of buildRDF() 

   /** */ 
   PropertyElement extractProperty(Element aNode)
   {
      PropertyElement ret = null;

      org.w3c.dom.NamedNodeMap attribs = aNode.getAttributes();
      org.w3c.dom.Attr a1 = null;

      if(attribs!=null) 
           a1 = (org.w3c.dom.Attr) attribs.getNamedItem("resource");

      if(a1!=null) 
         ret = new RDFProperty(aNode.getOwnerDocument(),aNode.getTagName(),a1.getValue());
      else
       {
          Node current = aNode.getFirstChild();
          if( current!=null && current.getNodeType()==org.w3c.dom.Node.TEXT_NODE)
            ret = new RDFProperty(aNode.getOwnerDocument(),aNode.getTagName(),(Text)current);
          if(current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE )
          {

            if( ((Element)current).getTagName().equals("Bag") )
            {
               IWTBag bElement = extractBag((Element)current);
               if(bElement!=null) ret = new RDFProperty(aNode.getOwnerDocument(),aNode.getTagName(),bElement);
            }// end of if

            if( ((Element)current).getTagName().equals("Seq") )
            {
               IWTSequence bElement = extractSequence((Element)current);
               if(bElement!=null) ret = new RDFProperty(aNode.getOwnerDocument(),aNode.getTagName(),bElement);
            }// end of if

            if( ((Element)current).getTagName().equals("Alt") )
            {
               IWTAlternative bElement = extractAlternative((Element)current);
               if(bElement!=null) ret = new RDFProperty(aNode.getOwnerDocument(),aNode.getTagName(),bElement);
            }// end of if

              
          }// end of if

       }// end of else

      return ret;

   }// end of extractProperty() 

   /** */ 
   IWTBag extractBag(Element aNode)
   {
      IWTBag ret = null;
      Node   ab;

      if( aNode.getTagName().equals("Bag") )
      {
         ret = new IWTBag(aNode.getOwnerDocument());  

         NodeList bList = aNode.getChildNodes();
         if( bList!=null && bList.getLength()>0 ) 
         {
            int i=0;
            while( i<bList.getLength() )
            {
               Node current = bList.item(i++);
               if( current!=null && 
                   current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                   ((Element)current).getTagName().equals("li") ) 
               {
                  ab = current.getFirstChild();
                  if( ab!=null && ab.getNodeType()==org.w3c.dom.Node.TEXT_NODE)
                      ret.addElement((Text)ab);

               }// end of if

             }// end of while^M

         }// end of if

      }// end of if

      return ret;

   }// end of extractBag() 

   /** */ 
   IWTSequence extractSequence(Element aNode)
   {
      IWTSequence ret = null;
      Node   ab;

      if( aNode.getTagName().equals("Seq") )
      {
         ret = new IWTSequence(aNode.getOwnerDocument());  

         NodeList bList = aNode.getChildNodes();
         if( bList!=null && bList.getLength()>0 ) 
         {
            int i=0;
            while( i<bList.getLength() )
            {
               Node current = bList.item(i++);
               if( current!=null && 
                   current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                   ((Element)current).getTagName().equals("li") ) 
               {
                  ab = current.getFirstChild();
                  if( ab!=null && ab.getNodeType()==org.w3c.dom.Node.TEXT_NODE)
                      ret.addElement((Text)ab);

               }// end of if

             }// end of while^M

         }// end of if

      }// end of if

      return ret;

   }// end of extractSequence() 

   /** */ 
   IWTAlternative extractAlternative(Element aNode)
   {
      IWTAlternative ret = null;
      Node   ab;

      if( aNode.getTagName().equals("Alt") )
      {
         ret = new IWTAlternative(aNode.getOwnerDocument());  

         NodeList bList = aNode.getChildNodes();
         if( bList!=null && bList.getLength()>0 ) 
         {
            int i=0;
            while( i<bList.getLength() )
            {
               Node current = bList.item(i++);
               if( current!=null && 
                   current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE &&
                   ((Element)current).getTagName().equals("li") ) 
               {
                  ab = current.getFirstChild();
                  if( ab!=null && ab.getNodeType()==org.w3c.dom.Node.TEXT_NODE)
                      ret.addElement((Text)ab);

               }// end of if

             }// end of while^M

         }// end of if

      }// end of if

      return ret;

   }// end of extractAlternative() 

}// end of RDFFactory

