package fr.dyade.koala.xml.xsl;
/** DefaultErrorHandler.java 
 */
import org.xml.sax.SAXParseException;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.Locator;
import fr.dyade.koala.xml.xsl.exception.*;
import java.io.*;
/** This interface defines an Error Handler with Text result
 *  @author Jrmy CALLES
 *  @version 0.7b1
 */
public class DefaultErrorHandler implements XSLErrorHandler{ 

    protected PrintWriter err;
    
    public DefaultErrorHandler(){
	err = new PrintWriter(System.err);
    }
    
    public DefaultErrorHandler(OutputStream s){
	err = new PrintWriter(s);
    }
    public void setOutputStream(OutputStream s){
	err = new PrintWriter(s);
    }
    public void setPrintWriter(PrintWriter s){
	err = s;
    }
    /**
    * Receive notification of a warning
    */
    public void warning (String error){
	err.println("Warning: " + error);
	err.flush();
    }
    /**
    * Receive notification of an error
    */
    public void error (String error){
	err.println("Error: " + error);
	err.flush();
    }
    /**
     * Receive notification of a fatal error.
     */
    public void fatalError(String error) throws XSLFatalErrorException{
	throw new XSLFatalErrorException(error);
    }
    /**
     * Receive notification of a warning from the XML SAX Parser
     */
    public void warning(SAXParseException e) throws SAXException {
	    showMessage("Warning:", e);
    }
    /**
     * Receive notification of an error from the XML SAX Parser
     */
    public void error(SAXParseException e) throws SAXException {
    }
    /**
     * Receive notification of a fatal error from the XML SAX Parser
     */
    public void fatalError(SAXParseException e) throws SAXException,XSLFatalErrorException {
	showMessage("Fatal Error:", e);
	throw new XSLFatalErrorException();
    }
    // Error Handler 
    private void showMessage(String kind, SAXParseException e) {	
	String file = e.getSystemId();

	err.println(kind + " l:" + e.getLineNumber() + " c:" + e.getColumnNumber() + " " + e.getMessage());
	err.flush();
    }
    /**
     * print all preceding errors
     */
    public void printAll(int type){
	//Nothing
    }        
}
