package fr.dyade.koala.xml.xsl;
/** HtmlErrorHandler.java 
 */
import org.xml.sax.SAXParseException;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.Locator;
import fr.dyade.koala.xml.xsl.exception.*;
import java.io.*;
import java.util.*;
/** This classe defines an Error Handler with HTML output
 *  @author Jrmy CALLES
 *  @version 0.7b1
 */
public class HtmlErrorHandler implements XSLErrorHandler{ 
        
    protected Vector warning;
    protected Vector error;
    protected Vector parserWarning;
    protected Vector parserError;
    
    protected PrintWriter err;

    public HtmlErrorHandler(){
	err = new PrintWriter(System.err);
	warning = new Vector();
	error = new Vector();
	parserWarning = new Vector();
	parserError = new Vector();
    }
    
    public HtmlErrorHandler(OutputStream s){
	err = new PrintWriter(s);
	warning = new Vector();
	error = new Vector();
	parserWarning = new Vector();
	parserError = new Vector();
    }

    public void setOutputStream(OutputStream s){
	err = new PrintWriter(s);
    }

    public void setPrintWriter(PrintWriter s){
	err = s;
    }

    /**
    * Receive notification of a warning
    */
    public void warning (String error){
	warning.addElement(error);
    }
    /**
    * Receive notification of an error
    */
    public void error (String e){
	error.addElement(e);
    }
    /**
     * Receive notification of a fatal error.
     */
    public void fatalError(String error) throws XSLFatalErrorException{
	throw new XSLFatalErrorException(error);
    }
    /**
     * Receive notification of a warning from the XML SAX Parser
     */
    public void warning(SAXParseException e){
	String file = e.getSystemId();
	parserWarning.addElement(file);
	parserWarning.addElement(" Line : " + e.getLineNumber() + " Column : " + e.getColumnNumber() + " " + e.getMessage());
    }
    /**
     * Receive notification of an error from the XML SAX Parser
     */
    public void error(SAXParseException e){
    }
    /**
     * Receive notification of a fatal error from the XML SAX Parser
     */
    public void fatalError(SAXParseException e) throws XSLFatalErrorException {
	String file = e.getSystemId();
	parserError.addElement(file +": line:" + e.getLineNumber() + " column:" + e.getColumnNumber() + " " + e.getMessage());
	throw new XSLFatalErrorException(e.getMessage());
    }

    private String addAmpTag(String s){
	// Change < to &lt; if the error should be printed in text/html
	int i = 0;
	while ((i = s.indexOf("<")) != -1){
	    String left = s.substring(0,i);
	    String right = s.substring(i+1);	    
	    s = left + "&lt;" + right;
	}
	return s;
    }
    
    /** Print the error/warning report in HTML or TEXT format
     */
    public void printAll(int type){
	String tmp = new String();
	if ((!(parserWarning.isEmpty())) || (!(warning.isEmpty()))){
	    if (type == TEXT_HTML){
		tmp += "<a name=\"warnings\"></a> <h2>Warnings : </h2>";
	    }else{
		tmp += "Warnings : \n";
	    }
	// Parser Warnings
	if (!(parserWarning.isEmpty())){
	    if (type == TEXT_HTML){
		tmp +=  "<dl><ul>";
	    }else{
		tmp += "\t";
	    }
	    Enumeration enum = parserWarning.elements();
	    String file = "";
	    while (enum.hasMoreElements()){
		String filename = (String) enum.nextElement();
		String error = (String) enum.nextElement();
		if (!(file.equals(filename))){
		    file = filename;
		    if (type == TEXT_HTML){
			tmp += "<dt> URI: <a href=\"" + file + "\">"+ file + "</a>";
		    }else{
			tmp += "\tURI: " + file + "\n";
		    }
		}
		if (type ==  TEXT_HTML){ 
		    tmp += "<dd><li>" + addAmpTag(error);
		}else{
		    tmp += error + "\n";
		}
	    }
	    if (type == TEXT_HTML){
		tmp += "</dl></ul>"; 
	    }else{
		tmp += "\n";
	    }
	}
	//XSl Warnings
	if (!(warning.isEmpty())){
	    if (type == TEXT_HTML){
		tmp += "<ul><dl>";	   
	    }else{
		tmp += "\t";
	    }
	    Enumeration enum = warning.elements();
	    while (enum.hasMoreElements()){
		String error = (String)enum.nextElement();
		if (type == TEXT_HTML){
		    tmp += "<dd><li>" + addAmpTag(error);
		}else{
		    tmp += "\t" + error + "\n";
		}
	    }
	    if (type == TEXT_HTML){
	    tmp += "</dl></ul>";
	    }else{}
	}
	if (type == TEXT_HTML){
	    tmp +="</dl>\n";	
	}else{
	}
	}

	if (!(error.isEmpty())){
	    if (type == TEXT_HTML){
		tmp += "<a name=\"errors\"></a> <h2>Errors : </h2>";
		tmp += "<ul><dl>";	    
	    }else{
		tmp += "Errors: \n";
	    }
	    Enumeration enum = error.elements();
	    while (enum.hasMoreElements()){
		String error = (String)enum.nextElement();
		if (type == TEXT_HTML){
		    tmp += "<dd><li>" + addAmpTag(error);
		}else{
		    tmp += error + "\n";
		}
	    }
	    if (type == TEXT_HTML){
		tmp += "</dl></ul>";
		tmp +="</dl>\n";		
	    }else{
		tmp += "\n";
	    }
	}

	if (!(parserError.isEmpty())){
	    if (type == TEXT_HTML){
		tmp += "<a name=\"errors\"></a> <h2>Parser Errors : </h2>";
		tmp += "<ul><dl>";	    
	    }else{
		tmp += "Parser Errors: \n";
	    }
	    Enumeration enum = parserError.elements();
	    while (enum.hasMoreElements()){
		String error = (String)enum.nextElement();
		if (type == TEXT_HTML){
		    tmp += "<dd><li>" + addAmpTag(error);
		}else {
		    tmp += error + "\n";
		}
	    }
	    if (type == TEXT_HTML){
		tmp += "</dl></ul>";
		tmp +="</dl>\n";	
	    }else{
		tmp += "\n";
	    }	    
	}
	err.println(tmp);
	err.flush();
	// Obligatoire
	warning = new Vector();
	error = new Vector();
	parserWarning = new Vector();
	parserError = new Vector();
    }     

    // Constants
    public static final int TEXT_HTML = 1;
    public static final int TEXT_TEXT = 0;
}
