package fr.dyade.koala.xml.xsl;
import org.w3c.dom.*;
import java.util.*;
import java.io.*;
import fr.dyade.koala.xml.xsl.util.*;
import java.net.*;

public class Main{
    protected static boolean verbose = false;
    protected static String files[]; 
    protected static String xslFile; 
    protected static ResourceBundle resources=null;
    protected static boolean toStdout = false; 

    public static void main (String args[]) throws Exception{
	// Create Error and Output Handler instances
	createInstances();
	// Command line parsing 
	ParseCommand(args);
	// Parsing Files	
	ParseFiles();
    }
    /** Parsing Files */
    public static void ParseFiles() throws Exception{
	DOMParse dom = new DOMParse(errHandler); 
	URL xsl = dom.createURL(xslFile);
	// Parsage Xsl vers DOM
	System.err.println("Xsl, parsing " + xslFile);
	Document XslDOM = dom.parseFile(xsl);       
	for (int i = 0; i < files.length; i++){
	    URL	tmp = dom.createURL(files[i]);		
	    // Parsage Xml vers DOM
	    System.err.println("Xml, parsing " + tmp);
	    Document XmlDOM = dom.parseFile(tmp);       
	    if ((XmlDOM != null) && (XslDOM != null)){
		// Interpretation
        //Timucin
		//XslEval eval = new XslEval(XmlDOM,XslDOM,errHandler);

        OutputStream out = new FileOutputStream("xsl.out");
		XslEval eval = new XslEval(XmlDOM,XslDOM,out,errHandler);

		// Ecriture des erreurs 
		errHandler.printAll(0);
	    }else{
		if (XmlDOM != null)
		    System.err.println("Error: unable to find file " + tmp);
		else
		    System.err.println("Error: unable to find file " + xsl);
	    }
	}
    } 

    /** Parsage des arguments de la ligne de commande */
    private static void ParseCommand(String args[]) {
	int NbFiles = 0;
	int i;
	for (i = 0; (i < args.length)&&(NbFiles == 0); i++){
	    if (args[i].equals("-v")){
		verbose = true;
		System.err.println("Verbose mode: on");
	    }else if (args[i].equals("-help")){
		System.err.println("Usage: [-v|-help] -r xslUrl [xmlUrl]");
		System.exit(1);	
	    }else if (args[i].equals("-r")){
		if (++i < args.length){
		    xslFile = args[i];
		}else{
		    System.err.println("Error: -r option must be followed by the name of the xsl file to Parse");
		    System.exit(1);	
		}
	    }else{
		NbFiles = args.length - i;
		if (NbFiles > 1){
		    System.err.println("Error: Only one XML file.");
		    System.exit(1);
		}
	    }	    
	}
	// Construction du tableau de fichiers
	if (NbFiles == 0){
	    System.err.println("Error: You haven't give me any XML files");
	    System.err.println("Usage: [-v|-help] -r xslUrl [xmlUrl]*");
	    System.exit(1);	
	}
	files = new String[NbFiles];
	if (NbFiles > 0){
	    System.arraycopy(args,
			     i-1,
			     files,
			     0,
			     NbFiles);
	}
    }  

    public static void createInstances(){
	try{ 
	    /*Class outputClass;	
	      outputClass = Class.forName(outputHandler);
	      Object outputInstance = outputClass.newInstance();
	      
	      if (!(outputInstance instanceof OutputDocumentHandler)){
	      throw new IllegalArgumentException("OutputDocumentHandler named "+ outputHandler +
	      " does not support OutputDocumentHanlder interface");
	      }
	      outHandler = (OutputDocumentHandler) outputInstance;
	    */
	    Class errorClass;	
	    errorClass = Class.forName(errorHandler);
	    Object errorInstance = errorClass.newInstance();
	    
	    if (!(errorInstance instanceof XSLErrorHandler)){
		throw new IllegalArgumentException("XSLErrorHandler named "+ errorHandler +
						  " does not support XSLErrorHanlder interface");
	    }
	    errHandler = (XSLErrorHandler) errorInstance;
	}catch (Exception e){
	    e.printStackTrace();
	}
    }
    
    /** Error Handler to use */ 
    private static String errorHandler;
    private static XSLErrorHandler errHandler;
    /** Output Document Handler to use */ 
    /*    private static String outputHandler;
    private static OutputDocumentHandler outHandler;*/
    // Initialisation
    static {
	if ((errorHandler = System.getProperty("koala.xsl.errorHandler")) == null){
	    errorHandler = "fr.dyade.koala.xml.xsl.DefaultErrorHandler";
	}	
    }
}
