package fr.dyade.koala.xml.xsl;
import org.w3c.dom.*;
import java.util.*;
/** Number.java
 */
/** This class defines an XSL:Number 
 *  @author Jrmy CALLES
 *  @version 0.7b1
 *  @see XSLNumberConstants
 */
public class Number{
    protected int[] number;
    protected int level;
    protected String from;
    protected Vector counts;
    protected String target;
    protected Vector format;
    protected Vector before_format;
    protected String after_format;
    /** Initialiszer with no argument */
    public Number(){
	number = new int[1];
	level = XSLNumberConstants.SINGLE;
	from = null;
	counts = null;
	format = null;
	before_format = null;
	after_format = "";
    }
    /** Initializer. (see WD 1.0 for Details)
     *  @param l level
     *  @param f from
     *  @param c count
     *  @param fo format
     *  @param ta target, the node that has called the XSL:number action
     */
    public Number(String l,String f,String c,String fo,String ta){
	// ATTENTION le TARGET pourrait etre un truc du type fg/toto[attribute(fghgfd)]
	number = new int[10];
	number[0] = 0;
	if (l == null){
	    level = XSLNumberConstants.SINGLE;
	}else if (l.equals(XSLNumberConstants.LEVEL_MULTI)){
	    level = XSLNumberConstants.MULTI;
	}else if (l.equals(XSLNumberConstants.LEVEL_ANY)){
	    level = XSLNumberConstants.ANY;
	}else{
	    level = XSLNumberConstants.SINGLE;
	}
	from = f;
	parseCount(c); 
	int x = ta.lastIndexOf(XSLTags.SEP);
	if (x == -1){
	    target = ta;
	}else{
	    target = ta.substring(x+1);
	}
	parseNumberFormat(fo);
    }
    /** toString.
     *  format the number as it was request.
     */
    public String toString(){
	String tmp = "";
	Enumeration tmpBF = before_format.elements();
	Enumeration tmpFo = format.elements();
	int i = 0;
	int  lastFo = 0;
	int  fo = 0;
	String bF = "";
	while (i < number.length){
	    try {
	        fo = ((Integer)tmpFo.nextElement()).intValue();
	    }catch (NoSuchElementException e){
		fo = lastFo;
	    }
	    lastFo = fo;
	    try{
		bF  = (String)tmpBF.nextElement();
	    }catch (NoSuchElementException e){
		bF = "";
	    }

	    tmp += bF;
	    tmp += format(number[i],fo);
	    i++;
	}
	while (tmpBF.hasMoreElements()){
	    bF  = (String)tmpBF.nextElement();
	    tmp += bF;
	}
	return tmp;
    }
    /** the type of format requested A,a,1,i (see WD 1.0)*/
    private static String format(int number,int fo){
	if (fo == XSLNumberConstants.FORMAT_1){
	    return Integer.toString(number);
	}else if (fo == XSLNumberConstants.FORMAT_A){
	    return format_A(number,'A');
	}else if (fo == XSLNumberConstants.FORMAT_a){
	    return format_A(number,'a');
	}else if (fo == XSLNumberConstants.FORMAT_i){
	    return format_R(number,'i').toLowerCase();
	}else if (fo == XSLNumberConstants.FORMAT_I){
	    return format_R(number,'I');
	}
	return null;
    }
    
    /** Format a number in A or a form (see WD 1.0) */
    private static String format_A(int num,char t){
	String tmp = "";
	while (num > 26){
	    int r = num % 26;
	    num = num / 26;
	    if (r == 0){ 
		r = 26;
		num--;
	    }
	    tmp = new Character((char)(t + r -1)) + tmp;	
	}
	tmp = new Character((char)(t+num-1)) + tmp;
	return tmp;
    }
    /** Format a number in roman (see WD 1.0) */
    private static String format_R(int num,char t){
	String tmp = "";
	if (num >= 9000)
	    tmp = "%%%%% Out of Range %%%%%";
	else{
	    tmp = DToR(num);
	}
	return tmp;
    }
    /** Decimal to Roman conversion */
    private static String DToR(int num){
	String tmp = "";
	while (num>0){
	    if (num>=5000){
		num=num-5000; tmp+="P";
	    }else{
		if (num>=1000){
		    if (num>= 4000) {
			num=num-4000; tmp+="MP";
		    }else {
			num=num-1000; tmp+="M";
		    }
		}else {
		    if (num>=500){
			if (num>=900) {
			    num=num-900; tmp+="CM";
			}else {
			    num=num-500; tmp+="D";
			}
		    }else {
			if (num>=100){
			    if (num>=400) {
				num=num-400; tmp+="CD";
			    }else {
				num=num-100; tmp+="C";
			    }
			}else{
			    if (num>=50){
				if (num>=90){
				    num=num-90; tmp+="XC";
				}else {
				    num=num-50; tmp+="L";
				}
			    }else {
				if (num >=10){
				    if (num>=40) {
					num=num-40; tmp+="XL";
				    }else {
					num=num-10; tmp+="X";
				    }
				} else {
				    if (num>=5){
					if (num>=9){
					    num=num-9; tmp+="IX";
					}else {
					    num=num-5; tmp+="V";
					}
				    }else{
					if (num>=1){
					    if (num>=4) {
						num=num-4; tmp+="IV";
					    }else {
						num=num-1; tmp+="I";
					    }
					}
				    }
				}
			    }
			}
		    }
		}
	    }
	}
	return tmp;
    }     
    /** Process a node, and do the numbering */
    protected void process(Element e){
	if (level == XSLNumberConstants.SINGLE){
	    number = new int[1];
	    number[0] = processSingle(e);
	}else 	if (level == XSLNumberConstants.MULTI){
	    processMulti(e);
	}else if (level == XSLNumberConstants.ANY){
	    // Doit compter le nombre total de fils
	    number = new int[1];	  
	    number[0] += processAny(e);
	}
    }      
    /** parse the "format" attribute given in the XSL:number */
    protected void parseNumberFormat(String fo){       
	format = new Vector();
	before_format = new Vector();
	if (fo == null){	
	    format.addElement(new Integer(XSLNumberConstants.FORMAT_1));
	    before_format.addElement("");
	}else{
	    int i = 0;
	    String tmp = "";
	    while (i < fo.length()){
		char c = fo.charAt(i);
		if ((c == '1')||(c == 'A')||(c == 'a')||(c=='I')||(c=='i')){		    
		    break;
		}else{
		    tmp += String.valueOf(c);
		    i++;
		}
	    } 
	    before_format.addElement(tmp);
	    while (i < fo.length()){
	        char c = fo.charAt(i++);	
		switch (c){
		case '1': 
		    format.addElement(new Integer(XSLNumberConstants.FORMAT_1));
		    break;
		case 'A':
		    format.addElement(new Integer(XSLNumberConstants.FORMAT_A));
		    break;
		case 'a':
		    format.addElement(new Integer(XSLNumberConstants.FORMAT_a));
		    break;
		case 'I':
		    format.addElement(new Integer(XSLNumberConstants.FORMAT_I));
		    break;
		case 'i':
		    format.addElement(new Integer(XSLNumberConstants.FORMAT_i));
		    break;
		default:
		    before_format.addElement(String.valueOf(c));		    
		}
	    }
	}
    }
    /** parse the "count" attribute given in the XSL:number */
    public void parseCount(String c){
	counts = new Vector();
	if (c != null){
	    int i = 0;
	    while ((i = c.indexOf(XSLTags.MULTI_SEP)) != -1){
		String tmp = c.substring(0,i);
		counts.addElement(tmp);
		c = c.substring(i+1);
	    }
	    if (!(c.equals("")))
		counts.addElement(c);
	}
    }
    /** do a multi-level numbering (See WD 1.0)*/
    private void processMulti(Element e){
	number = new int[counts.size()];
	int i = 0;
	Enumeration enum = counts.elements();     
	String frm = null;
	String search = null;
	while (enum.hasMoreElements()){
	    if (i == 0){
		from = frm;
	    }
	    search = (String) enum.nextElement();
	    
	    number[i++] = process(e,search,true);
	    from = search; 
	}
    }
    /** do a  Single or Multi level numbering (See WD 1.0)*/
    private  int process(NodeList e,String search,boolean deep){
	int cpt = 0;
	int size = e.getLength();
	while (size > 0){
	    Node node = e.item(--size);	    
	    if (node.getNodeType() == Node.ELEMENT_NODE){
		cpt += process((Element)node,search,deep);
	    }
	    if (node.hasChildNodes()){
		cpt += process(node.getChildNodes(),search,deep);
	    }
	}
	return cpt;
    } 
    /** do a Single level numbering (See WD 1.0)*/
    private int processSingle(Element e){
	int cpt = 0;
	String search;
	if (counts.isEmpty()){
	    search = target;
	}else{
	    search = (String)counts.firstElement();
	}	
	return process(e,search,false);
    }
    /** do a  Single or Multi level numbering (See WD 1.0)*/
    private int process(Element e,String search,boolean deep){
	int cpt = 0;
	Node node = e;	
	while(node != null){
	    if (node.getNodeName().equals(search)){
		cpt ++;		
	    }
	    node = node.getPreviousSibling();	    	    
	}
	
	if (deep == true){
	    // Pour Multi
	    node = e.getParentNode();	    
	    if (node.getNodeType() == Node.DOCUMENT_NODE){
		return cpt;		
	    }else if (node.getNodeName().equals(from)){
		return cpt;		
	    }else{			       
		cpt += process((Element)node,search,true);
		return cpt;    
	    }
	}else{
	    return cpt;
	}
    }
    /** do an Any level numbering (see WD 1.0)*/
    private int processAny(Node node){       
	String search = target;
	if (counts.size() == 1){
	    search = (String)counts.firstElement();
	}       
	int cpt = parcours1(node,search);
	if (node.getNodeName().equals(search)){
	    cpt++;
	}
	if (cpt < 0)
	    return 0;
	else{
	    return cpt;
	}
    }
    /** tree processing */
    private int parcours1(Node node,String search){
	if (node.getNodeType() == Node.DOCUMENT_NODE){
	    if (from == null){
		return 0;
	    }else{
		return Integer.MIN_VALUE;
	    }
	}else if (node.getNodeName().equals(from)){
	    return 0;
	}else{
	    int cpt = 0;
	    Node parent = node.getParentNode();
	    Node child = parent.getFirstChild();
	    
	    while ((child != null) && (child != node)){
		cpt += parcours2(child,search);
		child = child.getNextSibling();
	    }
	    
	    if (search.equals(parent.getNodeName()))
		cpt ++;

	    cpt += parcours1(parent,search);
	    return cpt;
	}
    }
    /** tree processing */
    private int parcours2(Node n,String search){
	int cpt = 0;	
	if (search.equals(n.getNodeName())){
	    cpt ++;
	}
	if (n.hasChildNodes()){
	    Node child = n.getFirstChild();
	    
	    while ((child != null)){
		cpt += parcours2(child,search);
		child = child.getNextSibling();
	    }
	}
	return cpt;
    }
}
