package fr.dyade.koala.xml.xsl;
import java.util.*;
import org.w3c.dom.*;

import fr.dyade.koala.xml.xsl.util.*;
/** TagAttributes.java
 */
/** This class defines an Xsl Pattern in the document.
 *  @author Jrmy CALLES
 *  @version 0.7b1
 *  @see fr.dyade.koala.xml.xsl.XSLTags
 *  @see fr.dyade.koala.xml.xsl.TagAttributes
 */
public class Pattern{  
    /** the Error Handler to use */
    protected static XSLErrorHandler err;
    /** The Target Tag of the Xsl Pattern */
    protected TagAttributes targetElement;
    /** A Vector of TagAttributes */
    protected Vector path;
    /** how many WildCard the pattern has? */
    protected int hasWildCard;
    /** Priority attribute of the pattern.
     *  The value of this must be an integer (positive or negative). The default priority is 0. The integer must not be greater
     *   than 2147483647 nor less than -2147483648.
     */
    protected int priority; 
    /** a private counter */
    private int i = 0;
    /** Initializer.
     *  @param pat a Vector of TagAttributes
     */
    public Pattern (Vector pat, XSLErrorHandler error){
	path = pat;
	targetElement = findTargetElement();
	priority = 0;
	hasWildCard = countWildCard();
	err = error;
    }
    /** Initializer
     *  @param pat a Vector of TagAttributes
     *  @param prio the priority of the Pattern
     */
    public Pattern (Vector pat, int prio, XSLErrorHandler error){
	path = path;
	priority = prio;
	targetElement = findTargetElement();
	hasWildCard = countWildCard();
	err = error;
    }

    /** toString */
    public String toString(){
	String tmp = XSLTags.PATTERN + "=\"";
	int cpt = 0;
	if (path != null){
	    Enumeration enum = path.elements();
	    while (enum.hasMoreElements()){
	       TagAttributes tA = (TagAttributes)enum.nextElement();
	       if (cpt > 0){
		   if (tA.getSourcePattern() == XSLTags.CHILD_PATTERN)
		       tmp += XSLTags.QUALIFIER_OP;
		   else
		       tmp += XSLTags.SEP;		   

	       }else{
		   cpt++;
	       }
	       tmp += tA.toString(); 
	       if (tA.getSourcePattern() == XSLTags.CHILD_PATTERN)
		       tmp += XSLTags.QUALIFIER_CP;
	    }
	}
	tmp += "\"";
	return tmp;
    } 
    /** toString for debug */
    public String toStringDebug(){
	int cpt = 0;
	String tmp;
	tmp = "*" + targetElement.toString() + targetElement.getSourcePattern() +"\n";
	if (path != null){
	    Enumeration enum = path.elements();
	    while (enum.hasMoreElements()){
		TagAttributes tA = (TagAttributes)enum.nextElement();
		tmp += tA.getSourcePattern() + ":"; 
		tmp += tA.toString() + "\n" ; 
	    }
	}
	return tmp;
    }
    
    /** null?.
     *  @return Is the target Element of the pattern null or if the pattern is null?
     */
    public boolean isNull(){
	return ((targetElement == null) && ((path == null) || (path.size() == 0)));
    }
    
    /** Count the wildcards in the pattern
     */
    protected int countWildCard(){
	int cpt = 0;
	Enumeration enum = path.elements();
	TagAttributes elem = null;
	while (enum.hasMoreElements()){
	    elem = (TagAttributes) enum.nextElement();
	    if (elem.isWildCard())
		cpt ++;
	}
       return cpt;
    }
  
    /** Find the target-element in the pattern */
    protected TagAttributes findTargetElement(){
	Enumeration enum = path.elements();
	TagAttributes elem = null;
	while (enum.hasMoreElements()){
	    elem = (TagAttributes) enum.nextElement();
	    if (elem.isTargetElement())
		return elem;
	}
	return null;   
    }

    /** Accessor. 
     *  @return the target Element of the Pattern */    
    public TagAttributes getTargetElement(){
	return targetElement;
    }
    /** Accessor. 
     *  @return the number of WildCards in the Pattern */
    public int getWildCardsCount(){
	return hasWildCard;
    }
    /** Accessor. 
     *	@return <b>true</b> if the pattern has WildCards */ 
    public boolean hasWildCards(){
	return (hasWildCard > 0);
    }
    /** Accessor. 
     *  @return the Vector of the Pattern filled with TagAtrributes */
    public Vector getPath(){
	return path;
    }
    /** Accessor. 
     *  @return the priority of the Pattern */
    public int getPriority(){
	return priority;
    }
    /** Compare priorities
     *  @param x an integer
     */
    public int comparePriority(int x){
	if (priority == x){
	    return EQUAL;
	}else if (priority < x){
	    return LOWER;
	}else{
	    return GREATER;
	}
    }
    /** Equals.
     */
    public boolean equals(Object e){
	String tmp = make();
	String tmp2;
	try {
	    tmp2 = ((Pattern)e).make();
	}catch (ClassCastException ex){
	    ex.printStackTrace();
	    return false;
	}
	return (tmp.equals(tmp2));
    }
    /** HashCode.
     */
    public int hashCode(){
	String tmp = make();
	if (tmp != null) 
	    tmp += tmp.length();
	return tmp.hashCode();
    }
    /** Make the HashCode */
    private String make(){      
        // REMPLACER LE TOUT PAR UN SIMPLE TO STRING
        String str = new String();
        Enumeration enum = path.elements();
        if (priority != 0)
            str += "?" + priority + "?";
        while (enum.hasMoreElements()){
            TagAttributes elem = (TagAttributes)enum.nextElement();        
            str += "*";
            if (elem.isTargetElement()){
                str += "%";     
            }
            if (elem.isWildCard()){
                str += "~" + elem.getSourcePattern();
            }else{
                str += elem.getTag();
            }
            Vector  att = elem.getAttributes();
            if (att != null){
                Enumeration attenum = att.elements();
                while (attenum.hasMoreElements()){               
                    Attribute el = (Attribute) attenum.nextElement();
                    str += ":" + el.getName();
                    str += "^" + el.hasValue();
                    if (el.hasValue())
                     str += "=" + el.getValue();
                }
            }
        }
        return str;
    }

    /** Make a Pattern from a DOM Xsl:Select */
    protected static Pattern fromDOMXslSelect(String pattern,XSLErrorHandler error){
	myVector v = new myVector();
	
	v = fromDOMTemplateToTagAttribute(pattern,v,new myVector(),error);	
	if (v.size() > 0){
	    Pattern tmp = new Pattern(v,error);
	    if (tmp.isWellFormedSelectPattern())
		return  tmp;
	    else{ 
		err.fatalError("A Select/Value Pattern is not well formed.\n\t" + pattern);
		return null;
	    }
	}else{
	    return null;
	}
    }

    /** Make a Pattern from a DOM XML Element*/
    public static Pattern fromDOMXml(Element n,Vector v,XSLErrorHandler error){
	String TagName = n.getTagName();
	int type = 0;
	v.addElement(new TagAttributes(TagName,XSLTags.TARGETELEMENT_PATTERN,null));
	return new Pattern(v,error);
    }   
    /** Make a Pattern from a DOM Template */
    public static myVector fromDOMTemplate(String pattern,myVector mv,XSLErrorHandler error){
	myVector v = new myVector();
	int or = pattern.indexOf(XSLTags.OR_SEP);
	if (pattern == null){
	    return null;
	}else if (or != -1){
	    // On a trouve un OU
	    String patt1    = pattern.substring(0,or);
	    String pattNext = pattern.substring(or+XSLTags.OR_SEP.length());

	    fromDOMTemplate(patt1,mv,error);
	    fromDOMTemplate(pattNext,mv,error);
	    return mv;
	}else if (pattern.equals(XSLTags.ROOT_PATTERN0)){
	    mv.addElement(ROOTTEMPLATE());
	    return mv;
	}else if (pattern.equals(XSLTags.ELEMENT_PATTERN0)){
	    mv.addElement(DEFAULTTEMPLATE());
	    return mv;
	}else{
	    myVector children = new myVector();	   
	    v = fromDOMTemplateToTagAttribute(pattern,v,children,error);
	    v.merge(children,1);
	}
	
	if (v.size() != 0){
	    Pattern tmp = new Pattern(v,error);
	    if (tmp.isWellFormedTemplatePattern()){
		mv.addElement(tmp);
		return  mv;
	    }else{ 
		err.fatalError("A Template Pattern is not well formed.\n\t" + pattern);
		return null;
	    }
	}else 
	    return null;	  	
    }
    /** Make a Pattern from a DOM Template */
    public static myVector fromDOMTemplate(Element n,XSLErrorHandler error){
	myVector v = new myVector();
	String pattern = n.getAttribute(XSLTags.PATTERN);	
	v = fromDOMTemplate(pattern,v,error);
	return v;
    } 

    /** Make a pattern from DOMTemplate, works only on the qualifiers of a pattern */ 
    private static void fromDOMTemplateToQualifier(String pattern,Vector attributs,myVector children,XSLErrorHandler error){       
	int pO = pattern.indexOf(XSLTags.ATTRIBUTE_OP);
	int pC = pattern.indexOf(XSLTags.ATTRIBUTE_CP);

	if (pO != -1){
	    String qualifier = pattern.substring(0,pO);

	    if (qualifier.equals(XSLTags.ATTRIBUTE_QUALIFIER)){
		// attribute(toto)='abc' or attribute(titi)
		int isO = pattern.indexOf(XSLTags.ATTRIBUTE_IS1);// Separateur '
		int isC = pattern.indexOf(XSLTags.ATTRIBUTE_CP);	       
		int end;

		if (isO == -1){
		    isO = pattern.indexOf(XSLTags.ATTRIBUTE_IS2);//separateur "
		    end = pattern.lastIndexOf(XSLTags.ATTRIBUTE_IS_SEP2); //separateur "
		}else{
		    end = pattern.lastIndexOf(XSLTags.ATTRIBUTE_IS_SEP1); // Separateur '
		}
				
		if ((isO < isC) && (isO != -1)){
		    err.fatalError("Error: malformed pattern " + pattern +
				   "\t='...' must be outside the parenthesis.");
		}
		if (isO != -1){
		    String option    =
			option = pattern.substring(pO+1,isO-1);
		    String value;
		    if (end == -1) {
			value = null;
			err.fatalError("Error: malformed pattern " + pattern);
		    }else{
			value = 
			    pattern.substring(isO+XSLTags.ATTRIBUTE_IS1.length(),end); // IS1 and IS2 sont le la meme longueur.
		    }
		    Attribute att = new Attribute(option,true,value);
		    attributs.addElement(att);		    
		}else{
		     String option    = pattern.substring(pO+1,pC); 
		     Attribute att = new Attribute(option,true,null);
		     attributs.addElement(att);
		}
	    }else{
		err.warning("Warning, unknown qualifier name : " + qualifier);
	    }
	}else{ // it's a child [toto,...]	    	    
	    int virgule = pattern.indexOf(XSLTags.QUALIFIER_SEP);
	    int cC = pattern.indexOf(XSLTags.QUALIFIER_CP);
	    String childName = null;
	    if (virgule != -1){
		childName = pattern.substring(0,virgule);
	    }else{
		childName = pattern;
	    }
	    TagAttributes tAtt = new TagAttributes(childName,XSLTags.CHILD_PATTERN,null);
	    children.addElement(tAtt);
	}
	int virgule = pattern.indexOf(XSLTags.QUALIFIER_SEP);
	if (virgule != -1){
	    pattern = pattern.substring(virgule+XSLTags.QUALIFIER_SEP.length());
	    fromDOMTemplateToQualifier(pattern,attributs,children,error);       
	}
    }

    /** Makes a TagAttributes from an element of the pattern.
     *  (i.e toto[attribute(titi),child(ttof)] --> TagAttributes) 
     */
    private static TagAttributes extractPattern(String pattern,myVector v,myVector children,Vector attributs,String tag,XSLErrorHandler error){
	int attribut = pattern.indexOf(XSLTags.QUALIFIER_OP);    // []
	int ancestor = pattern.indexOf(XSLTags.ANCESTOR+XSLTags.ANCESTOR_OSEP);
	int parent = pattern.indexOf(XSLTags.PARENT_ELEMENT);
	int attribute = pattern.indexOf(XSLTags.ATTRIBUTE_QUALIFIER+XSLTags.ATTRIBUTE_OP); // /attribute(toto)
	int virgule = pattern.indexOf(XSLTags.QUALIFIER_SEP);
	int type;
	if (attribut != -1){// Creer le vector d'attributes
	    int attribut_end = pattern.indexOf(XSLTags.QUALIFIER_CP);
	    String attrib = pattern.substring(attribut+1,attribut_end);
	    fromDOMTemplateToQualifier(attrib,attributs,children,error);
	    
	    tag = pattern.substring(0,attribut);
	    type = XSLTags.ELEMENT_PATTERN;
	}else if(ancestor != -1){// Creer l'ancestor
	    if (virgule != -1){
		err.warning("separator ',' found in ancestor pattern \"" + pattern + "\"");
	    }
	    int end_ancestor = pattern.indexOf(XSLTags.ANCESTOR_CSEP);
	    String ancestorTag = pattern.substring((XSLTags.ANCESTOR+XSLTags.ANCESTOR_OSEP).length(),end_ancestor);
	    
	    tag = ancestorTag;	
	    type = XSLTags.ANCESTOR_PATTERN;
	}else if(parent != -1){
	    // Parent Element ".."
	    tag = "";
	    if (virgule != -1){
		err.warning("separator ',' found in pattern \"" + pattern + "\"");
	    }
	    type = XSLTags.PARENT_PATTERN;
	}else if (attribute != -1){
	    if (virgule != -1){
		err.warning("separator ',' found in pattern \"" + pattern + "\"");
	    }
	    int endAtt = pattern.indexOf(XSLTags.ATTRIBUTE_CP);
	    String attributeName = pattern.substring((attribute + (XSLTags.ATTRIBUTE_QUALIFIER).length() + XSLTags.ATTRIBUTE_OP.length()),endAtt);
	    tag = attributeName;
	    type = XSLTags.ATTRIBUTE_PATTERN;
	}else {// Creer le tag
	    tag = pattern;
	    type = XSLTags.ELEMENT_PATTERN;
	    if (virgule != -1){
		err.warning("separator ',' found in pattern \"" + pattern + "\"");
	    }
	}     
	if (!tag.equals(XSLTags.CURRENT_ELEMENT)){
	    // Tester si c'est une *, On en fait une WildCard
	    if (tag.equals(XSLTags.ELEMENT_PATTERN0)){
		tag = XSLTags.NULL_ELEMENT;
	    }else if (pattern.equals(XSLTags.ANY_SEP)){
		// ANY		    
		tag = null;
		type = XSLTags.ANY_PATTERN;
	    }		
	    return new TagAttributes(tag,type,attributs);
	}else 	    
	    // On a un . (C-A-D un Current_Element) dans le pattern, on l'oublie
	    return null;
    }
    /** Makes a TagAttribute from a DOMTemplate */
    private static myVector fromDOMTemplateToTagAttribute(String pattern,myVector v,myVector children,XSLErrorHandler error){
	//   item[attribute(toto='abc'),attribute(titi),child(aa,bb)]
	//or item[attribute(toto='abc',titi),child(aa),child(bb)]   
	Vector attributs = new Vector();
	
	String tag = null;
	int type;
	if (!pattern.equals("")){
	    int or = pattern.indexOf(XSLTags.OR_SEP);
	    if (or > 0){
		err.warning("Or not yet implemented! La question est de savoir si on a le droit de mettre | ici.");
	    }
	    
	    int sep = pattern.indexOf(XSLTags.SEP);
	    int any_sep = pattern.indexOf(XSLTags.ANY_SEP);
	    // On est sur le dernier element ou en mileu de pattern
	    String substring;
	    if (sep != -1){
		if (any_sep != 0){
		    substring = pattern.substring(0,sep);
		}else{
		    substring = pattern.substring(0,XSLTags.ANY_SEP.length());
		}
	    }else 
		substring = pattern;
	    TagAttributes tA = extractPattern(substring,v,children,attributs,tag,error);
	    if (tA != null){
		// On a un element simple (peut-etre une *)		
		if ((sep == -1) && (tA.getSourcePattern() != XSLTags.ATTRIBUTE_PATTERN)){
		    tA.setSourcePattern(XSLTags.TARGETELEMENT_PATTERN);
		}
		// On ajoute l'element au Pattern
		v.addElement(tA);
	    }
	    if (sep != -1){
		// On est en milieu de pattern	    
		int pas = 0;
		if (any_sep == -1)
		    pas = 1;
		else if (any_sep == 0)
		    pas = 2;
		else if (any_sep > substring.length())
		    pas = 1;
		
		String prochain = pattern.substring(sep+pas,pattern.length());		
		fromDOMTemplateToTagAttribute(prochain,v,children,error);
	    }
	}
	return v;
    }

    /** Root Template.
     *  A Faster meaning to get the Root-template.
     *  @return a Pattern with the Root-template
    */
    public static Pattern ROOTTEMPLATE(){
	Vector v = new Vector();
	v.addElement(new TagAttributes(null,XSLTags.ROOT_PATTERN,null));
	return new Pattern(v,err);       
    }
    /** Default Template
     *  A Faster meaning to get the Default-template (i.e: "*")
     *  @return a Pattern with the Default-template
    */
    public static Pattern DEFAULTTEMPLATE(){
	Vector v = new Vector();
	v.addElement(new TagAttributes(XSLTags.NULL_ELEMENT,XSLTags.TARGETELEMENT_PATTERN,null));
	Pattern t = new Pattern(v,err);
	return t;       
    }
    
    /** Compatible.
     *  Tests if the pattern matches the <i>XML DOM Element</i>
     */    
    public boolean compatible(Element node){
	if (targetElement != null){
	    if (targetElement.compatible(node)){
		return pathCompatible(node);
	    }
	}
	return false;
    }

    /** Path Compatible.
     *  Tests if the pattern matches the <i>XML DOM Element</i>
    */
    private boolean pathCompatible(Element node){
	if (path == null){
	    return true;
	}else{
	    i = path.indexOf(targetElement);	  
	    int size = path.size();       
	    int cpt = i + 1;
	    if (cpt < size){
		TagAttributes tmp = (TagAttributes)path.elementAt(cpt);					       
		// On va apres les childs du target Element
		while ((cpt < size) && (tmp.getSourcePattern() == XSLTags.CHILD_PATTERN)){
		    tmp = (TagAttributes)path.elementAt(cpt++);					       
		}
	    }
	    cpt = cpt - 1;
	    Node parent = node;
	    if (( cpt != i ) && ( i == 0)){
		i = cpt;
		return pathComp(node,path,cpt);
	    }else{
		// i est la longueur du Pattern 
		// si elle est inferieure ou egale a 1, il n'y a qu'un
		// element dedans, donc on l'a deja teste en "compatible"
		return pathBeforeTargetElementComp(parent,path);
	    }
	}
    }
    /** pathBeforeTargetElementComp.
     *  Tests if the elements of the pattern before the Target Element matches the <i>XML DOM Element</i>
     */
    private boolean pathBeforeTargetElementComp(Node parent,Vector path){
	boolean flag = pathComp(parent,path,i);
	while ((i > 0) && (flag)){
	    i--;
	    parent = parent.getParentNode();
	
	    flag = pathComp(parent,path,i);
	}
	return flag;
    }	    
    /** pathComp.
     *  Tests if the TagAttribute at <b>j</b> of the pattern matches the <i>XML DOM Element <b>parent</b></i>
     */
    private boolean pathComp(Node parent, Vector path, int j){
	if (j < 0){
	    return true;
	}else if ((parent.getNodeType() == Node.DOCUMENT_NODE) && (j == 0)){
	   return true;
	}else if (parent.getNodeType() == Node.DOCUMENT_NODE){
	    return false; 
	}else if (parent.getNodeType() == Node.ELEMENT_NODE){
	    Element element = (Element)parent;
	    String tag = element.getTagName();
	    TagAttributes tA = (TagAttributes)path.elementAt(j); 
	    if ((tA.isWildCard()) && (tA.getSourcePattern() == XSLTags.ELEMENT_PATTERN)){
		// <element/>
		Node node = element.getParentNode();
		if (element.getNodeType() == Node.ELEMENT_NODE){
		    return (tA.attributesCompatibles(element) && (pathComp(node,path,--i)));
		}else{
		    return true;
		}
	    }else if (tA.isWildCard()){	      
		// <any> on recherche si l'element qui precede dans le pattern est dans 
		//       l'XML
		return anyCompatible(parent,path,j);
	    }else if (tA.isParentElement()){
		return parentCompatible(parent,path,j); // Voir si on a besoin de tous ses params
	    }else if (tA.getSourcePattern() == XSLTags.CHILD_PATTERN){
		NodeList nodeI = element.getElementsByTagName(tA.getTag());
		// Retourne vrai si on a trouve un child nomme tA.getTag()
		// Et si le parent matche aussi
		i = j - 1;
		return ((nodeI.getLength() != 0) && (pathComp(parent,path,i)));
	    }else{
		return (tA.compatible(element));
	    }
	}else{ 
	    err.fatalError("%%%%%%%%%%%%%%%%%Should never happend: Pattern: l 498%%%%%%%%%%%%%%%%%%%%%%%");
	    return true;
	}
    }

    

    /** Parent Compatible.
     *  Tests if a Parent-TagAttributes could match <i>XML DOM Element</i>
     *  In fact, test if the element in the <i>XML DOM Element</i> matches the previous TagAttributes
     */
    protected boolean parentCompatible(Node node,Vector path, int j){
	if ((node.getNodeType() == Node.DOCUMENT_NODE) && (j == 0)){	    
	    err.warning("Parent in first position");
	    i = j;
	    return true;
	}else{
	    i = j - 1;
	    return true;
	}
    }    /** Any Compatible.
     *  Tests if an Any-TagAttributes could match <i>XML DOM Element</i>
     *  In fact, search for an element in the <i>XML DOM Element</i> matches the previous TagAttributes
     */
    protected boolean anyCompatible(Node node,Vector path, int j){
	if (j < 1){
	    i = j;
	    return true; // A VOIR
	    // Car le any est alors le pattern 0
	}else if ((node.getNodeType() == Node.DOCUMENT_NODE) && (j == 0)){	    
	    i = j;
	    return true;
	}else if (node.getNodeType() == Node.DOCUMENT_NODE){
	    i = j;
	    return false; 
	}else if (node.getNodeType() == Node.ELEMENT_NODE){
	    Element element = (Element)node;
	    TagAttributes tA1 = (TagAttributes)path.elementAt(j); // C'est le <any>
	    TagAttributes tA0 = (TagAttributes)path.elementAt(j-1); // C'est son parent
	    if ((tA0.isWildCard()) && (tA0.getSourcePattern() == XSLTags.ELEMENT_PATTERN)){
		//Si le parent de <any> est un <element>(WildCard), alors il faut verifier
		//Si le parent de <element> matche lui aussi
		Node n = element.getParentNode();
		if (n.getNodeType() == Node.DOCUMENT_NODE){
		    return (tA0.compatible(element));
		}else if (n.getNodeType() == Node.ELEMENT_NODE){
		    Element e = (Element)n;
		    if (j == 1){
			//Le parent est a la racine donc on verifie si le parent matche simplement
			return (tA0.compatible(element));
		    }else{
			TagAttributes tA = (TagAttributes)path.elementAt(j-2); // C'est son parent
			if (tA.isWildCard() && (tA.getSourcePattern() == XSLTags.ELEMENT_PATTERN)){
			    err.warning("Too much wildcard in this pattern.");
			    return tA.compatible(e);
			}else if ((tA.compatible(e)) && (tA0.compatible(element))){ 
			    i = j - 1;
			    return true;			    
			}else if (tA.compatible(e)){ 
			    return false;
			}else if (tA0.compatible(e)){ 
			    return anyCompatible(node.getParentNode(),path,j);
			}else{
			    return false;
			}
		    }
		}else{
		    err.fatalError("anyCompatible: Ni un DOC ni un ELEMENT");
		    return true;
		}
	    }if (tA0.compatible(element)){
		// On a trouve un element qui matche le parent d'<any>,
		// On s'arrete
		i = j - 1;
		return true;
	    }else if (tA1.compatible(element)){
		// L'element matche <any> mais pas le parent, donc on remonte encore
		return anyCompatible(node.getParentNode(),path,j);
	    }else{
		// L'element ne matche ni <any>, ni sont parent, on renvoie false
		return false;
	    }
	}else{
	    err.fatalError("anyCompatible" + node + "ni Node.DOC, ni Node.ELEMENT_NODE");
	    return false;
	}
    }
    /** Comparison of 2 patterns.
     * <ul>
     *  <li>First, all matching template rules that are less important than the most 
     *    important matching template rule or rules are eliminated from consideration.<br>
     *  <li>Next, all matching template rules that are less specific than the most
     *    specific matching template rule or rules are eliminated from consideration.
     *    When a source element is matched against multiple patterns, it is possible 
     *    for it to match more than one distinct pattern. <br>In
     *    this situation, XSL defines which pattern or patterns are the most specific.
     *    A pattern that starts with an IdAnchor is more specific than a pattern that does not. 
     *    If two patterns both start with an IdAnchor or both do not start with an IdAnchor, 
     *    then the one with the more components is the more specific, where a
     *    component is either a Qualifier or a OneElementTypePattern.<br>
     *  <li>Next, all matching template rules that have a lower priority than the matching 
     *    template rule or rules with the highest priority are eliminated from consideration. 
     *    The priority of a rule is specified by the priority attribute on the rule.<br></ul>
     *  @see XSLSpec
     */
    public int compareTo(Pattern other){
    /* Comparaison de 2 pattern, selon les regles de conflits edictes dans la norme XSL<BR>
     *  1 - verification de l'attribut important!, en faisant attention au fait qu'une regle importee
     *      est toujours moins importante qu'une regle normale
     *  2 - Verification de la presence d'un attribut ID 
     *  3 - Comptage du nombre de components de la regle:
     *      C'est-a-dire du nombre de Qualifiers et de /
     *  4 - Verification de l'attribut priority, et de sa valeur (par defaut = 0)
     */
	// D'abord verification des important! A FAIRE
	// Puis verification de l'attribut ID  A FAIRE
	// Ensuite on verifie le nombre de Components
	int verifComponents = compareNbComponent(other);	
	if (verifComponents != EQUAL){
	    return verifComponents;
	}else{
	    // Verification du nombre d'ANY
	    int verifAnyElement = compareNbAnyElement(other);
	    if (verifAnyElement != EQUAL){
		return verifAnyElement;
	    }else{
		// Verification du nombre d'*
		int verifNullElement = compareNbNullElement(other);
		if (verifNullElement != EQUAL){
		    return verifNullElement;
		}else
		    // Verification de l'attribut priority
		    return comparePriority(other);
	    }
	}
    }

    /** Result of a comparison */
    public static final int GREATER =  1;
    /** Result of a comparison */
    public static final int EQUAL   =  0;
    /** Result of a comparison */
    public static final int LOWER   = -1;
    
    /** Compare priorities
     *  @param x a pattern
     */
    protected int comparePriority(Pattern other){
	if (priority == other.priority){
	    return EQUAL;
	}else if (priority < other.priority){
	    return LOWER;
	}else{
	    return GREATER;
	}
    }
    /** Compare the number of "Any" in two Patterns
     */
    protected int compareNbAnyElement(Pattern other){
	int nb = countAnyElement();
	int nbOther = other.countAnyElement();
	if (nb == nbOther){
	    return EQUAL;
	}else if (nb > nbOther){
	    return LOWER;
	}else{
	    return GREATER;
	}
    }
    
    /** Compare the number of "*" in two patterns*/
    protected int compareNbNullElement(Pattern other){
	int nb = countNullElement();
	int nbOther = other.countNullElement();
	if (nb == nbOther){
	    return EQUAL;
	}else if (nb > nbOther){
	    return LOWER;
	}else{
	    return GREATER;
	}
    }
    /** Compare the number of Components in 2 patterns
     */
    protected int compareNbComponent(Pattern other){
	float nbComponents = countComponents();
	float other_nbComponents = other.countComponents(); 
	if (nbComponents < other_nbComponents)
	    return LOWER;
	else if(nbComponents == other_nbComponents)
	    return EQUAL;
	else 
	    return GREATER;
    }

    /** Count the number of Components in this pattern */
    protected float countComponents(){
	if (nbComponents != 0)
	    return nbComponents;
	else {
	    float count = 0;
	    Enumeration enum = path.elements();
	    TagAttributes element;
	    while (enum.hasMoreElements()){
		element = (TagAttributes)enum.nextElement();
		if (!element.isWildCard()){		    
		    count += 1 + element.getAttributesCount();
		}
	    }
	    // On fixe la valeur pour ne plus a la recalculer la prochaine fois
	    nbComponents = count;
	    return count;
	}
    }
    /** Count the number of ANY in this pattern.
     */
    protected int countAnyElement(){
	if (nbANYElement != 0)
	    return nbANYElement;
	else {
	    int count = 0;
	    Enumeration enum = path.elements();
	    TagAttributes element;
	    while (enum.hasMoreElements()){
		element = (TagAttributes)enum.nextElement();
		if (element.isAnyElement()){		    
		    count++;
		}
	    }
	    // On fixe la valeur pour ne plus a la recalculer la prochaine fois
	    nbANYElement= count;
	    return count;
	}
    }
    /** Count the number of "*" in this pattern.
     */
    protected int countNullElement(){
	if (nbNULLElement != 0)
	    return nbNULLElement;
	else {
	    int count = 0;
	    Enumeration enum = path.elements();
	    TagAttributes element;
	    while (enum.hasMoreElements()){
		element = (TagAttributes)enum.nextElement();
		if (element.isNullElement()){		    
		    count++;
		}
	    }
	    // On fixe la valeur pour ne plus a la recalculer la prochaine fois
	    nbNULLElement= count;
	    return count;
	}
    }
    // Private
    private float nbComponents = 0;
    private int nbANYElement = 0;
    private int nbNULLElement = 0;

    /** Tests if the Pattern is a Well Formed XSL Template. 
     */
    public boolean isWellFormedTemplatePattern(){
	if (targetElement == null){
	    return false;
	}else{
	    Enumeration enum = path.elements();
	    TagAttributes tA = null;
	    while (enum.hasMoreElements()){
		tA = (TagAttributes)enum.nextElement();
		if (tA.getSourcePattern() == XSLTags.ANCESTOR_PATTERN){
		    // Il n'y a pas d'ancestor dans une template 
		    return false;
		}
	    }
	}
	return true;
    }
    /** Tests if the Pattern is a Well Formed XSL Process Select. 
     */
    public boolean isWellFormedSelectPattern(){
	if (targetElement == null){
	    return false;	
	}else{
	    TagAttributes tmp = (TagAttributes)path.firstElement();	    
	    if (tmp.isAnyElement()){
		return false;
	    }else{
		Enumeration enum = path.elements();
		TagAttributes tA = null;
		while (enum.hasMoreElements()){
		    tA = (TagAttributes)enum.nextElement();
		    if (tA.isNullElement()){
			// Il n'y a pas d'* dans un select Pattern
			return false;
		    }
		}
	    }
	    return true;
	}
    }
}
