package fr.dyade.koala.xml.xsl;
import org.w3c.dom.*;
import java.util.Vector;
import java.util.Enumeration;
/** TagAttributes.java
 */
/** This class defines an Element in the document.
 *  @author Jrmy CALLES
 *  @version 0.7b1
 *  @see fr.dyade.koala.xml.xsl.XSLTags
 *  @see fr.dyade.koala.xml.xsl.Attribute
 */
public class TagAttributes {
    /** The Tag Name */
    protected String tag;
    /** the type of the pattern that have call this tag.
     *  @see fr.dyade.koala.xml.xsl.XSLTags
     */
    protected int source_pattern;
    /** The list of attributes of the tag */
    protected Vector attributes;
    /** Initializer
     * @param tag            the name of the tag
     * @param source_pattern the type of the pattern that have call this tag.
     * @param attributes     the attributes of the tag
     */
    public TagAttributes(String tag,int source_pattern,Vector attributes){
	this.tag = tag;
	this.source_pattern = source_pattern;
	this.attributes = attributes;
    }
    
    /** toString */
    public String toString(){
	String tmp = "";
	if (source_pattern == XSLTags.ROOT_PATTERN){
	    return XSLTags.SEP;
	}else if (source_pattern == XSLTags.ANY_PATTERN){
	    return XSLTags.SEP;
	}else if (source_pattern == XSLTags.ANCESTOR_PATTERN){
	    return XSLTags.ANCESTOR + XSLTags.ATTRIBUTE_OP + tag + XSLTags.ATTRIBUTE_CP;
	}else if (source_pattern != XSLTags.CHILD_PATTERN){
	    if (attributes != null){
		boolean flg = false;
		Enumeration enum = attributes.elements();
		if (enum.hasMoreElements()){
		    tmp = XSLTags.QUALIFIER_OP;
		    flg =true;
		}
		while (enum.hasMoreElements()){
		    Attribute att = (Attribute)enum.nextElement();
		    tmp += att.toString() + XSLTags.QUALIFIER_SEP;
		}
		if (flg == true){
		    tmp += XSLTags.QUALIFIER_CP;
		}
	    }
	    if (!isNullElement()) 
		return tag + tmp;
	    else 
		return XSLTags.ELEMENT_PATTERN0;
	}else{
	    return XSLTags.CHILD_QUALIFIER + XSLTags.ATTRIBUTE_OP + tag + XSLTags.ATTRIBUTE_CP;
	}
    }
    
    /** Tag Name's accessor */
    public String getTag(){
	return tag;
    }
    /** Type of pattern's accessor @see XSLTags.ELEMENT_PATTERN*/
    public int getSourcePattern(){
	return source_pattern;
    }
    /** Type of pattern's modifier @see XSLTags.ELEMENT_PATTERN*/
    public void setSourcePattern(int i){
        source_pattern = i;
    }

    /** List of Attributes' accessor */
    public Vector getAttributes(){
	return attributes;
    }
    /** Count Attributes */
    public float getAttributesCount(){
	if (attributes == null)
	    return 0;
	else{
	    float tmp = 0;
	    Enumeration enum = attributes.elements();
	    while (enum.hasMoreElements()){
		Attribute att = (Attribute) enum.nextElement();
		tmp += att.getAttributeCount();
	    }
	    return tmp;
	}
    }

    /** get One Attribute.
     *  @param name the name of the attribute to get */
    public Attribute getAttribute(String name){
	if (attributes != null){
	    Enumeration enum = attributes.elements();
	    Attribute elem = null;
	    while (enum.hasMoreElements()){
		elem = (Attribute)enum.nextElement();
		if (name.equals(elem.getName())){
		    return elem;
		}
	    }
	    return null;
	}else{
	    return null;
	}
    }
    /** Add an Attribute.
     *  @param att the attribute to add */
    public void addAttribute(Attribute att){
	attributes.addElement(att);
    }
    /** Tests if <b>this</b> is a Parent Element.<br>
     *  (if it is an ".." in the xsl Pattern)
     */
    public boolean isParentElement(){
	return (source_pattern == XSLTags.PARENT_PATTERN);
    }  
    /** Tests if <b>this</b> is a WildCard.<br>
     *  (if it is an "*" or an "//" in the xsl Pattern)
     */
    public boolean isWildCard(){
	//return ((tag == null) || (tag.equals(XSLTags.NULL_ELEMENT)));
	return ((source_pattern == XSLTags.ANY_PATTERN) ||
		(tag == null) ||
		(tag.equals(XSLTags.NULL_ELEMENT)));
    } 
    /** Tests if <b>this</b> is a Null Element<br>(i.e: an "*" in Xsl)
     */
    public boolean isNullElement(){
	return ((tag != null) &&  (tag.equals(XSLTags.NULL_ELEMENT)));
    }
    /** Tests if <b>this</b> is an Any Element<br>(i.e "//" in Xsl)
     */
    public boolean isAnyElement(){
	return ((tag == null) && (source_pattern == XSLTags.ANY_PATTERN));
    }
    /** Tests if <b>this</b> is the target of the pattern.(the last element of the pattern)
     */
    public boolean isTargetElement(){
	return ((tag != null) && ((source_pattern == XSLTags.TARGETELEMENT_PATTERN) || 
				  (source_pattern == XSLTags.ATTRIBUTE_PATTERN)));
    }
    /** Compatible:<br>
     *  @return return <i>true</i> if the XML DOM Element has the same name and has
     *  compatibles attributes. If <b>this</b> is a WildCard, only tests
     *  if attributes are compatibles.
     */
    public boolean compatible(Element node){
	if (isWildCard()){
	    return attributesCompatibles(node);
	}else if (tag.equals(node.getTagName())){
	    return  attributesCompatibles(node);
	}else{
	    return false;
	}
    }
    /** Compatible:<br>
     *  @return return <i>true</i> if every attributes are compatibles to XML DOM Element.
     */
    public boolean attributesCompatibles(Element node){
	boolean tmp = true;
	NamedNodeMap nodeI = node.getAttributes();
	if (attributes == null){
	    return true;
	}else if (attributes.size() == 0){
	    return true;	   
	}else if(nodeI == null){
	    return false;
	}else if (attributes.size() > nodeI.getLength()){
	    return false;
	}else{
	    Enumeration enum = attributes.elements();
	    Attribute att;
	    while (enum.hasMoreElements()){
		att = (Attribute) enum.nextElement();
		if (!att.compatible(node))
		    return false;		    
	    }
	    return true;
	}
    }
}
