package fr.dyade.koala.xml.xsl;
/** XslEval.java 
 */
import org.w3c.dom.*;
import org.xml.sax.*;
import java.util.*;
import java.io.*;
import java.net.*;
import fr.dyade.koala.xml.xsl.util.*;
import fr.dyade.koala.xml.xsl.exception.*;
/** This class apply an Xsl stylesheet to an Xml Document
 *
 *  @see fr.dyade.koala.xml.xsl.XSLTags
 *  @see fr.dyade.koala.xml.xsl.Pattern
 *  @version 0.7b1
 *  @author  Jeremy CALLES
 */
public final class XslEval {  
    
    /** The BootStrap for anyDOM Implementation */
    protected DOMParse dom;
    /** The Handler for the Output */
    protected OutputDocumentHandler out;
    /** The PrintWriter */
    protected OutputStream printWriter = null;
    /** The Error Handler */
    protected XSLErrorHandler err;
    /** the Xsl Document to parse */
    protected Document Xsl;
    /** the Xml Document to parse */
    protected Document Xml;
    /** The Current DOM Object Node within the XML Document */
    protected Element curXML;
    /** The Current DOM Object Node within the XSL Document */ 
    protected Element curXSL;
    /** The Current DOM Object NodeIterator within the XML Document */
    protected NodeList curIXml;
    /** The Current DOM Object NodeIterator within the XSL Document */
    protected NodeList curIXsl;
    /** The HashTable containing the list of xsl:template commands */
    protected Hashtable htemplates;
    /** The HashTable containing the list of xsl:number elements */
    protected Hashtable numbers;
    /** The HashTable containing the list of xsl:define-constant elements */
    protected Hashtable constants;
    /** The HashTable containing the list of xsl:define-macro elements */
    protected Hashtable macros;
    /** The QSort to use with some patterns */
    private QSortPattern qs ;
    /** Result NameSpaces */
    private NameSpace result_ns;
    /** 
     *  Initializer
     *  @param XML the XML Document to parse
     *  @param XSL the XSL Document to parse and apply to the <i>XML</i>document
     */
    public XslEval(Document XML, Document XSL,
		   XSLErrorHandler error) throws NotAnXslFileException{
	Xsl = XSL;
	Xml = XML;
	curXML = Xml.getDocumentElement();
	curXSL = Xsl.getDocumentElement();
	curIXml = curXML.getChildNodes();
	curIXsl = curXSL.getChildNodes();
	if (!isAnXslDocument()){
	    // Test if the file given is an XSL file
	    throw new NotAnXslFileException();
	}	    
        qs = new QSortPattern();
	dom = new DOMParse(err);
	numbers = new Hashtable();
	htemplates = new Hashtable();
	constants = new Hashtable();
	createBasicRules();
	// Handlers to use
	//	out = output;
	err = error;
        // Start processing
	Begin();
    }
    /** Only for the servlet use */
    public XslEval(Document XML, Document XSL,
		   OutputStream print, XSLErrorHandler error) throws NotAnXslFileException{
	Xsl = XSL;
	Xml = XML;
	curXML = Xml.getDocumentElement();
	curXSL = Xsl.getDocumentElement();
	curIXml = curXML.getChildNodes();
	curIXsl = curXSL.getChildNodes();
	if (!isAnXslDocument()){
	    // Test if the file given is an XSL file
	    throw new NotAnXslFileException();
	}	    
        qs = new QSortPattern();
	dom = new DOMParse(err);
	numbers = new Hashtable();
	htemplates = new Hashtable();
	constants = new Hashtable();
	createBasicRules();
	// Handlers to use
	printWriter = print; 
	err = error;	
        // Start processing
	Begin();
    }
    /** Add Basic rules to the HashTable. 
     *  Basic Rules are specified in the WD 1.0.<br>
     *  <pre>&lt;xsl:template match="*|/"><br>     &lt;xsl:process-children/><br>&lt;/xsl:template></pre>
     */
    protected void createBasicRules(){
	// Create Basic Rules
	Element elem = Xsl.createElement(XSLTags.CHILDREN);
	Vector v = new Vector();
	v.addElement(elem);
	// Put them in the HashTable
	htemplates.put(Pattern.DEFAULTTEMPLATE(),v);
	htemplates.put( Pattern.ROOTTEMPLATE(),v);
    }
    /** Initialize namespaces-result */
    protected void initResultNS(Element XSL){
	String ns = XSL.getAttribute(XSLTags.STYLESHEET_RESULT);
	String hrefns = XSL.getAttribute(XSLTags.XMLNS + ns);
	if ((hrefns == null)||(ns.equals(""))){
	    result_ns = NameSpace.getDefault();
	}else{
	    result_ns = new NameSpace(ns,hrefns,err);
	}
	out = result_ns.getOutputHandler();
	if (printWriter != null){
	    out.setOutputStream(printWriter);
	}
    }
    /** Prepare constants, rules, and macros for being processed */
    protected void initXSL(Element XSL){
	/* Differents commands in an Xsl Document */
	NodeList defined_constants,define_scripts,define_styles,define_macros,
	    templates,root_templates,style_templates,import_sheets,include_sheets;
	
	// Import XSL Files
	import_sheets    = XSL.getElementsByTagName(XSLTags.IMPORT);
	importStyleSheets(htemplates,import_sheets);
	// Include XSL Files
	include_sheets    = XSL.getElementsByTagName(XSLTags.INCLUDE);
	includeStyleSheets(htemplates,include_sheets);

	// Template rules
	templates        = XSL.getElementsByTagName(XSLTags.TEMPLATE);
	htemplates = arrangeTemplates(htemplates,templates);
	
	// Define constants
	defined_constants = XSL.getElementsByTagName(XSLTags.DEFINE_CONSTANT);
	constants = defineConstants(constants,defined_constants);

	// Define macros
	define_macros = XSL.getElementsByTagName(XSLTags.DEFINE_MACRO);
	macros = defineMacros(macros,define_macros);
	
    }

    /** Interpret the XSL StyleSheet.
     * Begin the evaluation:
     * <ol><li> scripts 
     *     <li> define-style
     *     <li> macros
     *     <li> evaluate templates
     *     <li> apply style-templates
     * </ol>
     */
    public void Begin(){
	initXSL(curXSL);
	initResultNS(curXSL);
	out.startDocument();
	doRootTemplate();
	out.endDocument();
    }

    /** Evaluate the Root Template.
     *  This is the starting point of the evaluation.
     */
    protected void doRootTemplate(){
	doFlow(curXML,getTemplate(Pattern.ROOTTEMPLATE()),false);
    }
    /** Process a DOM Element.
     *  @param e a DOM Element from the XML File to parse
     *  @param callerIsDoAttribute if <b>true</b>, return a String, otherwise write the result in 
     *  the Output Handler
     */
    protected String doElement(Node e,boolean callerIsDoAttribute){
	String text = "";
	if (e == null){
	    // Nothing to do
	}else if (e.getNodeType() == Node.TEXT_NODE){
	    text += ((org.w3c.dom.Text)e).getData();
	}else if (e.getNodeType() == Node.ATTRIBUTE_NODE){	 
	    text += ((org.w3c.dom.Attr)e).getValue();
	}else if(e.getNodeType() == Node.ELEMENT_NODE){
	    text += doFlow(e,getTemplate((org.w3c.dom.Element)e),callerIsDoAttribute);	    
	}
	if (callerIsDoAttribute){
	    return text;
	}else{
	    out.string(text);
	    return "";
	}
    }

    /** Process a DOM Element.
     *  @param e a DOM Element from the XML File to parse
     *  @param r the Vector containing the XSL actions to apply to the <i>e</i>
     *  @param callerIsDoAttribute if <b>true</b>, return a String, otherwise write the result in 
     *  the Output Handler
     */
    protected String doFlow(Node e,Vector r,boolean callerIsAttribute){
	String text = "";
	int size = 0;
	if (r != null){
	    size = r.size();
	}
	// No rule was found for this Node --> return Basic Rule
	if (size == 0){
	    r = getTemplate(Pattern.DEFAULTTEMPLATE());
	}
	if (e == null){
	    err.fatalError("doFlow has found an Error, null Exception on " + r + BUG_FIX);
	}else{
	    // Apply the rule to the XML DOM Node
	    Enumeration enum = r.elements();
	    Node node;
	    int i = 0;
	    while (enum.hasMoreElements()){
		node = (Node) enum.nextElement();
		if (e.getNodeType() == Node.TEXT_NODE){
		    text += ((Text)e).getData();
		}else{
		    if (node.getNodeType() == Node.ELEMENT_NODE){
			Element element = (Element)node;			
			if (element.getTagName().equals(XSLTags.CHILDREN)){
			    /* xsl:prcess-children */
			    NodeList nodeI = e.getChildNodes();
			    Node tmp = null;
			    int cpt = 0;
			    // process children
			    while(cpt != nodeI.getLength()){
				tmp = nodeI.item(cpt);
				text += doElement(tmp,callerIsAttribute);
				cpt++;
			    }
			}else if (element.getTagName().equals(XSLTags.TEXT)){
			    /* xsl:text */
			    text += printNode(node,callerIsAttribute);
			}else if (element.getTagName().equals(XSLTags.EMPTY)){
			    /* xsl:empty */
			    // Nothing to do
			}else if (element.getTagName().equals(XSLTags.SELECT)){
			    /* xsl:process select=".." */
			    selectedPattern(element,(Element)e);
			}else if (element.getTagName().equals(XSLTags.VALUE_OF)){
			    /* xsl:value-of */
			    valueOfPattern(element,(Element)e);
			}else if (element.getTagName().equals(XSLTags.PI)){
			    /* Processing intruction */
			    //text += "<?" + printNode(node) + "?>"; // A Voir
			}else if (element.getTagName().equals(XSLTags.COMMENT)){
			    /* Comments */
			    //text += "<!-- " + printNode(node) + " -->"; //A voir
			}else if (element.getTagName().equals(XSLTags.NUMBER)){
			    /* xsl:number */
			    addNumber(element,(Element)e);
			}else if (element.getTagName().equals(XSLTags.FOR_EACH)){
			    /* xsl:for-each */
			    forEachPattern(element,(Element)e);
			}else if (element.getTagName().equals(XSLTags.MACRO_INVOKE)){			    
			    /* xsl:invoke-macro */
			    macroInvocation(element,(Element)e);
			}else{
			    // Creation du vecteur contenant les regles a l'interieur de celle-la
			    Vector nv = getChildNodesInVector(node);
			    boolean isEmpty = ((nv == null) || (nv.size() == 0));
			    startTag(element,(Element)e, isEmpty);
			    if (!isEmpty){
				doFlow(e,nv,false);			    
				endTag(element);			    
			    }
			}
		    }else{
			if (node.getNodeType() == Node.TEXT_NODE){
			    text += ((Text)node).getData();
			}
			i++;
		    }
		}
	    }
	}
	if (callerIsAttribute){
	    return text;
	}else{
	    out.string(text);
	    return "";
	}	
    }
    
    /** Does numbering based on the position of the current node in the source tree. */
    public void addNumber(Element xsl,Element xml){
	// Trouver le pattern qui a ete matche
	String pattern = findParentPattern(xsl);
	// Trouver le from
	String from = xsl.getAttribute(XSLTags.NUMBER_FROM);
	// Trouver le level
	String level = xsl.getAttribute(XSLTags.NUMBER_LEVEL);
	// Trouver le count
	String count = xsl.getAttribute(XSLTags.NUMBER_COUNT);
	// Trouver le format
	String format = xsl.getAttribute(XSLTags.NUMBER_FORMAT);
	// LE chercher dans la HashTable des numbers 
	// Incrementer le compteur de 1
	String hash = from+count+format+level+xsl.hashCode();
	Number number = (Number)numbers.get(hash); // == hashCode de Number

	if (number == null){
	    // Pas trouve, on l'ajoute dans la HashTable
	    number = new Number(level,from,count,format,xml.getNodeName());
	}
	number.process(xml);
	numbers.put(hash,number);
	// Sortir le nombre au format desire
	out.string(number.toString());
    }
    
    /** Find the pattern that has been matched for this action */
    private String findParentPattern(Element e){
	//ATTENTION, ne fonctionne que pour les TEMPLATES
	Node parent = e.getParentNode();
	while ((parent.getNodeType() != Node.DOCUMENT_NODE)&&(!parent.getNodeName().equals(XSLTags.TEMPLATE))){	    
	    parent = parent.getParentNode();
	}
	if (parent.getNodeType() == Node.DOCUMENT_NODE)
	    return null;
	else{
	    Element element = (Element)parent;
	    return element.getAttribute(XSLTags.PATTERN);
	}
    }
    
    /** Makes a Vector from the children of a DOM Node.
     *  @param e a DOM Node
     *  @return a myVector with every children of the Node
     */ 
    protected myVector getChildNodesInVector(Node e){
	myVector v = null;
	if (e.hasChildNodes()){
	    NodeList nodeI = e.getChildNodes();
	    v = new myVector ();
	    Node node = null;
	    int cpt = 0;
	    while (cpt != nodeI.getLength()){		
		node = nodeI.item(cpt);
		v.addElement(node);
		cpt++;
	    }
	} else{
	}
	return v;
    }

    /** Print this DOM node.    
     */
    protected String printNode(Node e, boolean callerIsDoAttribute){ 
	String text = "";
	if (e.getNodeType() == Node.ATTRIBUTE_NODE){
	    text += ((org.w3c.dom.Attr)e).getValue();
	    if (callerIsDoAttribute == false){
		out.string(text);
	    }
	}else{
	    int cpt = 0;
	    NodeList children = e.getChildNodes();
	    Node child = null;
	    while (cpt != children.getLength()){
		child = children.item(cpt++);
		if  (child.getNodeType() == Node.TEXT_NODE){ 
		    text = text + ((Text)child).getData();
		    if (callerIsDoAttribute == false)
			out.string(((Text)child).getData());
		} else if (child.getNodeType() == Node.ELEMENT_NODE){ 
		    String tmp = printNode((Element)child,callerIsDoAttribute);
		    text = text + tmp;
		    if (callerIsDoAttribute == false)
			out.string(tmp);
		}
	    }
	}
	if (callerIsDoAttribute){	    
	    return text;
	}else{
	    return "";
	}
    }
    /** Process an attribute (see WD 1.0: 2.7.11.3)
     */
    protected String doAttribute(String value,Element xml){
	String text = "";
	// ATTention parser dans le sens --> si avant {} on a des trucs
        // {Constant et Attributes, le reste c'est une balise}
	             // Pas de {} --> Texte pur
	String next = null;
	int x;
	if (value == null){
	    
	}else if ((x = value.indexOf(XSLTags.CONSTANT_OBRACE)) == 0){
	    // { Rencontre
	    if ((x = value.indexOf(XSLTags.CONSTANT_BRACE)) != -1){
		// {constant( rencontre	      
		int cbrace =  value.indexOf(XSLTags.CONSTANT_CBRACE);
		if ((cbrace == -1)||(cbrace < x)){
		    err.error("unclosed or misclosed curly brace in " + XSLTags.CONSTANT + ",  value:" + value);		
		}else{
		    String constant = value.substring(x+XSLTags.CONSTANT_BRACE.length(),cbrace);
		    String val = (String)constants.get(constant);
		    if (val == null){
			err.error("Call of an undefined constant, named \"" + constant + "\"");
		    }else{
			text = val;
		    }
		    next = value.substring(cbrace+XSLTags.CONSTANT_CBRACE.length());
		}    
	    }else {
		// node rencontre 
		// Attention peut-etre un attribute dedans
		int cbrace =  value.indexOf(XSLTags.VALUE_CBRACE);
		if (cbrace == -1){
		    err.error("unclosed or misclosed curly brace in " + XSLTags.CONSTANT + ",  value:" + value);		
		}else{		    
		    String node = value.substring(XSLTags.CONSTANT_OBRACE.length(),cbrace);
		    if (node.equals(".")){		       
			text = printNode(xml,true);
		    }else if (xml.getNodeName().equals(node)){
			err.warning("unimplemented yet, {" + node + "} in node named " + node);
		    }else{
			Pattern pat = Pattern.fromDOMXslSelect(node,err);
			Vector v = findInNode(pat,xml);
			if ((v != null) || (v.size() != 0)){			   
			    text = doVector(v,true);
			}
		    }
		    next = value.substring(cbrace+XSLTags.VALUE_CBRACE.length());
		}
	    }
	}else {
	    if ((x = value.indexOf(XSLTags.CONSTANT_OBRACE)) != -1){
		text = value.substring(0,x);
		next = value.substring(x);
	    }else{
		text = value;
	    }
	}
	if (next == null){
	    // On arrive a la fin de l'attribute value
	    return text;
	}else{
	    // Il reste encore des champs dans l'attribute value
	    return text + doAttribute(next,xml);
	}
    }

    /** Return a start tag with all its attributes.
     *  @param f a DOM Element
     *  @return a String containing the Start tag and its attributes.
     */
    protected void startTag(Element xsl,Element xml,boolean isEmpty){
	NamedNodeMap att = xsl.getAttributes();
	boolean hasAtt = (!((att == null)||(att.getLength() == 0)));
	out.startElement(xsl.getTagName(),hasAtt,isEmpty);
	if (att != null){
	    int size = att.getLength();
	    org.w3c.dom.Attr node;
	    
	    if (size > 0){
		// On par a l'envers pour mettre les attributes a l'endroit
		int cpt = size - 1;
		while (cpt >= 0){
		    // ATTENTION AUX STYLES
		    node = (org.w3c.dom.Attr) att.item(cpt);
		    String tmp = doAttribute(node.getValue(),xml); 
		    out.attribute(node.getName(),tmp,(cpt==0),isEmpty);
		    cpt --;
		}
	    }
	}
    }
    /** Return an end tag.
     *  @param f a DOM Element
     */
    protected void endTag(Element f){
	out.endElement(f.getTagName());
    }
    
    /** Import StyleSheet */
    protected void importStyleSheets(Hashtable h,NodeList templates){
	Vector names = extractHrefAtt(templates);
	Enumeration enum = names.elements();
	while (enum.hasMoreElements()){
	    String elem = (String) enum.nextElement();
	    Document tmp = null;
	    try{
		URL url = dom.createURL(elem);
		tmp = dom.parseFile(url);
	    }catch (java.io.IOException e){
		err.warning("Unable to find file " + elem);
	    }catch (Exception e){
		err.fatalError("Error in file " + elem);
	    }
	    if (tmp != null)
		initXSL(tmp.getDocumentElement());
	}
    }

    /** Include StyleSheet */
    protected void includeStyleSheets(Hashtable h,NodeList templates){
	Vector names = extractHrefAtt(templates);
	Enumeration enum = names.elements();
	while (enum.hasMoreElements()){
	    String elem = (String) enum.nextElement();
	    Document tmp = null;
	    try{
		URL url = dom.createURL(elem);
		tmp = dom.parseFile(url);
	    }catch (java.io.IOException e){
		err.warning("Unable to find file " + elem);
	    }catch (Exception e){
		err.fatalError("Error in file " + elem);
	    }
	    if (tmp != null)
		initXSL(tmp.getDocumentElement());
	}
    }
    /** Extracts href attributes of Includes or Import Rules in a Vector*/
    private Vector extractHrefAtt(NodeList templates){
	int size = templates.getLength();
	int cpt = 0;
	Vector names = new Vector(size);
	Node node = null;
	while (cpt != size){
	    node = templates.item(cpt++);
	    if (node.getNodeType() == Node.ELEMENT_NODE){
		printNode(node,false);
		Element elem = (Element)node;
		String att = elem.getAttribute(XSLTags.HREF);
		names.addElement(att);
	    }else{
		err.warning("extractHrefAtt,XslEval.java found a " + node.getNodeType() +
			    " type in Include or Import rule " /*+ printNode(node)*/);
	    }
	}	
	return names;
    }
    
    /** Makes an HashTable with every defined constants
     */
    protected Hashtable defineConstants(Hashtable h, NodeList templates){
	// RAJOUTER l'IMPORTANCE
	int cpt = 0;
	Element node = null;
	while (cpt != templates.getLength()){
	    node = (Element) templates.item(cpt);
	    String name = node.getAttribute(XSLTags.CONSTANT_NAME); 
	    String value = node.getAttribute(XSLTags.CONSTANT_VALUE); 
	    if ((name == null) || (value == null)){
		err.fatalError("in " + XSLTags.DEFINE_CONSTANT + " : received null value for name or value attribute."
			       +"\t name: " + name + ", value:" + value);
	    }else{
		int obrace = value.indexOf(XSLTags.CONSTANT_OBRACE);
		int cbrace = value.indexOf(XSLTags.CONSTANT_CBRACE);
		if (h.get(name) != null){
		    err.warning("Define Constant, redefinition of a constant named = " + name);
		}
		if (obrace != -1){
		    err.warning("Define Constant, {} not yet implemented inside a constant.");
		}		
		h.put(name,value);
	    }	   
	    cpt++;
	}
	return h;
    }
    /** Makes an HashTable with every defined macros
     */
    protected Hashtable defineMacros(Hashtable h, NodeList templates){
	// RAJOUTER l'IMPORTANCE
	int cpt = 0;
	Element node = null;
	while (cpt != templates.getLength()){
	    node = (Element) templates.item(cpt);
	    String name = node.getAttribute(XSLTags.MACRO_NAME); 
	    if (name == null){
		err.fatalError("in " + XSLTags.DEFINE_MACRO + " : received null value for name attribute." +
			       "\t name: " + name);
	    }else{
		if (h.get(name) != null){
		    err.error("Define macro, redefinition of a macro named \"" + name + "\"");
		}		
		// La HashTable est remplie avec une NodeList
		h.put(name,node.getChildNodes());
	    }	   
	    cpt++;
	}
	return h;
    }

    /** Makes an HashTable to sort every template rules.
     */
    protected Hashtable arrangeTemplates(Hashtable h, NodeList templates){
	// On obtient une hashtable de vectors(contenant les Actions)
	Pattern[] arrElem = new Pattern[5];
	int cptT = 0;
	Node node = null;
	while (cptT != templates.getLength()){
	    node = templates.item(cptT++);
	    if (node.getNodeType() == Node.ELEMENT_NODE){
		NodeList childs = node.getChildNodes();
		String element = new String();
		Node cnode = null;

		/*Pattern tmp = Pattern.fromDOMTemplate((Element)node);*/
		Vector tmp = Pattern.fromDOMTemplate((Element)node,err);		
		Enumeration enum = tmp.elements();	      

		Vector v = new Vector();
		int cpt = 0;
		while (cpt != childs.getLength()){
		    cnode = childs.item(cpt++);
		    if (cnode.getNodeType() == Node.ELEMENT_NODE){
			v.addElement(cnode);
		    }
		}
		while (enum.hasMoreElements()){
		    Pattern aux = (Pattern)enum.nextElement();	
		    // On verifie si une regle egale n'existe pas deja.
		    // PENSER AUX PRIORITES DIFFERENTES
		    if ((h.get(aux) != null) && (!((aux.equals(Pattern.DEFAULTTEMPLATE())) || (aux.equals(Pattern.ROOTTEMPLATE()))))) {
			err.fatalError("Two rules matches the same pattern\n\t<" + XSLTags.TEMPLATE+ " " + aux + ">");
		    }else {
			h.put(aux,v);	
		    }
		}
	    }
	}
	return h;
    }

    /** Makes an HashTable to sort every template rules.
     *  @param templates an DOM Node Iterator containing the rules
     */ 
    protected Hashtable arrangeTemplates(NodeList templates){
	int i = templates.getLength();
	Hashtable h = new Hashtable(i);
	h = arrangeTemplates(h,templates);
	return h;
    }
    
    /** Find every templates patterns in the HashTable that could match the DOM XML Element
     *  @param e an XML DOM Element
     */
    protected Vector getMatchingTemplates(Element e){
	Vector v = new Vector();
	Enumeration enum = htemplates.keys();
	Pattern pat;
	while (enum.hasMoreElements()){
	    pat = (Pattern) enum.nextElement();
	    if (pat != null)
		if (pat.compatible(e)){
		    v.addElement(pat);
		}
	}
	//Si aucune regle ne matche la balise, on renvoie sur la regle basique <element/><children/>
	if (v.size() == 0){
	    v.addElement(Pattern.DEFAULTTEMPLATE());
	}

	return v;
    }
    /** Process an Xsl:Select pattern 
	@param xsl the DOM XSL Element containing the select rule to apply
	@param xml the DOM XML Element.
    */
    protected void  selectedPattern(Element xsl,Element xml){
	String pattern = xsl.getAttribute(XSLTags.PROCESS_SELECT);
	if (pattern != null){
	    // Traitement du Pattern
	    Pattern select = Pattern.fromDOMXslSelect(pattern,err); 
	    Vector v = findInNode(select,xml);
	    if ((v == null)||((v.size()==1)&&(v.firstElement() == null))){ 
		err.warning(XSLTags.SELECT +" "+ XSLTags.PROCESS_SELECT +"=\""+ pattern +"\" was not found in \'" +
			    xml.getTagName() + "' tag.");
	    }else{
		doVector(v,false);
	    }
	}else{
	    err.warning("null pattern in an " + XSLTags.SELECT + " command");
	}
    }
    /** Process an Xsl:invoke macro
	@param xsl the DOM XSL Element containing the invoke rule to apply
	@param xml the DOM XML Element.
    */
    protected void macroInvocation(Element xsl,Element xml){
	// A FAIRE
	String macroName = xsl.getAttribute(XSLTags.MACRO_INVOKE_MACRO);
	System.err.println("macroName: " + macroName);
	// Liste contenant les args et les actions de la macro
	NodeList list = xsl.getChildNodes();	
	// Creation d'un Vector avec les arguments  et d'un avec les Actions
	Vector args = new Vector();
	Vector actions = new Vector();
	int cpt = 0;
	while (cpt < list.getLength()){
	    Node node = list.item(cpt++);	    
	}
    }
    /** Process an Xsl:for-each pattern 
	@param xsl the DOM XSL Element containing the for-each rule to apply
	@param xml the DOM XML Element.
    */
    protected void forEachPattern(Element xsl,Element xml){
	String pattern = xsl.getAttribute(XSLTags.FOR_EACH_SELECT);
	if (pattern != null){
	    // Traitement du Pattern
	    Pattern select = Pattern.fromDOMXslSelect(pattern,err);
	    Vector v = findInNode(select,xml);
	    if ((v == null)||((v.size()==1)&&(v.firstElement() == null))){ 
		err.warning(XSLTags.FOR_EACH +" "+ XSLTags.FOR_EACH_SELECT +"=\""+ pattern +"\" was not found in \'" +
			    xml.getTagName() + "' tag.");
	    }else{
		applyActionsToVector(xsl,v);
	    }
	}else{
	    err.warning("null pattern in an " + XSLTags.FOR_EACH + " command");
	}
    }
    /** Process an Xsl:Value-Of pattern 
	@param xsl the DOM XSL Element containing the value-of rule to apply
	@param xml the DOM XML Element.
    */
    protected void valueOfPattern(Element xsl,Element xml){
	String pattern = xsl.getAttribute(XSLTags.VALUE_OF_EXPR);
	if ((pattern != null)&&(pattern.equals("."))){
	     printNode(xml,false);
	}else if (pattern != null){
	    // Traitement du Pattern
	    Pattern select = Pattern.fromDOMXslSelect(pattern,err);
	    if (select != null){
		Vector v = findInNode(select,xml);
		if ((v == null)||((v.size()==1)&&(v.firstElement() == null))){ 
		    err.warning(XSLTags.VALUE_OF +" "+ XSLTags.VALUE_OF_EXPR +"=\""+ pattern +"\" was not found in \'" +
				xml.getTagName() + "' tag.");
		}else{
		    printVector(v);
		}
	    }else{
		err.warning(XSLTags.VALUE_OF +" "+ XSLTags.VALUE_OF_EXPR +"=\""+ pattern +"\" null pattern found.");
	    }
	}else{
	    err.warning("null pattern in an " + XSLTags.VALUE_OF + " command");
	}
    }
    /** Process on a Vector
     *  @param v a vector filled with DOM Elements or Attribute
     *  @return a String
    */
    protected String doVector(Vector v, boolean callerIsDoAttribute){
	String text = new String();
	Enumeration enum = v.elements();
	Node elem = null;
	while (enum.hasMoreElements()){
	    elem = (Node) enum.nextElement();
	    if (elem != null)
		text += doElement(elem,callerIsDoAttribute);
	}
	return text;
    }

    /** Apply some actions to a Vector of XML Nodes
     *  @param node a Element that has XSL Actions' children
     *  @param xml a vector filled with XML DOM Elements or Attribute
     *  @return a String
    */
    protected void applyActionsToVector(Element node,Vector xml){
	int cpt = 0;
	NodeList list = node.getChildNodes();       
	Vector actions = new Vector(list.getLength());
	// Remplissage du Vector d'Actions pour l'appel a doFlow
	while (cpt < list.getLength()){
	    actions.addElement(list.item(cpt++));
	}
	// Parcours du Vector de Nodes XML pour l'application a doFlow
	Enumeration enum = xml.elements();
	while (enum.hasMoreElements()){
	    Node n = (Node)enum.nextElement();
	    if (n != null)
		doFlow(n,actions,false);
	}
    }

    /** Print 
     * @param v a Vector of DOM Nodes
     * @return a String filled with the DOM Nodes
     */
    protected void printVector(Vector v){
	Enumeration enum = v.elements();
	Node elem = null;
	while (enum.hasMoreElements()){	   
	    elem = (Node) enum.nextElement();
	    if (elem != null){
		printNode(elem,false);	  
	    }
	}
    }

    /** Search for a DOM Node
     *  @param select a Select Pattern containing the path to the DOM Node
     *  @param xml the DOM Element where to start the search
     *  @return a myVector of DOM Elements
     */
    protected myVector findInNode(Pattern select, Element xml){
	Vector path = select.getPath();
	Enumeration enum = path.elements();
	TagAttributes tA = (TagAttributes)enum.nextElement();
	myVector v = findInNode(tA,xml);	
	while (enum.hasMoreElements()){
	    tA = (TagAttributes)enum.nextElement();
	    // Verification si on a pas un // dans le pattern et traitement de celui-ci
	    if (tA.isWildCard()){
		TagAttributes tB = (TagAttributes)enum.nextElement();
		if (!tB.getTag().equals("")){ 
		    // Si le any est en milieu de Pattern
		    myVector tmp = findInVector(tB,v);

		    myVector aux = getChildNodesInVector(v);
		    v = tmp;
		    // On a pas trouve de child a ce niveau, on descend			
			aux = getChildNodesInVector(aux);	               
			tmp = findInVector(tB,aux);
			if (tmp.size() != 0){
			    v.merge(tmp);
			}
		}else{

		}
	    }else{
		v = findInVector(tA,v);
	    }
	}
	return v;
    }
    
    /** Search for a DOM Node
     *  @param select a TagAttribute with one DOM Element to find
     *  @param xml the DOM Element where to start the search
     *  @return a myVector of DOM Elements
     */
    protected myVector findInNode(TagAttributes tagA, Element xml){
	myVector tmp = null;
	NodeList res = null;
	String tag = tagA.getTag();
	if (tagA.getSourcePattern() == XSLTags.ANCESTOR_PATTERN){
	    /* On a rencontre un Ancestor
	       Il faut donc chercher dans les Parents Si on a pas de Tag
	       de nom de la variable "tag"
	    */
	    boolean flag = false;
	    Node node = xml;
	    while (flag == false){
		node = node.getParentNode();
		if (node.getNodeType() == Node.DOCUMENT_NODE){
		    flag = true;
		}else if (node.getNodeType() == Node.ELEMENT_NODE){
		    Element aux = (Element)node;
		    if (tag.equals(aux.getTagName())){
			flag = true;
		    }
		}
	    }

	    if (node.getNodeType() == Node.DOCUMENT_NODE){
		err.error("Ancestor not found in process command\n\t" + tag + 
			  " is not an ancestor of " + xml.getTagName()); 
		return new myVector();
	    }else{
		tmp = new myVector();
		tmp.addElement(node);
		return tmp;
	    }	    
	}else if (tag.equals(xml.getTagName())){
	    // ATTENTION VALABLE UNIQUEMENT APRES UN ANY
	    // Normalement
	    tmp = new myVector();
	    tmp.addElement(xml);
	    return tmp;
	}else if (tagA.getSourcePattern() == XSLTags.ATTRIBUTE_PATTERN){
	    // Attribute:
	    org.w3c.dom.Attr att = xml.getAttributeNode(tag);
	    tmp = new myVector();
	    tmp.addElement(att);
	}else if (tag.startsWith(XSLTags.ATTRIBUTE_QUALIFIER)){
	    // Attention, ceci n'est valable que dans un <value-of>
	    int at_op = tag.indexOf(XSLTags.ATTRIBUTE_OP);
	    int at_cp = tag.indexOf(XSLTags.ATTRIBUTE_CP);
	    if ((at_op == -1) || (at_cp == -1)){
		err.error("Error: Malformed Attribute " + tag);
	    }	    
	    String attr = tag.substring(at_op+1,at_cp);

	    org.w3c.dom.Attr att = xml.getAttributeNode(attr);

	    tmp = new myVector();
	    tmp.addElement(att);
	}else{
	    tmp = getElementsByTagNameAtFirstLevel(xml,tag);	    
	}
	return tmp;
    }

    /** Search for a DOM Node 
     *  @param select a TagAttribute with one DOM Element to find
     *  @param v a myVector filled with DOM Element where to start the search
     *  @return a myVector of DOM Elements
     */
    protected myVector findInVector(TagAttributes tagA,myVector v){
	myVector tmp = new myVector();
	Enumeration enum = v.elements();
	Element element = null;
	while (enum.hasMoreElements()){
	    element = (Element)enum.nextElement();
	    // Recherche avec findInNode()
	    myVector aux = findInNode(tagA,element);
	    if (aux != null)
		tmp.merge(aux);
	}
	return tmp;
    }

    /** Makes a myVector from the elements of a DOM NodeList
     *  @param nodeI a DOM NodeList
     *  @return a myVector with every children of each elements in the NodeList 
     */ 
    protected myVector getNodesInVector(NodeList nodeI){
	if (nodeI != null){
	    Node n = null;
	    int cpt = 0;
	    myVector tmp = new myVector();
	    while (cpt != nodeI.getLength()){
		n = nodeI.item(cpt++);
		if (n.getNodeType() == Node.ELEMENT_NODE){
		    tmp.addElement(n);
		}
	    }
	    return tmp;
	}else{
	    return new myVector();
	}
    }

    /** Makes a myVector from the children of a DOM NodeList
     *  @param nodeI a DOM NodeList
     *  @return a myVector with every children of each elements in the NodeList 
     */
    protected myVector getChildNodesInVector(NodeList nodeI){
	myVector tmp = new myVector();
	myVector vnodeI  = getNodesInVector(nodeI);
	
	Enumeration enum = vnodeI.elements();
	Element element = null;
	while (enum.hasMoreElements()){
	    element = (Element)enum.nextElement();
	    NodeList child = element.getChildNodes();
	    myVector aux = getNodesInVector(child);
	    tmp.merge(aux);
	}

	return tmp; 
    }
    /** Makes a myVector from the children of every DOM Elements of the param.
     *  @param vnodeI a myVector of DOM Elements
     *  @return a myVector with every children of every elements in <i>vnodeI</i>
     */ 
    protected myVector getChildNodesInVector(myVector vnodeI){
	myVector tmp = new myVector();
	
	Enumeration enum = vnodeI.elements();
	Element element = null;
	while (enum.hasMoreElements()){
	    element = (Element)enum.nextElement();
	    NodeList child = element.getChildNodes();
	    myVector aux = getNodesInVector(child);
	    tmp.merge(aux);
	}

	return tmp; 
    }
    /** Test. 
     *  Tests whether the XML Document given as an XSL file is really an XSL File.
     */
    private boolean isAnXslDocument(){
	return (XSLTags.STYLESHEET.equals(curXSL.getTagName()));
    }

    // A METTRE DANS UNE CLASSE DOMUtils (peut-etre)
    /** Get elements by tag name at the first Level */
    private  myVector getElementsByTagNameAtFirstLevel(Node xml,String tag){
	myVector v = new myVector();
	if (xml.hasChildNodes()){
	    Node node = xml.getFirstChild();
	    while (node != null){
		if (node.getNodeName().equals(tag)){
		    v.addElement(node);
		}		
		node = node.getNextSibling();
	    }	    
	}
	return v;
    }
  /** Get Template Rule
     *  @param n the Pattern of the Rule 
     */
    protected Vector getTemplate(Pattern n){
	return (Vector)(htemplates.get(n));
    }

    /** Get Template Rule.
     *  Find a matching XSL rule for a DOM XML Element.
     *  @param n a DOM XML Element
     */
    protected Vector getTemplate(Element n){
	Vector vtA = new Vector();
	Pattern pat = Pattern.fromDOMXml(n,vtA,err);
	Vector vNode = getTemplate(pat); 
	Vector vpat = getMatchingTemplates(n); // La, un vector de pattern

	if ((vpat.size() <= 2) && (vNode != null)){
		return vNode; // Ici on a un vector de Node
	}else{
	    if (vpat.size() == 1)	    
		return (Vector)(htemplates.get((Pattern)vpat.firstElement()));
	    else{
		// Sinon, on trie les regles et prend la premiere
		qs.sort(vpat);
		// On compare neanmoins les deux premieres regles afin de voir si
		// Elles ne sont pas egales. (Si c'est le cas, on renvoie une erreur)
		Pattern elem1 = (Pattern)vpat.firstElement();
		Pattern elem2 = (Pattern)vpat.elementAt(1);
		if (elem1.compareTo(elem2) == Pattern.EQUAL){
		    err.fatalError("2 Templates matches one elements. " + n.getTagName() +
				   "\tFirst  Template is:" + elem1.toString() + 
			           "\tSecond Template is:" + elem2.toString());
		    return null;
		}else{
		    return (Vector)htemplates.get(elem1);
		}
	    }
	}
    }
    // Contant
    private static final String BUG_FIX = "\n Report bug at: Jeremy.Calles@sophia.inria.fr";
}
