package fr.dyade.koala.xml.xsl.slideMaker;
import fr.dyade.koala.xml.xsl.util.*;
import java.io.*;
import java.util.*;
import java.text.*;
/** HtmlTransform.java
 */

/** Post-processor: When the XSL Processor have transform XML slides to One HTML File, this class 
 *  do:<br>
 *  - Firstly, it split the HTML File in x. files. (One by slide)<br>
 *  - And then, it replaces tags such as &lt;Date/>,&lt;Date format="..."/>,&lt;noPage/>, &lt;Cursor/>, &lt;newPage/>,
 *  <goToPage title="..."/> that cannot be replaced by the XSL Processor.
 *  <h4>Recognized Tags:</h4><ul>
 *  <li><i>&lt;Date/></i> adds the current date in standard format. <code>"MMMMMMM yyyy"</code>
 *  <li><i>&lt;Date format ="..."/></i> adds the current date in the specified format between "".<br>
 *  <dl><dd>The date format <i>must be conforms</i>to the java.text.SimpleDateFormat API</dl>
 *  <li><i>&lt;newPage/></i> makes a new file
 *  <li><i>&lt;Cursor/></i> making a Cursor in order to go to Next and Previous page
 *  <li><i>&lt;noPage/></i> adds the page number
 *  <li><i>&lt;goToTitle .../></i> makes a Title 
 *  </ul>
 *  @author Jrmy CALLES
 *  @version 0.5b
 *  @see fr.dyade.koala.xml.xsl.XSLEval
 *  @see java.text.SimpleDateFormat
 */
public class HtmlTransform{
    /** The HTML Input file (made by the XSL Processor) */
    protected File fil;
    private FileReader fis;
    private BufferedReader dis;
    /** Initializer.
     *  @param fil the name of the HTML input file
     */
    public HtmlTransform(String fil,String slidname){
	this.fil = new File(fil);
	NumPage = -1;
	goToPage = 1; // La Toc est en 1, donc les pages suivantes commencent a 2
	goToToc = 0; 
	MaxPage = 0;
	file = new String();
	// The name of the output files
	if (slidname != null)
	    SLIDENAME = slidname;
	try{
	    fis = new FileReader(this.fil);  
	} catch (IOException e){
	    e.printStackTrace();
	    System.exit(1);
	} 
	// the Date Format
	SimpleTimeZone pdt = new SimpleTimeZone(-8 * 60 * 60 * 1000, "PST");
	pdt.setStartRule(Calendar.APRIL, 1, Calendar.SUNDAY, 2*60*60*1000);
	pdt.setEndRule(Calendar.OCTOBER, -1, Calendar.SUNDAY, 2*60*60*1000);
	formatter
	    = new SimpleDateFormat (dateFormat);
	// Parsing File
	transform();
    }
    /** Split the file, and replace defined tags */
    protected void transform(){
	String line = "";
	MaxPage = countPages() - 1;
	dis = new BufferedReader(fis);
	try{
	    while(line != null){
		line = parseLine(line);
		file += line + "\n";
		line = dis.readLine();
	    }
	}catch(IOException e){
	    e.printStackTrace();
	}
	// Save Last Page
	if (slideNumbers.isEmpty()){	
	    Files.StringToFile(SLIDENAME + NumPage + ".html",file);
	}else{
	    //System.err.println("slideNumbers.elementAt(NumPage)" +slideNumbers.elementAt(NumPage) );
	    Files.StringToFile(SLIDENAME+ slideNumbers.elementAt(NumPage) + ".html",file);
	}
    }

    /** parse a Line */
    public String parseLine(String line_tmp){
	String line = line_tmp;
	int i = line.indexOf(NEWPAGE);
	if (i != -1){
	    // &lt;newPage/>, Making a new file
	    String gauche = line.substring(0,i);
	    String droite = line.substring(i+NEWPAGE.length());
	    if (NumPage != -1){
		Files.StringToFile(SLIDENAME + NumPage + ".html",file + gauche);
	    }
	    file = new String();
	    NumPage++;
	    line = droite;

	}else if ((i = line.indexOf(NEWPAGEwTOC)) != -1){
	    // &lt;newPage toc=".." no=".."/>, Making a new file
	    int e_nam = line.indexOf(NEWPAGEwNO_END);
	    String gauche = line.substring(0,i);
	    //	    String name = line.substring(i+NEWPAGEwNO.length(),e_nam);//
	    String droite = line.substring(e_nam+NEWPAGEwNO_END.length());
	    
	    if (NumPage != -1){
		Files.StringToFile(SLIDENAME + slideNumbers.elementAt(NumPage) + ".html",file + gauche);
	    }
	    
	    file = new String();
	    NumPage++;
	    goToPage = NumPage;
	    //	    System.err.println("********" + gauche);
	    line = droite;
	}else if((i = line.indexOf(CURSOR)) != -1){
	    // &lt;/Cursor> making a Cursor in order to go to Next and Previous page
	    String gauche = line.substring(0,i);
	    String droite = line.substring(i+CURSOR.length(),line.length());
	    line = gauche;
	    if (NumPage > 0)
		if (slideNumbers.isEmpty()){
		    line += "\n" + TEXT_COMMON + SLIDENAME + (NumPage - 1) + ".html" + TEXT_TOPREVIOUS;
		}else{
		    line += "\n" + TEXT_COMMON + SLIDENAME  + slideNumbers.elementAt(NumPage - 1) + ".html" + TEXT_TOPREVIOUS;
		}
	    else{
		if (slideNumbers.isEmpty()){
		    line += "\n" + TEXT_COMMON + SLIDENAME +  0 + ".html" + TEXT_TOPREVIOUS;
		}else{
		    line += "\n" + TEXT_COMMON + SLIDENAME +  slideNumbers.elementAt(0) + ".html" + TEXT_TOPREVIOUS;
		}
	    }
	    if (slideNumbers.isEmpty()){
		line += "\n" + TEXT_COMMON + SLIDENAME + 0 + ".html" + TEXT_TOTOC;
	    }else{
		String toc = (String)slideNumbers.elementAt(NumPage);
		// Si on est sur une sous-table des matieres, on pointe sur la Toc Generale
		if (toc.endsWith(".1")){
		    toc = ".0.1";
		}else{		
		    toc = toc.substring(0,toc.lastIndexOf(".")) + ".1" ;
		}		
		line += TEXT_COMMON + SLIDENAME + toc + ".html" + TEXT_TOTOC ;
	    }
	    if (NumPage != MaxPage)
		if (slideNumbers.isEmpty()){
		    line += TEXT_COMMON + SLIDENAME +  (NumPage + 1) + ".html" + TEXT_TONEXT ;
		}else{
		    line += TEXT_COMMON + SLIDENAME + slideNumbers.elementAt(NumPage + 1) + ".html" + TEXT_TONEXT ;
		}
	    else{	
		if (slideNumbers.isEmpty()){
		    line += TEXT_COMMON + SLIDENAME +  0 + ".html" + TEXT_TONEXT ;
		}else{
		    line += TEXT_COMMON + SLIDENAME +  slideNumbers.elementAt(0) + ".html" + TEXT_TONEXT ;
		}
	    }
	    line += droite;		
	}else if((i = line.indexOf(NOPAGE)) != -1){
	    // &lt;noPage/> adds the page number
	    String gauche = line.substring(0,i);
	    String droite = line.substring(i+NOPAGE.length());
	    
	    line = gauche+ " " + NumPage + "/" + MaxPage + " " +droite;		    
	}else if((i = line.indexOf(TITLE)) != -1){
	    // &lt;goToTitle .../> makes a Title 
	    int indexTitleEnd = line.indexOf(TITLE_END);
	    String gauche = line.substring(0,i);
	    String title = line.substring(i+TITLE.length(),indexTitleEnd);
	    String droite = line.substring(indexTitleEnd+TITLE_END.length());
	    line = gauche;
	    /*	    if ((GoToPage == -1)|| (GoToPage > MaxPage)){
		    line += title;
		    }else if(GoToPage == 0){ // Without the toc in the list of pages
		    line += "";
		    }else{*/
	    goToPage++;
	    if (slideNumbers.isEmpty()){
		line += TEXT_COMMON + SLIDENAME + (goToPage-2) + ".html\">" + title + "</a>";
	    }else{
		line += TEXT_COMMON + SLIDENAME+ slideNumbers.elementAt(goToPage) + ".html\">" + title + "</a>";
	    }
		/*	    }		    */
	    //	    GoToPage++;
	    line += droite;
	    //	    System.err.println(line);
	}else if((i = line.indexOf(TOC)) != -1){
	    //System.err.println("goTo: " + line);
	    // &lt;goToToc .../> makes a Title 
	    int indexTitleEnd = line.indexOf(TITLE_END);
	    String gauche = line.substring(0,i);
	    String title = line.substring(i+TOC.length(),indexTitleEnd);
	    String droite = line.substring(indexTitleEnd+TITLE_END.length());
	    line = gauche;
	    goToToc++;
	    if (slideNumbers.isEmpty()){
		System.err.println("Error: the use of <goToToc/> is forbidden if you don't use <newPage toc=\"..\" name=\"..\"/> tag.");
		System.exit(1);
	    }else{
		line += TEXT_COMMON + SLIDENAME + "." + goToToc + ".1.html\">" + title + "</a>";
	    }
		/*	    }		    */
	    //	    GoToPage++;
	    line += droite;
	    //	    System.err.println(line);
	}else if((i = line.indexOf(DATE)) != -1){
	    // &lt;Date/>, adds the current date in standard format. "MMMMMMM yyyy"
	    String gauche = line.substring(0,i);
	    String droite = line.substring(i+DATE.length());
	    formatter
		= new SimpleDateFormat (dateFormat,pays);
	    String dateString = formatter.format(new Date());
	    line = gauche+dateString+droite;
	}else if((i = line.indexOf(TOTOC)) != -1){
	    String gauche = line.substring(0,i);
	    String droite = line.substring(i+TOTOC.length());
	    String tmp;
	    if (slideNumbers.isEmpty()){
		tmp = "0";
	    }else{
		tmp = (String)slideNumbers.elementAt(NumPage);
		// Si on est sur une sous-table des matieres, on pointe sur la Toc Generale
		if (tmp.endsWith(".1")){
		    tmp = ".0.1";
		}else{		
		    tmp = tmp.substring(0,tmp.lastIndexOf(".")) + ".1" ;
		}		
	    }
	    line = gauche + tmp + droite;
	}else if((i = line.indexOf(PREVIOUS)) != -1){
	    String gauche = line.substring(0,i);
	    String droite = line.substring(i+PREVIOUS.length());
	    String tmp;
	    if (slideNumbers.isEmpty()){
		if (NumPage == 0){
		    tmp = "0";
		}else{
		    tmp = Integer.toString(NumPage - 1);
		}
	    }else{
		tmp = (String)slideNumbers.elementAt(NumPage);
		// Si on est sur une sous-table des matieres, on pointe sur la Toc Generale
		if (tmp.endsWith(".1")){
		    tmp = ".0.1";
		}else{		
		    tmp = tmp.substring(0,tmp.lastIndexOf(".")) + ".1" ;
		}		
	    }
	    line = gauche + tmp + droite;
	}else if((i = line.indexOf(NEXT)) != -1){
	    String gauche = line.substring(0,i);
	    String droite = line.substring(i+NEXT.length());
	    String tmp;
	    if (slideNumbers.isEmpty()){
		if (NumPage == MaxPage){
		    tmp = "0";
		}else{
		    tmp = Integer.toString(NumPage + 1);
		}
	    }else{
		tmp = (String)slideNumbers.elementAt(NumPage);
		// Si on est sur une sous-table des matieres, on pointe sur la Toc Generale
		if (tmp.endsWith(".1")){
		    tmp = ".0.1";
		}else{		
		    tmp = tmp.substring(0,tmp.lastIndexOf(".")) + ".1" ;
		}		
	    }
	    line = gauche + tmp + droite;
	}else if((i = line.indexOf(DATEF)) != -1){
	    // &lt;Date format ="..."/>, adds the current date in the specified format between ""
	    // The format must be conforms to the java.text.SimpleDateFormat API
	    String gauche = line.substring(0,i);
	    int end = line.indexOf(TITLE_END);	  
	    String dateFormat = line.substring(i+DATEF.length(),end);
	    //	    System.err.println("date: " + dateFormat + ": " + i + " " + end + pays);
	    formatter
		= new SimpleDateFormat (dateFormat,pays);
	    String droite = line.substring(end+TITLE_END.length());
	    
	    String dateString = formatter.format(new Date());
	    line = gauche+dateString+droite;
	}else if (line.indexOf(LANGUEFR) != -1){
	    // Fixes the Locale variable
	    pays = Locale.FRENCH;
	    formatter
		= new SimpleDateFormat (dateFormat,pays);
	}else if (line.indexOf(LANGUEEN) != -1){
	    // Fixes the Locale variable
	    pays = Locale.ENGLISH;
	    formatter
		= new SimpleDateFormat (dateFormat,pays);
	}
	if (line.equals(line_tmp)){
	    return line;
	}else{
	    return parseLine(line);
	}
    }
    /** Counts the number of Slides in the HTML Document*/
    public int countPages(){ 
	BufferedReader in = null;
	int pages = 0;
	int previous_no = 0;
	slideNumbers = new Vector();
	try{
	    FileReader fi = new FileReader(fil);  
	    in = new BufferedReader(fi);
	} catch (IOException e){
	    e.printStackTrace();
	    System.exit(1);
	}

	try{
	    String line = "";
	    while(line != null){
		int i = line.indexOf(NEWPAGE);		
		if (i != -1){
		    pages++;
		}else if ( (i = line.indexOf(NEWPAGEwTOC)) != -1){	    
		    pages++;
		    int toc_nam = line.indexOf(i);
		    int e_nam = line.indexOf(NEWPAGEwNO);
		    int end_nam = line.indexOf(NEWPAGEwNO_END);
		    int toc = Integer.valueOf(line.substring(i+NEWPAGEwTOC.length(),e_nam)).intValue();
		    int no = Integer.valueOf(line.substring(e_nam+NEWPAGEwNO.length(),end_nam)).intValue();
		    // Pour compenser les problemes de NewPage avec Xsl:number
		    if (toc >= 1){
			toc--;
		    }
		    if (no <= previous_no){
			toc++;
		    }
		    slideNumbers.addElement("." + toc + "." + no);
		    previous_no = no;
		}
		line = in.readLine();
	    }
	}catch(IOException e){
	    e.printStackTrace();
	}
	//	System.err.println("slideNumbers: " + slideNumbers);
	return pages;
    }
    // Variables:
    private Vector slideNumbers;
    private String previousSlideNumber = "";    
    // Constantes
    private String SLIDENAME = "slide";
    private static final String DATEF    = "<Date format=\"";
    private static final String DATE    = "<Date/>";
    private static final String TOTOC    = "<toc/>";
    private static final String PREVIOUS    = "<previous/>";
    private static final String NEXT    = "<next/>";
    private static final String NOPAGE = "<noPage/>";
    private static final String NEWPAGE = "<newPage/>";   
    private static final String NEWPAGEwTOC = "<newPage toc=\"";
       private static final String NEWPAGEwNO = "\" no=\"";
    private static final String NEWPAGEwNO_END = "\"/>";   
    private static final String CURSOR = "<Cursor/>";   
    private static final String TITLE = "<goToPage title=\"";   
    private static final String TITLE_END = "\"/>";   
    private static final String TOC = "<goToToc title=\"";   
    private static final String LANGUEFR = "lang=\"fr";  
    private static final String LANGUEEN = "lang=\"en"; 
    private static final String TEXT_COMMON =   "<a href=\"";
    private static final String TEXT_TOPREVIOUS =  "\"><img src=\"images/left.gif\" border=\"0\"></a>";
    private static final String TEXT_TOTOC =  "\"><img src=\"images/up.gif\" border=\"0\"></a>";
    private static final String TEXT_TONEXT =  "\"><img src=\"images/right.gif\" border=\"0\"></a>";    
    private boolean fr = true;
    // Counter
    private int NumPage;
    private int goToPage;
    private int goToToc;
    private int MaxPage;
    private String file;
    // Date Format
    private Locale pays = Locale.ENGLISH;
    private SimpleTimeZone pdt;	
    private SimpleDateFormat formatter; 
    private String dateFormat = "MMMMMMMMM yy";
}
