package fr.dyade.koala.xml.xsl.util;
/** DOMParse.java 
 */
/** This class parse an XML and return a DOM Document
 *
 *  @see fr.dyade.koala.xml.xsl.XSLTags
 *  @see fr.dyade.koala.xml.xsl.Pattern
 *  @version 0.7b
 *  @author  Jeremy CALLES
 */
import fr.dyade.koala.xml.xsl.*;
import fr.dyade.koala.xml.xsl.exception.*;
import com.docuverse.freedom.*;

import org.w3c.dom.*;
import org.xml.sax.*;
import java.net.*;
import java.io.*;

public class DOMParse implements DocumentHandler{
   /** 
     *  Initializer with an Error Handler
     *  
     */
    public DOMParse(XSLErrorHandler err){
	saxError = err;
    }
    /** The SAX Parser */
    public Parser parser;
    /** The DOM Document */
    public Document document;
    /** The error Handler */
    public XSLErrorHandler saxError;
    /** The DOM Context */
    DOMContext context;
    // Create instance of the SAX Parser and the DOM Implementation to use 
    private void createInstance(){
	try{
	    Class saxClass;
	    saxClass = Class.forName(saxDriver);
	    Class domClass;
	    domClass = Class.forName(domDriver);
	    
	    Object saxInstance = saxClass.newInstance();
	    Object domInstance = domClass.newInstance();
	    
	    if (!(saxInstance instanceof org.xml.sax.Parser))
		throw new IllegalArgumentException("SAX driver named " + saxDriver +
						   " does not support SAX Parser interface");
	    
	    parser = (Parser)saxInstance;
	    
	    if (!(domInstance instanceof org.w3c.dom.Document))
		throw new IllegalArgumentException("DOM driver named " + domDriver +
						   " does not support DOM interface");
	    
	    document = (Document)domInstance;
	}catch(Exception e){
	    e.printStackTrace();
	}
	
    }     
    /** Parsing URL */
    public Document parseFile(Object uri) throws Exception{
	createInstance();
    Freedom dom = new Freedom("com.docuverse.freedom.SAXDriver");
    dom.setProperty("sax.driver","com.microstar.xml.SAXDriver");

    Document doc = dom.openDocument(uri);
    //ImRDocument.makeAbsoluteURL(fileName));
    return doc;


    /*
	return parser.openDocument(uri);   
	return dom.openDocument(uri);   
	if ((parser == null) || (document == null)){
	    // Logiquement doit planter avant
	    return null;
	}else{
	    context = new DOMContext(document);
	    parser.setDocumentHandler(this);
	    parser.setErrorHandler(saxError);
	    InputSource input = createInputSource(uri);
	    parser.parse(input);
	    return context.getDocument();
	}*/

    }
    /** Create a Sax Input Source from an Object */
    protected InputSource createInputSource (Object source){       
	if (source instanceof InputSource){
	    return (InputSource)source;
	}else if (source instanceof InputStream)
	    return new InputSource((InputStream)source);
	else if (source instanceof URL){
		return new InputSource(((URL)source).toString());//openStream());
	}else if (source instanceof Reader){
	    return new InputSource((Reader)source);
	}else if (source instanceof String){
	    return new InputSource((Reader)new StringReader((String)source));	    
	}else if (source != null)
	    return new InputSource(source.toString());
	else
	    return null;
    }

    /** SAX & DOM driver */
    private static String saxDriver;
    /** SAX & DOM driver */
    private static String domDriver;
    static {
	if ((saxDriver = System.getProperty("org.xml.sax.parser")) == null){
	    //saxDriver = "fr.dyade.koala.xml.sax.SAXParser"; TIMUCIN
	    saxDriver = "com.microstar.xml.SAXDriver";
	}
	if ((domDriver = System.getProperty("org.w3c.domImplementation")) == null){
	    //domDriver = "fr.dyade.koala.xml.xsl.util.DOMSDKDocument"; TIMUCIN
	    domDriver = "com.docuverse.freedom.FreedomDocument";
	}
    }
    /** Create an URL from a String */
    public URL createURL(String uri) throws IOException {
	URL url = null;
	try {
	    url = new URL(uri);
	} catch (MalformedURLException ex) {
	    File f = new File(uri);
	    url = new URL("file", null, -1, f.getCanonicalPath());
	}
	return url;
    }
    
    /* Sax Handlers */
    public void setDocumentLocator (Locator locator){
    }
    /* Sax Handlers */
    public void startDocument () throws SAXException{	
    }
    /* Sax Handlers */
    public void endDocument () throws SAXException{
    }
    /* Sax Handlers */
    public void startElement (String name, org.xml.sax.AttributeList atts) throws SAXException{
	Element elem = document.createElement(name);
	if (atts != null)
	    {
		int count = atts.getLength();
		for (int i = 0; i < count; i++)
		    elem.setAttribute(atts.getName(i), atts.getValue(i));
	    }	
	context.add(elem);
	context.enter(elem);
    }
    /* Sax Handlers */
    public void endElement (String name) throws SAXException{
	context.leave();
    }
    /* Sax Handlers */
    public void characters (char ch[], int start, int length) throws SAXException{
	Text text = document.createTextNode(new String(ch, start, length));
	context.add(text);
    }
    /* Sax Handlers */
    public void ignorableWhitespace (char ch[], int start, int length) throws SAXException{
	Text text = document.createTextNode(new String(ch, start, length));
	context.add(text);
    }
    /* Sax Handlers */
    public void processingInstruction (String name, String remainder)throws SAXException{
	ProcessingInstruction PI = document.createProcessingInstruction(name, remainder);
	context.add(PI);
    }
}
