package test.xml4j;

import java.net.*;
import java.io.*;

import org.w3c.dom.*;
import com.ibm.xml.parser.*;

import iwt.web.xmldomxsl.*;

/**
*
* Uses our DOM implementation to build the DOM representation of
* a document.
*
* org.xml.sax.driver
* @see edu.npac.iwt.DOMFactory 
*
* @author H. Timucin Ozdemir
* @date   March 3, 1999 (Updated March 3, 1999)
*
*/
public class XMLJDOMFactory implements DOMFactory 
{
   String xmlParser    = "com.microstar.xml.SAXDriver";

   /**
   *
   * It only allows to set the xml parser.
   * org.xml.sax.driver property name.
   *
   * @param pName   the name of property.
   * @param pValue  the value of property.
   */
   public void 
   setProperty(String pName, String pValue)
   {
     if( pName.equals("org.xml.sax.driver") )
     {
        try
        {
	    Class parserClass = Class.forName(pValue);;
	    	    
	    Object parser     = parserClass.newInstance();
    
	    if ( parser instanceof org.xml.sax.Parser ) xmlParser = pValue;

        } catch(Exception e) {}

     }// end of if

   }// end of setProperty()

   /**
   * 
   * @return The Document object of the given XML document.
   */
   public org.w3c.dom.Document 
   openDocument(String xmlFile)
   { 
     Document doc = null;
     if( xmlFile != null) 
     {
         InputStream is;
         try 
         { 
           is = new FileInputStream(xmlFile); 
           //*** The doc is the root of the DOM Tree. It is of type
           //*** TXDocument which implements the DOM Document interface. 
           Parser parse = new Parser(xmlFile,new ErrorFlagger(),null);
           doc = parse.readStream();

         } catch (FileNotFoundException notFound) 
           { System.err.println(notFound); }

     }// end of if

     return doc;

   }// end of openDocument()

   /**
   * 
   * @return The Document object of the given XML document.
   */
   public org.w3c.dom.Document 
   openDocument(URL docurl)
   { 
     Document doc = null;
     try
     { 
        doc = (new Parser(docurl.toString())).readStream(docurl.openConnection().getInputStream(), new ErrorFlagger(),null);
     } catch(Exception e) {}

     return doc;

   }// end of openDocument()

   /**
   * 
   * @return The Document object of the given XML document.
   */
   public org.w3c.dom.Document 
   openDocument(InputStream docIn, String fName)
   { 
     return (new Parser(fName)).readStream(docIn, new ErrorFlagger(),null);

   }// end of openDocument()

   /** Create an URL from a String */
   public URL 
   createURL(String uri) throws IOException 
   {
	URL url = null;
	try {
	    url = new URL(uri);
	} catch (MalformedURLException ex) {
	    File f = new File(uri);
	    //TIMUCIN url = new URL("file", null, -1, f.getCanonicalPath());
          try { url = new URL(makeAbsoluteURL(uri)); } catch (MalformedURLException ex1) {} 
	}
	return url;
   }

   /**
   */
   public static String 
   makeAbsoluteURL (String url) throws MalformedURLException
   {
      URL baseURL;
      String currentDirectory = System.getProperty("user.dir");
      String fileSep = System.getProperty("file.separator");
      String file = currentDirectory.replace(fileSep.charAt(0), '/') + '/';
      if (file.charAt(0) != '/') { file = "/" + file; }
      baseURL = new URL("file", null, file);
      return new URL(baseURL,url).toString();

   }// end of makeAbsoluteURL()

}// end of IWTDOMFactory

class ErrorFlagger implements ErrorListener
{
   public int 
   error(String fname, int lineno, int charoff, Object key, String mes)
   {
      
     System.out.println("Error: "+fname+" "+lineno+" "+mes);

     return 0;

   }// end of error()
}
